
package com.akwabasystems.filters;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.zip.GZIPOutputStream;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * A filter that compresses output with the "gzip" method if the browser supports it. This allows faster page
 * downloads and a smoother user experience
 * 
 */
public class CompressionFilter implements Filter {
    
    
    private FilterConfig config;
    private ServletContext context;
    
    
    /**
     * If the browser doesn't support gzip, invoke the service normally. 
     * Otherwise, set the Content-Encoding response header and invoke the service
     * with a wrapped response that collects all the output. Extract the output
     * and write it into gzipped byte array. Finally, write that array to the 
     * client's output stream.
     */
    @Override
    public void doFilter(ServletRequest request,
                        ServletResponse response,
                        FilterChain chain)
                        throws ServletException, IOException {
    
        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse resp = (HttpServletResponse) response;
        
        if(!isGzipSupported(req)) {
            chain.doFilter(req, resp);
        
        } else {
            
            //Set the content encoding to gzip
            resp.setHeader("Content-Encoding", "gzip");
            
            //Invoke resource, accummulating output in the wrapper
            StringWrapper responseWrapper = new StringWrapper(resp);
            chain.doFilter(req, responseWrapper);
            
            //Make a writer that compresses data and puts it into a byte array
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            GZIPOutputStream zipOut = new GZIPOutputStream(byteStream);
            OutputStreamWriter tempOut = new OutputStreamWriter(zipOut);
            
            //Compress the original output and put it into byte array
            tempOut.write(responseWrapper.toString());
            
            //GZIP streams must be explicitly closed
            tempOut.close();
            
            //Send compressed output to the client
            OutputStream realOut = resp.getOutputStream();
            byteStream.writeTo(realOut);            
        }
    }
    
    
    /**
     * Store the FilterConfig object for reference by subclasses
     */
    public void init(FilterConfig config) throws ServletException {
        this.config = config;
        context = config.getServletContext();
    }
    

    /**
     * Stores the {@code FilterConfig} object in case subclasses need it
     *
     * @return the {@code FilterConfig} object in case subclasses need it
     */
    protected FilterConfig getFilterConfig() {
        return config;
    }
    
    
    /**
     * Invoked when the context is destroyed. No specific action needs to be taken here
     */
    @Override
    public void destroy() {

    }
    
    
    /**
     * Check whether the client browser supports GZIP compression
     */
    private boolean isGzipSupported(HttpServletRequest request) {
        
        String encodings = request.getHeader("Accept-Encoding");
        return (encodings != null && encodings.indexOf("gzip") > -1); 
        
    }   
}
