
package com.akwabasystems.listeners;


import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import java.util.UUID;
import java.util.logging.Logger;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;



/**
 * A listener that initializes the application configuration when the Web application starts. It sets
 * application-wide properties such as the name of the company, the current build and build time and other
 * such information. Various servlets and JSP pages will later use this attribute.
 *
 * This listener also registers all the service providers used by the application Service Framework. Those
 * providers initialize their respective services
 */
public class WebApplicationListener implements ServletContextListener {

    private static String webRoot;
    private static final String DIGEST = UUID.randomUUID().toString();
    private static final String PROPERTIES_FILE = "application.properties";
    private static Properties properties;
    private static final Logger logger = Logger.getLogger(WebApplicationListener.class.getName());
    

    /**
     * This method initializes the listener: sets the properties of the application and notifies the
     * {@link ApplicationDelegate} for further initialization of the application components
     * 
     * @param event         A ServletContextEvent fired by the application framework when it's initialized
     */
    @Override
    public void contextInitialized(ServletContextEvent event) {
        setApplicationProperties(event);
    }


    /**
     * Sets the application properties used by the different JSP and servlets: context path, build time,
     * session keys, etc
     *
     * @param event         the {@code ServletContextEvent} event fired by the application
     */
    private void setApplicationProperties(ServletContextEvent event) {
        ServletContext context = event.getServletContext();
        
        webRoot = context.getContextPath();
        context.setAttribute("webRoot", webRoot);
        context.setAttribute("buildTime", new SimpleDateFormat("yyyyMMdd").format(new Date()));
        
        logger.info(String.format("%s: Context initialized, the store is now open...",
            DateFormat.getDateTimeInstance().format(new Date())));
    }

    
    /**
     * Returns the servlet context (Web root) of the application
     *
     * @return the servlet context (Web root) of the application
     */
    public static String getWebRoot() {
        return webRoot;
    }

    
    /**
     * Returns the daily digest string used to generate login signatures
     *
     * @return the daily digest string used to generate login signatures
     */
    public static String getDailyDigest() {
        return DIGEST;
    }


    /**
     * Servlet context is destroyed: No special handling needed here.
     * @param event             the {@code ServletContextEvent} event fired by the application
     */
    @Override
    public void contextDestroyed(ServletContextEvent event) {
    }

}
