
package com.akwabasystems.web;

import org.springframework.stereotype.Controller;
import com.akwabasystems.exceptions.UIException;
import java.io.IOException;
import java.util.Locale;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;


/**
 * A controller that handles all the requests from the UI and routes them according to their URI
 * patterns. It processes them and returns a logical view name that's translated to the actual JSP
 * page in the {@literal /WEB-INF/jsp} folder
 *
 * This controller uses method-based annotations to map the specific handlers to the request. If no
 * suitable handler is found, or if the request cannot be understood, it redirects the user to the
 * main exception page, with details on the exception
 *
 * This implementation is marked final and shouldn't be subclassed
 */
@Controller
@RequestMapping("/calendar")
public final class CalendarController {

    private static final Logger LOG = Logger.getLogger(CalendarController.class.getName());
    

    /**
     * URL mapping: "/home".
     * This method processes all requests to the {@literal /home} URI pattern and displays the
     * application login page
     *
     * @param request           An HTTPServletRequest object
     * @param response          An HTTPServletResponse object
     * @param model             A Spring ModelMap object
     * @param locale            The locale for the current user
     *
     * @return a ModelAndView containing the model to make available to the login page
     * @throws ServletException  a ServletException if the request cannot be processed
     * @throws IOException an IOException if content cannot be output
     */
    @RequestMapping(value = "/home", method = RequestMethod.GET)
    public ModelAndView loginPage(HttpServletRequest request,
                             HttpServletResponse response,
                             ModelMap model,
                             Locale locale)
        throws ServletException, IOException {
        
        return new ModelAndView("calendar/index", model);

    }

    
    /**
     * Handles an exception if no matching URI pattern is found for a request. It redirects the user
     * to the error page with the specific error message
     *
     * @param request               the request object to handle
     * @param response              the response to output to the client
     * @param model                 the model map containing the page parameters
     * @return a ModelAndView with the view set to the error message page and the model containing the
     *      error parameters
     * @throws ServletException  a ServletException if the request cannot be processed
     * @throws IOException an IOException if content cannot be output
     */
    private ModelAndView handleException(HttpServletRequest request,
                                         HttpServletResponse response,
                                         ModelMap model)
        throws ServletException, IOException {

        model.addAttribute("exception", getUIExceptionDetails());
        return new ModelAndView("/errors/ui-exception", model);

    }


    /**
     * Returns the detailed description of an exception, including its possible causes
     *
     * @return the detailed description of an exception, including its possible causes
     */
    private UIException getUIExceptionDetails() {
        String message = "Oops! Couldn't find the page you requested...";
        String description = new StringBuffer("<p>The page URL you requested couldn't be found. </p>" +
            "<p>Please check the URL of the request and try again...</p>"
        ).toString();

        return new UIException(message, description);
    }


    /**
     * Returns a string representation of this controller class by displaying its URI pattern
     * 
     * @return a string representation of this controller class
     */
    @Override
    public String toString() {
       return "Calendar Web Controller";
    }
    
}
