/**---------------------------------------------------------------------------------------------------------------------
                                        CONCIERGE MEDIA

     A Media Player framework that provides a unified interface for creating different types of Media elements (HTML5
video, Quicktime video, Flash video) and for interacting with those elements through a single Control Bar element. 
The default control bar uses the iTunes theme, but the implementation also provides other themes such as Hulu, 
custom minimalist theme or other cool ones.

This implementation provides a factory method that can be invoked directly to create the type of Media player if known 
in advance. It also provides the ability to automatically detect whether a media-related feature is supported in the 
given environment and to create the appropriate Media player accordingly.

This implementation supports most browsers (Safari, Firefox, Chrome, Opera, IE) as well as iOS devices (iPad or other 
iOS devices). When running in those environments, the default control bar is unsed instead, and Airplay is enabled

Author: Daniel Ziaoure
(c) 2013 - Akwaba Systems, Inc.
----------------------------------------------------------------------------------------------------------------------*/

if(!Concierge) {
    throw "The Concierge.Media module requires the Concierge core module";
}

Concierge.createNamespace("Concierge.Media");

Concierge.Media.Detector = (function() {

    "use strict";
    
    var _isWebKit, _isSafari2, _isChrome, _isQTInstalled,
    _isFlashInstalled;
    
    
    /**
     * Returns the user agent for the current browser
     * 
     * @return the user agent for a current browser
     */
    function getAgent() {
        return navigator.userAgent.toLowerCase();
    }


    /**
     * Returns true if the specified user agent is running on a Mac; otherwise, returns false
     * 
     * @param userAgent         the user agent string to test
     * @return true if the specified user agent is running on a Mac; otherwise, returns false
     */
    function isMac(userAgent){
        var agent = (userAgent || getAgent());
        return !!agent.match(/mac/i);
    }


    /**
     * Returns true if the specified user agent is running on MacOS Snow Leopard; otherwise, returns false
     * 
     * @param userAgent         the user agent string to test
     * @return true if the specified user agent is running on MacOS Snow Leopard; otherwise, returns false
     */
	function isSnowLeopard(userAgent) {
		var agent = (userAgent || getAgent());
		return !!agent.match(/mac os x 10_6/i);
	}


    /**
     * Returns true if the specified user agent is running on Windows; otherwise, returns false
     * 
     * @param userAgent         the user agent string to test
     * @return true if the specified user agent is running Windows; otherwise, returns false
     */
    function isWin(userAgent) {
        var agent = (userAgent || getAgent());
        return !!agent.match(/win/i);
    }


    /**
     * Returns true if the specified user agent is running on Windows 2000; otherwise, returns false
     * 
     * @param userAgent         the user agent string to test
     * @return true if the specified user agent is running on Windows 2000; otherwise, returns false
     */
    function isWin2k(userAgent) {
        var agent = (userAgent || getAgent());
        return !!(isWin(agent) && (agent.match(/nt\s*5/i)));
    }

    
    /**
     * Returns true if the specified user agent is running on Windows Vista; otherwise, returns false
     * 
     * @param userAgent         the user agent string to test
     * @return true if the specified user agent is running on Windows Vista; otherwise, returns false
     */
    function isWinVista(userAgent) {
        var agent = (userAgent || getAgent());
        return !!(isWin(agent) && (agent.match(/nt\s*6\.0([0-9]{0,2})?/i)));
    }


    /**
     * Returns true if the browser with the specified user agent is running the WebKit engine; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is running the WebKit engine; otherwise, returns false
     */
    function isWebKit(userAgent) {
        if(_isWebKit === undefined) {
            var agent = (userAgent || getAgent());
            _isWebKit =  !!agent.match(/AppleWebKit/i);
            /*isWebKit = function() {
                return this._isWebKit;
            };*/
        }
        
        return _isWebKit;
    }


    /**
     * Returns true if the browser with the specified user agent is Safari 2 otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is Safari 2 otherwise, returns false
     */
    function isSafari2(userAgent) {
        if(_isSafari2 === undefined) {
            if (!isWebKit(userAgent)) {
                _isSafari2 = false;
            } else {
                var ua = navigator.userAgent.toLowerCase(),
                version = parseInt(parseFloat(ua.substring(ua.lastIndexOf('safari/') + 7)), 10);
                _isSafari2 = (version >= 419);
            }
        }
        
        return _isSafari2;
    }

    
    /**
     * Returns true if the browser with the specified user agent is Chrome; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the specified user agent is from Chrome; otherwise, returns false
     */
	function isChrome(userAgent) {
		if(_isChrome === undefined) {
			var agent = (userAgent || getAgent());
			_isChrome = !!agent.match(/Chrome/i);
		}
        
		return _isChrome;
	}
	
    
    /**
     * Returns true if the browser with the specified user agent is Opera; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is Opera; otherwise, returns false
     */
    function isOpera(userAgent){
        var agent = (userAgent || getAgent());
        return !!agent.match(/opera/i);
    }

    
    /**
     * Returns true if the browser with the specified user agent is IE; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is IE; otherwise, returns false
     */
    function isIE(userAgent) {
        var agent = (userAgent || getAgent());
        return !!agent.match(/msie/i);
    }


    /**
     * Returns true if the browser with the specified user agent is IE in strict mode; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is IE in strict mode; otherwise, returns false
     */
    function isIEStrict(userAgent) {
        var agent = (userAgent || getAgent());
        return (agent.match(/msie/i) && !isOpera(agent));
    }

	
    /**
     * Returns true if the browser with the specified user agent is IE8; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is IE8; otherwise, returns false
     */
	function isIE8(userAgent) {
		var agent = (userAgent || getAgent()),
        match = agent.match(/msie\D*([\.\d]*)/i);
     	
        if (match && match[1]) {
            version = match[1];
      	}

		return (+version >= 8);
	}


    /**
     * Returns true if the browser with the specified user agent is Firefox; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is Firefox; otherwise, returns false
     */
    function isFirefox(userAgent) {
        var agent = (userAgent || getAgent());
        return !!agent.match(/firefox/i);
    }


	/**
     * Returns the list of version numbers for the specified user agent
     * 
     * @param userAgent            the user agent for which to return the version numbers
     */
	function getIPhoneOSVersion(userAgent) {
		var agent = (userAgent || getAgent()),
        isMobile = isMobile(agent),
        OSString, OSStringParts, version;
		
        if(isMobile) {
			OSString = agent.match(/.*CPU ([\w|\s]+) like/i);
			
            if(OSString && OSString[1]) {
				OSStringParts = OSString[1].split(" ");
				version = OSStringParts[2].split("_");
				return version;
			} else {
				//iPhone running  : Mozilla/5.0 (iPod; U; CPU iPhone OS 2_2_1 like Mac OS X; pl-pl) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5H11 Safari/525.20
				//iPod touch running iPhone OS 1.1.3 user agent string: Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/4A93 Safari/419.3
				//iPhone running iPhone OS 1.0 user agent string: Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543 Safari/419.3

				return [1];
			}

		} else {
            return null;
        }
	}

    
    /**
     * Returns true if the browser with the specified user agent is running on an iPad; otherwise, returns false
     * 
     * UA: iPad running: Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) 
     * Version/4.0.4 Mobile/7B334b Safari/531.21.10
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is running on an iPad; otherwise, returns false
     */
    function isiPad(userAgent) {
        var agent = (userAgent || getAgent());
        return (!!(isWebKit(agent) && agent.match(/ipad/i)));
    }

    
    /**
     * Returns true if the browser with the specified user agent is running on a mobile device; otherwise, returns false
     * 
     * @param userAgent             the user agent string to test
     * @return true if the browser with the specified user agent is running on a moviel device; otherwise, returns false
     */
    function isMobile(userAgent) {
        var agent = (userAgent || getAgent());
        return !!(isWebKit(agent) && (agent.match(/Mobile/i) && !isiPad(agent)));
    }
    

    function isHTML5Supported() {
        var html5ElementExists = ("HTMLMediaElement" in window);
        
        if (!html5ElementExists){
            return false;
		}
        
        var video = document.createElement("video");
        return (video.canPlayType && video.canPlayType("video/mp4") !== "");
    }
    
    /**
     * Returns true if QuickTime is installed in the current browser; otherwise, returns false
     * 
     * @return true if QuickTime is installed in the current browser; otherwise, returns false
     */
    function isQTInstalled() {

		if(_isQTInstalled === undefined) {
	        var qtInstalled = false;

	        if (navigator.plugins && navigator.plugins.length) {
	            for(var i=0; i < navigator.plugins.length; i++ ) {
	                var plugin = navigator.plugins[i];

	                if (plugin.name.indexOf("QuickTime") > -1) {
	                    qtInstalled = true;
	                }
	            }
	        } else if (typeof(execScript) != 'undefined') {
	            qtObj = false; //global variable written to by vbscript for ie
	            execScript('on error resume next: qtObj = IsObject(CreateObject("QuickTimeCheckObject.QuickTimeCheck.1"))','VBScript');
	            qtInstalled = qtObj;
	        }

			_isQTInstalled = qtInstalled;
		}
        
		return _isQTInstalled;
    }


    /**
     * Returns the current QuickTime version
     * 
     * @return the current QuickTime version
     */
    function getQTVersion() {
        var version = "0";

        if (navigator.plugins && navigator.plugins.length) {
            for (var i = 0; i < navigator.plugins.length; i++) {
                var plugin = navigator.plugins[i];

                //Match: QuickTime Plugin X.Y.Z
                var match = plugin.name.match(/quicktime\D*([\.\d]*)/i);
                
                if (match && match[1]) {
                    version = match[1];
                }
            }
        } else if (typeof(execScript) != 'undefined') {
            ieQTVersion = null;
            execScript('on error resume next: ieQTVersion = CreateObject("QuickTimeCheckObject.QuickTimeCheck.1").QuickTimeVersion','VBScript');

            if(ieQTVersion){
                // ieQTVersion is comes back as '76208000' when 7.6.2 is installed.
                version = ieQTVersion.toString(16);
                version = [version.charAt(0), version.charAt(1), version.charAt(2)].join('.');
            }
        }

        return version;
    }

    
    /**
     * Returns true if a compatible version of QuickTime is installed; otherwise, returns false
     * 
     * @param required              the required version of QuickTime
     * @param actual                the currently installed version of QuickTime
     * @return true if a compatible version of QuickTime is installed; otherwise, returns false
     */
    function isQTCompatible(required, actual) {
        
        function areCompatible(required, actual) {
            var requiredValue = parseInt(required[0], 10);
            
            if (isNaN(requiredValue)) {
                requiredValue = 0;
            }

            var actualValue = parseInt(actual[0], 10);
            
            if (isNaN(actualValue)) {
                actualValue = 0;
            }

            if (requiredValue === actualValue) {
                if (required.length > 1) {
                    return areCompatible(required.slice(1), actual.slice(1));
                } else {
                    return true;
                }
            } else if (requiredValue < actualValue) {
                return true;
            } else {
                return false;
            }
        }

        var expectedVersion = required.split(/\./),
        actualVersion = (actual)? actual.split(/\./) : getQTVersion().split(/\./);

        return areCompatible(expectedVersion, actualVersion);
    }
    
    
    /**
     * Returns true if a valid version of QuickTime is available; otherwise, returns false
     * 
     * @param required              the required version of QuickTime
     * @return true if a valid version of QuickTime is available; otherwise, returns false
     */
    function isValidQTAvailable(required) {
        return (isQTInstalled() && isQTCompatible(required));
    }
    

    /**
     * Returns true if SBVDP is available; otherwise, returns false
     * 
     * @param required          the required version to check
     * @return true if SBVDP is available; otherwise, returns false
     */
	function isSBVDPAvailable(required) {
        return false;
	}


    /** 
     * Returns true if the given CSS property is available; otherwise, returns false
     * 
     * @param property          the CSS property to check
     * @return true if the given CSS property is available; otherwise, returns false;
     */
    function isCSSAvailable(property) {
        var temp = document.createElement('div').style;
        property = property.toLowerCase();
        temp.setProperty('-webkit-' + property, 'inherit', null);
        temp.setProperty('-moz-' + property, 'inherit', null);
        temp.setProperty('-o-' + property, 'inherit', null);
        temp.setProperty(property, 'inherit', null);

        return (temp.getPropertyValue('-webkit-' + property) == 'inherit' ||
                temp.getPropertyValue('-moz-' + property) == 'inherit' ||
                temp.getPropertyValue('-o-' + property) == 'inherit' ||
                temp.getPropertyValue(property) == 'inherit');
    }
    
    return {
        getUserAgent : getAgent,
        isMac : isMac,
        isSnowLeopard : isSnowLeopard,
        isWin : isWin,
        isWin2k : isWin2k,
        isWinVista : isWinVista,
        isWebKit : isWebKit,
        isSafari2 : isSafari2,
        isChrome : isChrome,
        isOpera : isOpera,
        isIE : isIE,
        isIEStrict : isIEStrict,
        isIE8 : isIE8,
        isFirefox : isFirefox,
        getIPhoneOSVersion : getIPhoneOSVersion,
        isiPad : isiPad,
        isMobile : isMobile,
        isHTML5Supported : isHTML5Supported,
        isQTInstalled : isQTInstalled,
        getQTVersion : getQTVersion,
        isQTCompatible : isQTCompatible,
        isValidQTAvailable : isValidQTAvailable,
        isSBVDPAvailable : isSBVDPAvailable,
        isCSSAvailable : isCSSAvailable
    };
	
})();



/*----------------------------------------------------------------------------------------------------------------------
                       "MEDIAELEMENT" MIXIN
                       
  This module serves as the base mixin object for all widgets. It includes properties and methods that are shared by all 
widget class implementations
 ---------------------------------------------------------------------------------------------------------------------*/
Concierge.MediaElement = {

    /**
     * Initializes the widget and its options. The resulting options object
     * is the user-defined options merged with those of the widget constructor's
     * default options
     *
     * @param options		the widget declaration options; these options will be merged with the widget 
     *                      constructor's default options
     */
    init : function(options) {
        this.setOptions(options);
        this.element = (Object.isElement(this.options.element))? this.options.element :
            Concierge.DOM.get(this.options.element);

        if(!this.element) {
            throw DOMException.NOT_FOUND_ERR;
        }

        return this;
    },


    /**
     * Sets the options for this widget instance. The basic options are defined on
     * the widget constructor and copied over to each widget instance
     *
     * @param options		the options hash to merge with the widget constructor's
     *				default options
     * @return a JSON object resulting from the merge of widget options
     */
    setOptions : function(options) {
        this.options = Object.clone(this.constructor.defaultOptions || { });
        return Object.extend(this.options, options || { });
    },


    /**
     * Publishes a topic (custom event) with the related data object. The topic
     * must follow the "eventType:eventName" pattern (i.e "widget:loaded"). The
     * data contains the event model and will be made available to the listeners
     * through the "memo" property of the event object
     *
     * @param topic		the topic to publish
     * @param data		the data to send with the topic
     */
    publish : function(topic, data) {
        if(!topic.include(":")) {
            throw DOMException.CUSTOM_EVENT_ERR;
        }
        
        Concierge.publish(this.element, topic, data);
        return this;
    },


    /**
     * Subscribes to a topic (custom event) published by a widget or any other code,
     * provided the topic follows the same "eventType:eventName" format. The event model
     * will be made available to the "callback" method through the "memo" property
     * of the event object
     *
     * @param topic		the topic to subscribe to
     * @param callback		the callback method to invoke when the topic event is fired
     */
    subscribe : function(topic, callback) {
        Concierge.subscribe(document, topic, callback);
        return this;
    },


    /**
     * Displays a string representation of the current widget
     *
     * @return a JSON string representing the structure of the widget
     */
    inspect : function() {
        return String.format("Widget : ID: #{id}", { options : this.id });
    }
};


Concierge.UID = 1;


Concierge.generateUID = function() {
    return Concierge.UID++;
};


Concierge.MediaType = {
    AUDIO : "audio",
    VIDEO : "video",
    IMAGE : "img"
};

Concierge.VideoType = {
    HTML5 : "HTML5",
    QUICKTIME : "Quicktime",
    FLASH : "Flash"
};


/**---------------------------------------------------------------------------------------------------------------------
 *                                  VideoElement
 *                      
 *  A class that serves as the superclass for all Video elements (HTML5, Quicktime, Flash). It defines the generic 
 *  initialization logic for the Video instances and defines the abstract methods that those subclasses need to
 *  implement
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.Media.VideoElement = Concierge.Class.create(Concierge.MediaElement, {
    
    /**
     * Initializes this VideoElement instance with the given options
     * 
     * @param options               an object that contains the options to customize this instance
     */
    initialize : function(options) {
        var defaultOptions = Object.clone(Concierge.Media.VideoElement.defaultOptions);
        this.options = Object.extend(defaultOptions, options || { });
        this.id = (defaultOptions.id || ("Concierge-VideoElement-" + Concierge.generateUID()));
        
        var sizeProvided = (Object.isNumber(this.options.width) && Object.isNumber(this.options.height));
        
        if(sizeProvided) {
            this.setVideoSize(this.options.width, this.options.height);
        } else {
            this.setVideoSize(this.options.defaultWidth * this.options.scaleFactor, 
                this.options.defaultHeight * this.options.scaleFactor);
        }
    },
    
    
    /**
     * Renders the view and related elements for this VideoElement instance. This method must be implemented by subclasses
     */
    render : Function.empty,
    
    
    /**
     * Returns the DOM element that contains this Video instance
     * 
     * @return the DOM element that contains this Video instance
     */
    getElement : function() {
        return this.element;
    },
    
    
    /**
     * Returns the ID of this Video instance
     * 
     * @return the ID of this Video instance
     */
    getID : function() {
        return this.id;
    },
    
    
    /**
     * Returns the underlying Video element (HTML5, Quicktime, Flash) for this instance. This method is abstract and
     * must be implemented by subclasses
     */
    getVideo : Function.abstractFunction,
    
    
    /**
     * Plays the Video for this instance. This method is abstract and must be implemented by subclasses
     */
    play : Function.abstractFunction,
    
    
    /**
     * Pauses the Video for this instance. THis method is abstract and must be implemented by subclasses
     */
    pause : Function.abstractFunction,
    
    
    /**
     * Seeks to a given time within the playable time range for the Video for this instance. This method is abstract
     * and must be implemented by subclasses
     */
    seek : Function.abstractFunction,
    
    
    /**
     * Returns true if the underlying Video for this instance is playing; otherwise, returns false. This method is
     * abstract and must be implemented by subclasses
     */
    isPlaying : Function.abstractFunction,
    
    
    /**
     * Returns the media type for this instance
     * 
     * @return the media type for this instance
     */
    getMediaType : function() {
        return this.options.type;
    },
    
    
    /**
     * Returns true if this instance is an HTML5 Video; otherwise, returns false
     * 
     * @return true if this isntance is an HTML5 Video; otherwise, returns false
     */
    isHTML5 : function() {
        return this.options.isHTML5;
    },
    
    
    /**
     * Returns true if this instance is a Quicktime Video; otherwise, returns false
     * 
     * @return true if this isntance is a Quicktime Video; otherwise, returns false
     */
    isQuicktime : function() {
        return this.options.isQuicktime;
    },
    
    
    /**
     * Returns true if this instance is a Flash Video; otherwise, returns false
     * 
     * @return true if this isntance is a Flash Video; otherwise, returns false
     */
    isFlash : function() {
        return this.options.isFlash;
    },
    
    
    /**
     * Returns true if the Video for this instance is streaming (playing a video with the M3U8 or F4M extensions); 
     * otherwise, returns false
     * 
     * @return true if this Video for this instance is streaming (playing a video with the M3U8 or F4M extensions); 
     * otherwise, returns false
     */
    isStreaming : function() {
        return this.options.isStreaming;
    },
    
    
    /**
     * Returns true if the Video for this instance is playing a live stream; otherwise, returns false
     * 
     * @return true if this Video for this instance is playing a live stream; otherwise, returns false
     */
    isLiveStreaming : function() {
        return this.options.isLiveStreaming;
    },
    
    
    /**
     * Sets the size for the underlying video for this instance. This is the dimension of the container that holds the 
     * video, so the video will be resized to fit within the specified rectangle while maintaining its aspect ratio.
     * This method is abstract and must be implemented by subclasses
     */
    setVideoSize : Function.abstractFunction,
    
    
    /**
     * Get the source of the underlying video for this instance. This method is abstract and must be implemented by 
     * subclasses
     */
    getSource : Function.abstractFunction,
    
    
    /**
     * Sets the video source to play. This method is abstract and must be implemented by subclasses
     */
    setSource : Function.abstractFunction,
    
    
    /**
     * Get the volume of the underlying video for this instance. This method is abstract and must be implemented by 
     * subclasses
     */
    getVolume : Function.abstractFunction,
    
    
    /**
     * Sets the video volume of the underlying video for this instance. This method is abstract and must be implemented by subclasses
     */
    setVolume : Function.abstractFunction,
    
    
    /**
     * Returns the width of the underlying video for this instance. This is the adjusted width at which the video is
     * being rendered so that it fits within its container
     * 
     * @return the width of the underlying video for this instance
     */
    getWidth : function() {
        return this.width;
    },
    
    
    /**
     * Returns the height of the underlying video for this instance. This is the adjusted height at which the video is
     * being rendered so that it fits within its container
     * 
     * @return the height of the underlying video for this instance
     */
    getHeight : function() {
        return this.height;
    },
    
    
    /**
     * Returns the natural width at which the Video was encoded
     * 
     * @return the natural width at which the video was encoded
     */
    getNaturalWidth : Function.abstractFunction,
    
    
    /**
     * Returns the natural height at which the Video was encoded
     * 
     * @return the natural height at which the video was encoded
     */
    getNaturalHeight : Function.abstractFunction
    
    
});


Concierge.Media.VideoElement.defaultOptions = {
    autoplay : true,
    type : Concierge.MediaType.VIDEO,
    defaultWidth : 160,
    defaultHeight : 90,
    scaleFactor : 2,
    volume : 0,
    duration : 0,
    currentTime : 0,
    src : null,
    poster : null,
    className : "akwaba-video-element",
    isStreaming : false,
    isLiveStreaming : false,
    isHTML5 : false,
    isQuicktime : false,
    isFlash : false,
    replay : false,
    metadataLoaded : false
};



/**---------------------------------------------------------------------------------------------------------------------
 *                                  HTML5 VIDEO
 *                      
 *  A class that renders an instance of an HTML5 Video element. It extends the Concierge.Media.VideoElement, sets the Video
 *  type to HTML5 and manages the creation of the underlying video, its lifecycle and its related events. 
 *  
 *  This class requires a delegate that will get notified of the different events fired by the Video element. The required
 *  handler methods for those events are specified in the "requiredMethods" property of the constructor function for
 *  this class
 *  
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.Media.HTML5Video = Concierge.Class.create(Concierge.Media.VideoElement, {
    
    /**
     * Initializes this instance with the given options
     * 
     * @param $super            a special pointer to the superclass implementation of the "initialize" method
     * @param options           an object that contains the customization options for this instance
     */
    initialize : function(options) {
        var defaultOptions = Object.extend({
            isHTML5 : true,
            autoplay : false,
            fullscreen : false,
            captions : null,
            className : "akwaba-video-element akwaba-html5-video"
        }, options || { });
        this._super(defaultOptions);
        
        this.handlers = {
            onVideoPlayable : this.onVideoPlayable.bind(this),
            onMetadataLoaded : this.onMetadataLoaded.bind(this),
            onTimeUpdate : this.onTimeUpdate.bind(this),
            onDurationChanged : this.onDurationChanged.bind(this),
            onProgress : this.onProgress.bind(this),
            onVideoPlayback : this.onVideoPlayback.bind(this),
            onVideoPaused : this.onVideoPaused.bind(this),
            onPlaybackEnded : this.onPlaybackEnded.bind(this),
            onFullscreenExit : this.onFullscreenExit.bind(this)
        };
        
        this.delegate = this.options.delegate;
        this.render();
    },
    
    
    /**
     * Sets the delegate for this widget
     * 
     * @param delegate              the delegate to set for this widget
     */
    setDelegate : function(delegate) {
        this.delegate = delegate;
        
        if(!this.initialized) {
            this.render();
        }
        
        return this;
    },
    
    
    /**
     * Checks whether the host environment supports HTML5 and, if so, renders and configures the underlying HTML5 Video 
     * element for this widget. It also checks whether the delegate implements the required methods for this widget, 
     * and throws an exception otherwise
     */
    render : function() {
        var proceed = (this.delegate && Concierge.Class.conformsToProtocol(this.delegate, this.constructor));
        
        if(!proceed) {
            var message = ["<p class=\"error-message\">Please provide a valid delegate for this HTML5 Video instance.</p>",
                "<p class=\"error-message\">Must implement the following required methods: #{methods}.</p>"].join("");

            var errorMessage = String.format(message, {
                methods : this.constructor.requiredMethods.join(", ")
            });
            
            throw {
                title : "IllegalState",
                message : errorMessage
            };
        }
        
        var detector = Concierge.Media.Detector,
        isHTML5Supported = detector.isHTML5Supported();
        
        if(!isHTML5Supported) {
            throw {
                name : "featureNotAvailable",
                message : "HTML5 is not available in this browser"
            };
        }
        
        var alwaysShowControls  = (detector.isiPad() || detector.isMobile()),
        videoElement = Concierge.DOM.createElement("video");
        videoElement.playerType = "video";
        videoElement.width = this.width;
        videoElement.height = this.height;
        
        var canPlayMP4Video = (videoElement.canPlayType("video/mp4") || videoElement.canPlayType("application/x-mpegURL"));
        
        if(canPlayMP4Video) {
            videoElement.playerType = "video/mp4";
        }
        
        videoElement.id = (this.options.id || ["html5Video", Concierge.generateUID()].join("_"));
        var isSourceAvailable = !!this.options.src;
        
        if(isSourceAvailable) {
            var source = this.options.src,
            extension = source.substring(source.lastIndexOf(".") + 1, source.length),
            isStreaming = (extension && (extension == "m3u8"));

            videoElement.setAttribute("src", source);
            
            if(isStreaming) {
                videoElement.setIsStreaming(true);
            }
            
            if(alwaysShowControls) {
                videoElement.setAttribute("controls", "controls");
            }
        }
        
        this.video = videoElement;
        this.playable = false;
        this.playing = false;
        this.ended = false;
        this.initialized = true;

        this.registerEvents();
        return this;
    },
    
    
    /**
     * Registers handlers for the events fired by the HTML5 video during its lifecycle (loading, metadata loading, etc)
     *  and user interactions (play, pause, seek, etc)
     */
    registerEvents : function() {
        var Event = Concierge.Event, 
        mediaEvents = Concierge.Media.HTML5Video.events;
        
        Event.add(this.video, mediaEvents.CAN_PLAY, this.handlers.onVideoPlayable);
        Event.add(this.video, mediaEvents.LOADED_METADATA, this.handlers.onMetadataLoaded);
        Event.add(this.video, mediaEvents.TIME_UPDATE, this.handlers.onTimeUpdate);
        Event.add(this.video, mediaEvents.DURATION_CHANGE, this.handlers.onDurationChanged);
        Event.add(this.video, mediaEvents.PROGRESS, this.handlers.onProgress);
        Event.add(this.video, mediaEvents.PLAYING, this.handlers.onVideoPlayback);
        Event.add(this.video, mediaEvents.PAUSE, this.handlers.onVideoPaused);
        Event.add(this.video, mediaEvents.ENDED, this.handlers.onPlaybackEnded);
        
        if(Concierge.Media.Detector.isWebKit()) {
            Event.add(this.video, mediaEvents.WEBKIT_END_FULLSCREEN, this.handlers.onFullscreenExit);
        }

        return this;
    },

    
    /**
     * A callback method invoked when the underlying Video has downloaded enough data so that it can start playing. 
     * It updates the playable state and notifies its delegate
     * 
     * @param event             the event fired by the video when it has downloaded enough data to start playback
     */
    onVideoPlayable : function(event) {
        this.playable = true;
        this.delegate.onVideoPlayable(this);
        return this;
    },
    
    
    /**
     * A callback method invoked when the metadata for the underlying Video is available. It notifies its delegate
     * 
     * @param event             the event fired by the video when its metadata is available
     */
    onMetadataLoaded : function(event) {
        this.delegate.onMetadataLoaded(this.video.metadata);
        return this;
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the playback time of the underlying Video is updated, It notifies
     * the delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when its playback time changes
     */ 
    onTimeUpdate : function(event) {
        this.delegate.onTimeUpdate({
            elapsed : this.video.currentTime,
            duration : this.video.duration
        });
        return this;
    },
    
    
    /**
     * A callback method that is invoked when the duration of the underlying Video is available, It notifies 
     * its delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when its duration is available
     */ 
    onDurationChanged : function(event) {
        this.delegate.onDurationChanged(this.video.duration);
        return this;
    },
    
    
    /**
     * A callback method that is invoked when playback starts or resumes, It notifies the delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when playback starts or resumes
     */ 
    onVideoPlayback : function(event) {
        this.delegate.onVideoPlayback(this);
        return this;
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the download progress of the video is updated. It computes the
     * percentage of data loaded and notifies the delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when its download progress is updated
     */ 
    onProgress : function(event) {
        try {
            
            var buffered = this.video.buffered.end(0),
            duration = this.video.duration,
            loaded = parseInt(buffered * 100 / duration);

            this.delegate.onProgress(loaded);
        
        } catch(invalidRange) {
            Concierge.log("Invalid range: %s", invalidRange.message);
        }
        
        return this;
    },
    
    
    /**
     * A callback method that is invoked when the video is paused, It notifies the delegate
     * 
     * @param event            the event fired when the video is paused
     */ 
    onVideoPaused : function(event) {
        this.delegate.onVideoPaused(this);
    },
    
    
    /**
     * A callback method that is invoked when playback ends, It notifies the delegate
     * 
     * @param event            the event fired by the underyling video when playback ends
     */ 
    onPlaybackEnded : function(event) {
        this.ended = true;
        this.delegate.onPlaybackEnded(this);
    },
    
    
    /**
     * A callback method that is invoked when the underlying video enters fullscreen mode. It notifies its delegate
     * 
     */ 
    enterFullscreenMode : function() {
        this.isFullscreen = true;
    },
    
    
    /**
     * A callback method that is invoked when the underlying video exits fullscreen mode. It notifies its delegate
     * 
     * @param event             the event fired when the video exits fullscreen mode
     */ 
    onFullscreenExit : function(event) {
        this.isFullscreen = false;
        this.delegate.onFullscreenExit(this);
    },
     
    
    /**
     * Returns a reference to the underlying HTML5 video for this widget
     * 
     * @return a reference to the underlying HTML5 video for this widget
     */
    getVideo : function() {
        return this.video;
    },
    
    
    /**
     * Returns the duration of the current video for this widget
     * 
     * @return the duration of the current video for this widget
     */
    getDuration : function() {
        return this.video.duration;
    },
    
    
    /**
     * Returns the current time of the video for this widget
     * 
     * @return the current time of the video for this widget
     */
    getCurrentTime : function() {
        return this.video.currentTime;
    },
    
    
    /**
     * Sets the current time for the underlying video for this widget. This action causes the video to jump to the
     * new time and start or resume playing
     * 
     * @param time              the current time to set for the video
     */
    setCurrentTime : function(time) {
        this.video.currentTime = time;
        return this;
    },
    
    
    /**
     * Returns true if the underlying video for this widget is set to autoplay; otherwise, returns false
     * 
     * @return true if the underlying video for this widget is set to autoplay; otherwise, returns false
     */
    isSetToAutoplay : function() {
        return this.options.autoplay;
    },
    
    
    /**
     * Returns true if the underlying video for this widget ismuted; otherwise, returns false
     * 
     * @return true if the underlying video for this widget is muted; otherwise, returns false
     */
    isMuted : function() {
        return (this.video.volume == 0);
    },
    
    
    /**
     * Mutes or unmutes the underlying video for this widget
     * 
     * @param muted         a flag that specifies whether to mute or unmute the video for this instance
     */
    setMuted : function(muted) {
        this.video.volume = 0;
        return this;
    },
    
    
    /**
     * Returns the playback rate of the underlying video for this instance
     * 
     * @return the playback rate of the underlying video for this instance
     */
    getPlaybackRate : function() {
        return this.video.playbackRate;
    },
    
    
    /**
     * Returns the default playback rate of the underlying video for this instance
     * 
     * @return the default playback rate of the underlying video for this instance
     */
    getDefaultPlaybackRate : function() {
        return this.video.defaultPlaybackRate;
    },
    
    
    /**
     * Sets the playback rate of the underlying video for this instance
     * 
     * @rate                the playback rate to set for the video
     */
    setPlaybackRate : function(rate) {
        this.playbackRate = rate;
    },
    
    
    /**
     * Sets the source of the video to play
     * 
     * @param src           the video source to set
     */
    setSource : function(src) {
        this.options.src = src;
        
        if(!this.video) {
            return this;
        }
        
        if(this.isPlaying()) {
            this.video.pause();
        }
        
        this.video.src = src;
        this.video.load();
        
        return this;
    },
    
    
    /**
     * Returns the source of the underlying video for this widget
     * 
     * @return the source of the underlying video for this widget
     */
    getSource : function() {
        return this.src;
    },

    
    /**
     * Get the volume of the underlying video for this instance
     */
    getVolume : function() {
        return this.video.volume;
    },
    
    
    /**
     * Sets the video volume of the underlying video for this instance
     */
    setVolume : function(volume) {
        this.video.volume = volume;
        return this;
    },
    
    
    /**
     * Get the DOM element that contains the element for this of the underlying video for this instance
     */
    getVideoContainer : function() {
        return this.videoContainer;
    },
    
    
    /**
     * Sets the DOM element that contains the element for this of the underlying video for this instance
     */
    setVideoContainer : function(container) {
        this.videoContainer = container;
        return this;
    },
    
    
    /**
     * Sets the size for the underlying video for this instance. This is the dimension of the container that holds the 
     * video, so the video will be resized to fit within the specified rectangle while maintaining its aspect ratio.
     * 
     * @param width                 the width to set for the video
     * @param height                the height to set for the video
     */
    setVideoSize : function(width, height) {
        this.width = width;
        this.height = height;
        
        if(this.video) {
            this.video.width = width;
        }
    },
    
    
    /**
     * Plays the current video source
     */
    play : function() {
        if(!this.playable) {
            return this;
        }
        
        this.video.play();
        this.playing = true;
        
        return this;
    },
    
    
    /**
     * Returns true if the underlying video is currently playing; otherwise, returns false
     *
     * @return true if the underlying video is currently playing; otherwise, returns false
     */
    isPlaying : function() {
        return this.playing;
    },
    
    
    /**
     * Pauses the current video source
     */
    pause : function() {
        if(!this.playable) {
            return this;
        }
        
        this.video.pause();
        this.playing = false;
        
        return this;
    },
    
    
    /**
     * Returns true if the underlying video is paused; otherwise, returns false
     *
     * @return true if the underlying video is paused; otherwise, returns false
     */
    isPaused : function() {
        return !this.isPlaying();
    },
    
    
    /**
     * Seeks to the given time within the playable time range for the Video for this instance, and starts or resumes
     * playing
     * 
     * @param time              the time to which to seek, in seconds
     */
    seek : function(time) {
        if(!this.playable) {
            return this;
        }
        
        this.video.currentTime = time;
        return this;
    },
    
    
    /**
     * Seeks back the specified number of seconds from the current playback time
     * 
     * @param seconds           the number of seconds to seek back from the current playhead position
     */
    seekBackNSeconds : function(seconds) {
        var proceed = (this.playable && Object.isNumber(seconds));
        
        if(!proceed) {
            return this;
        }
        
        this.video.currentTime = this.video.currentTime - seconds;
        return this;
    },
    
    
    /**
     * Seeks forward the specified number of seconds from the current playback time
     * 
     *  @param seconds           the number of seconds to seek forward from the current playhead position
     */
    seekForwardNSeconds : function(seconds) {
        this.seekBackNSeconds(-seconds);
        return this;
    },
    
    
    /**
     * Returns true if playback has ended; otherwise, returns false
     * 
     * @return true if playback has ended; otherwise, return false
     */
    isEnded : function() {
        return this.ended;
    },
    
    
    /**
     * Returns the natural width at which the Video was encoded
     * 
     * @return the natural width at which the video was encoded
     */
    getNaturalWidth : function() {
        return this.video.videoWidth;
    },
    
    
    /**
     * Returns the natural height at which the Video was encoded
     * 
     * @return the natural height at which the video was encoded
     */
    getNaturalHeight : function() {
        return this.video.videoHeight;
    },
    
    
    /**
     * Returns the time scale for the underlying video for this instance
     * 
     * @return the time scale for the underlying video for this instance
     */
    getTimeScale : function() {
        return this.video.timeScale;
    },
    
    
    /**
     * Returns true if the underlying video supports fullscreen mode; otherwise, returns false
     * 
     * @return true if the underlying video supports fullscreen mode; otherwise, returns false
     */
    supportsFullscreen : function() {
        return true;    /** Update logic */
    },
    
    
    /**
     * Stops playing the underlying video and clears its reference. This method performs some final cleanup before the
     * browser disposes of the current page
     * 
     * @param event             the "unload" event fired by the browser before recycling the current page
     */
    cleanup : function(event) {
        if(this.video) {
            this.video.stop();
            this.video.style.display = "none";
            this.video = null;
        }
    }
    
});


Object.extend(Concierge.Media.HTML5Video, {
    requiredMethods : ["onVideoPlayable", "onMetadataLoaded", "onTimeUpdate", "onDurationChanged", "onProgress",
        "onVideoPlayback", "onVideoPaused", "onPlaybackEnded"],
    
    events : {
        CAN_PLAY : "canplay",
        LOADED_METADATA : "loadedmetadata",
        TIME_UPDATE : "timeupdate",
        DURATION_CHANGE : "durationchange",
        PROGRESS : "progress",
        PLAYING : "playing",
        PLAY : "play",
        PAUSE : "pause",
        ENDED : "ended",
        WEBKIT_END_FULLSCREEN : "webkitEndedFullscreen"
    }
});



/**---------------------------------------------------------------------------------------------------------------------
 *                                  FLASH VIDEO
 *                      
 *  A class that renders an instance of a Flash Video element. It extends the Concierge.Media.VideoElement, sets the Video
 *  type to Flash and manages the creation of the underlying video, its lifecycle and its related events. 
 *  
 *  In order to properly embed the instance of the Flash video and still be able to overlay the control bar over it, 
 *  this component creates a DOM element inside the parent container and embeds the movie in that element.
 *  
 *  This class requires a delegate that will get notified of the different events fired by the Video element. The 
 *  required handler methods for those events are specified in the "requiredMethods" property of the constructor 
 *  function for this class
 *  
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.Media.FlashVideo = Concierge.Class.create(Concierge.Media.VideoElement, {
    
    /**
     * Initializes this instance with the given options
     * 
     * @param $super            a special pointer to the superclass implementation of the "initialize" method
     * @param options           an object that contains the customization options for this instance
     */
    initialize : function(options) {
        var constructorOptions = Object.clone(this.constructor.defaultOptions),
        defaultOptions = Object.extend(constructorOptions, options || { });
        this._super(defaultOptions);
        
        this.handlers = {
            onVideoPlayable : this.onVideoPlayable.bind(this),
            onMetadataLoaded : this.onMetadataLoaded.bind(this),
            onTimeUpdate : this.onTimeUpdate.bind(this),
            onDurationChanged : this.onDurationChanged.bind(this),
            onProgress : this.onProgress.bind(this),
            onVideoPlayback : this.onVideoPlayback.bind(this),
            onVideoPaused : this.onVideoPaused.bind(this),
            onPlaybackEnded : this.onPlaybackEnded.bind(this),
            onFullscreen : this.onFullscreen.bind(this)
        };

        this.delegate = this.options.delegate;
        this.render();
    },
    
    
    /**
     * Sets the delegate for this widget
     * 
     * @param delegate              the delegate to set for this widget
     */
    setDelegate : function(delegate) {
        this.delegate = delegate;
        
        if(!this.initialized) {
            this.render();
        }
        
        return this;
    },
    
    
    /**
     * Checks whether the host environment currently has a recent version of Flash (10.5 or newer) and, if so, initializes 
     * the DOM structure used to render the player.
     * It also checks whether the delegate implements the required methods for this widget, and throws an exception 
     * otherwise
     */
    render : function() {
        var proceed = (this.delegate && Concierge.Class.conformsToProtocol(this.delegate, this.constructor));
        
        if(!proceed) {
            var message = ["<p class=\"error-message\">Please provide a valid delegate for this HTML5 Video instance.</p>",
                "<p class=\"error-message\">Must implement the following required methods: {methods}.</p>"].join("");

            var errorMessage = String.format(message, {
                methods : this.constructor.requiredMethods.join(", ")
            });
            
            throw {
                title : "IllegalState",
                message : errorMessage
            };
        }
        
        var isFlashVersionCurrent = Concierge.MediaUtils.hasRequiredFlashVersion(10, 5);
        
        if(!isFlashVersionCurrent) {
            throw {
                name : "InvalidFlashVersion",
                message : "You're using an older version of Flash. Please upgrade to version 10.5 or newer"
            };
        }

        this.element = (Object.isString(this.options.element))? Concierge.DOM.get(this.options.element) : this.options.element;
        
        if(!this.element) {
            throw DOMException.NOT_FOUND_ERR;
        }
        
        this.initializeView();
        return this;
    },
    
    
    /**
     * Creates the DOM element that will contain the player, initializes the state of this component and creates the
     * instance of the LightviewVideoPlayer SWF file for this component
     */
    initializeView : function() {
        var DOM = Concierge.DOM;
        this.id = (this.options.id || ["Concierge-Player-Flash", Concierge.generateUID()].join("_"));
        
        this.swfMovieContainer = DOM.createElement("div", {
            id : this.id,
            className : this.options.className
        });

        DOM.setStyle(this.swfMovieContainer, {
            width : this.width + "px",
            height: this.height + "px",
            margin : "0 auto"
        });
        this.element.appendChild(this.swfMovieContainer);
        
        this.playerType = "video";
        var isSourceAvailable = !!this.options.src;
        
        if(isSourceAvailable) {
            var source = this.options.src,
            extension = source.substring(source.lastIndexOf(".") + 1, source.length);
            this.videoSource = source;
            this.options.isStreaming = (extension && (extension == "f4m"));
        }
       
        this.renderVideo();
        this.playable = false;
        this.playing = false;
        this.ended = false;
        this.registerEvents();
        this.initialized = true;
        
        return this;
    },
    
    
    /**
     * Creates the instance of the underlying LightviewVideoPlayer Flash video for this component
     */
    renderVideo : function() {
        var flashvars = { 
			width : this.width,
			height : this.height,
			videoSource : this.videoSource,
            autoplay : this.options.autoplay
        }, 
        params = {
            allowscriptaccess : "always",
			wmode : "window"
        }, 
        attributes = { };
        
        if(this.options.poster) {
            flashvars.poster = this.options.poster;
        }
        
        swfobject.embedSWF(this.options.swfMovieURL, this.id, this.width, this.height, "10.5.0", false, 
        		flashvars, params, attributes);
        
        return this;
    },
    
    
    /**
     * Registers handlers for the events fired by the Flash video during its lifecycle (loading, metadata loading, etc)
     *  and user interactions (play, pause, seek, etc)
     */
    registerEvents : function() {
        var mediaEvents = Concierge.Media.FlashVideo.events;
        
        Concierge.subscribe(mediaEvents.CAN_PLAY, this.handlers.onVideoPlayable);
        Concierge.subscribe(mediaEvents.LOADED_METADATA, this.handlers.onMetadataLoaded);
        Concierge.subscribe(mediaEvents.TIME_UPDATE, this.handlers.onTimeUpdate);
        Concierge.subscribe(mediaEvents.DURATION_CHANGE, this.handlers.onDurationChanged);
        Concierge.subscribe(mediaEvents.PROGRESS, this.handlers.onProgress);
        Concierge.subscribe(mediaEvents.PLAYING, this.handlers.onVideoPlayback);
        Concierge.subscribe(mediaEvents.PAUSE, this.handlers.onVideoPaused);
        Concierge.subscribe(mediaEvents.ENDED, this.handlers.onPlaybackEnded);
        //this.subscribe(events.FULLSCREEN, this.handlers.onFullscreen);

        return this;
    },

    
    /**
     * A callback method invoked when the underlying Video has downloaded enough data so that it can start playing. 
     * It updates the playable state and notifies its delegate
     * 
     * @param event             the event fired by the video when it has downloaded enough data to start playback
     */
    onVideoPlayable : function(event) {
        this.playable = true;
        this.delegate.onVideoPlayable(this);
        return this;
    },
    
    
    /**
     * A callback method invoked when the metadata for the underlying Video is available. It notifies its delegate
     * 
     * @param event             the event fired by the video when its metadata is available
     */
    onMetadataLoaded : function(event) {
        this.delegate.onMetadataLoaded({ });
        return this;
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the playback time of the underlying Video is updated, It notifies
     * the delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when its playback time changes
     */ 
    onTimeUpdate : function(event) {
        this.delegate.onTimeUpdate({
            elapsed : event.memo.elapsed, 
            duration : event.memo.duration
        });
        
        return this;
    },
    
    
    /**
     * A callback method that is invoked when the duration of the underlying Video is available, It notifies 
     * its delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when its duration is available
     */ 
    onDurationChanged : function(event) {
        var duration = parseInt(event.memo.duration);
        this.delegate.onDurationChanged(duration);
        return this;
    },
    
    
    /**
     * A callback method that is invoked when playback starts or resumes, This event is not used by Actionscript
     * 
     * @param event            the event fired repeatedly by the underyling video when playback starts or resumes
     */ 
    onVideoPlayback : function(event) {
        this.delegate.onVideoPlayback(this);
        return this;
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the download progress of the video is updated. It computes the
     * percentage of data loaded and notifies the delegate
     * 
     * @param event            the event fired repeatedly by the underyling video when its download progress is updated
     */ 
    onProgress : function(event) {
        var percentageLoaded = event.memo.loaded * 100  / event.memo.total;
        this.delegate.onProgress(percentageLoaded);
        
        return this;
    },
    
    
    /**
     * A callback method that is invoked when the video is paused, It notifies the delegate
     * 
     * @param event            the event fired when the video is paused
     */ 
    onVideoPaused : function(event) {
        this.delegate.onVideoPaused(this);
    },
    
    
    /**
     * A callback method that is invoked when playback ends, It notifies the delegate
     * 
     * @param event            the event fired by the underyling video when playback ends
     */ 
    onPlaybackEnded : function(event) {
        this.ended = true;
        this.delegate.onPlaybackEnded(this);
    },
    
    
    /**
     * A callback method that is invoked when the underlying video enters fullscreen mode. It notifies its delegate
     * 
     */ 
    enterFullscreenMode : function() {
        this.isFullscreen = true;
    },
    
    
    /**
     * A callback method that is invoked when the underlying video exits fullscreen mode. It notifies its delegate
     */ 
    onFullscreen : function(event) {
        this.isFullscreen = false;
        //this.delegate.onFullscreenExit(this);
    },
     
    
    /**
     * Returns a reference to the underlying HTML5 video for this widget
     * 
     * @return a reference to the underlying HTML5 video for this widget
     */
    getVideo : function() {
        return (this.video = swfobject.getObjectById(this.id));
    },
    
    
    /**
     * Returns the duration of the current video for this widget
     * 
     * @return the duration of the current video for this widget
     */
    getDuration : function() {
        return this.video.duration;
    },
    
    
    /**
     * Returns the current time of the video for this widget
     * 
     * @return the current time of the video for this widget
     */
    getCurrentTime : function() {
        var video = this.getVideo();
        return video.getCurrentTime();
    },
    
    
    /**
     * Sets the current time for the underlying video for this widget. This action causes the video to jump to the
     * new time and start or resume playing
     * 
     * @param time              the current time to set for the video
     */
    setCurrentTime : function(time) {
        var video = this.getVideo();
        video.seekTo(time);
    },
    
    
    /**
     * Returns true if the underlying video for this widget is set to autoplay; otherwise, returns false
     * 
     * @return true if the underlying video for this widget is set to autoplay; otherwise, returns false
     */
    isSetToAutoplay : function() {
        return this.options.autoplay;
    },
    
    
    /**
     * Returns true if the underlying video for this widget ismuted; otherwise, returns false
     * 
     * @return true if the underlying video for this widget is muted; otherwise, returns false
     */
    isMuted : function() {
        var video = this.getVideo();
        return (video.getVolume() == 0);
    },
    
    
    /**
     * Mutes or unmutes the underlying video for this widget
     * 
     * @param muted         a flag that specifies whether to mute or unmute the video for this instance
     */
    setMuted : function(muted) {
        var video = this.getVideo();
        video.setVolume(0);
        return this;
    },
    
    
    /**
     * Returns the playback rate of the underlying video for this instance
     * 
     * @return the playback rate of the underlying video for this instance
     */
    getPlaybackRate : function() {
        var video = this.getVideo();
        return video.getPlaybackRate();
    },
    
    
    /**
     * Returns the default playback rate of the underlying video for this instance
     * 
     * @return the default playback rate of the underlying video for this instance
     */
    getDefaultPlaybackRate : function() {
        var video = this.getVideo();
        return video.getDefaultPlaybackRate();
    },
    
    
    /**
     * Sets the playback rate of the underlying video for this instance
     * 
     * @rate                the playback rate to set for the video
     */
    setPlaybackRate : function(rate) {
        var video = this.getVideo();
        video.setDefaultPlaybackRate(rate);
    },
    
    
    /**
     * Sets the source of the video to play
     * 
     * @param src           the video source to set
     */
    setSource : function(src) {
        this.videoSource = src;
        var video = this.getVideo();
        
        if(!video) {
            return this;
        }

        video.setVideoSource(src);
        
        return this;
    },
    
    
    /**
     * Returns the source of the underlying video for this widget
     * 
     * @return the source of the underlying video for this widget
     */
    getSource : function() {
        return this.videoSource;
    },

    
    /**
     * Get the volume of the underlying video for this instance
     */
    getVolume : function() {
        var video = this.getVideo();
        return video.getVolume();
    },
    
    
    /**
     * Sets the video volume of the underlying video for this instance
     */
    setVolume : function(volume) {
        var video = this.getVideo();
        video.setVolume(volume);
        return this;
    },
    
    
    /**
     * Get the DOM element that contains the element for this of the underlying video for this instance
     */
    getVideoContainer : function() {
        return this.videoContainer;
    },
    
    
    /**
     * Sets the DOM element that contains the element for this of the underlying video for this instance
     */
    setVideoContainer : function(container) {
        this.videoContainer = container;
        return this;
    },
    
    
    /**
     * Sets the size for the underlying video for this instance. This is the dimension of the container that holds the 
     * video, so the video will be resized to fit within the specified rectangle while maintaining its aspect ratio.
     * 
     * @param width                 the width to set for the video
     * @param height                the height to set for the video
     */
    setVideoSize : function(width, height) {
        this.width = width;
        this.height = height;
    },
    
    
    /**
     * Plays the current video source
     */
    play : function() {
        var video = this.getVideo();
        video.playVideo();
        this.playing = true;
        
        return this;
    },
    
    
    /**
     * Returns true if the underlying video is currently playing; otherwise, returns false
     *
     * @return true if the underlying video is currently playing; otherwise, returns false
     */
    isPlaying : function() {
        return this.playing;
    },
    
    
    /**
     * Pauses the current video source
     */
    pause : function() {
        if(!this.playable) {
            return this;
        }
        
        var video = this.getVideo();
        video.pauseVideo();
        this.playing = false;
        
        return this;
    },
    
    
    /**
     * Returns true if the underlying video is paused; otherwise, returns false
     *
     * @return true if the underlying video is paused; otherwise, returns false
     */
    isPaused : function() {
        return !this.isPlaying();
    },
    
    
    /**
     * Seeks to the given time within the playable time range for the Video for this instance, and starts or resumes
     * playing
     * 
     * @param time              the time to which to seek, in seconds
     */
    seek : function(time) {
        if(!this.playable) {
            return this;
        }
        
        var video = this.getVideo();
        video.seekTo(time);
        return this;
    },
    
    
    /**
     * Seeks back the specified number of seconds from the current playback time
     * 
     * @param seconds           the number of seconds to seek back from the current playhead position
     */
    seekBackNSeconds : function(seconds) {
        var proceed = (this.playable && Object.isNumber(seconds));
        
        if(!proceed) {
            return this;
        }
        
        var video = this.getVideo(),
        currentTime = video.getCurrentTime();
        video.seekTo(currentTime - seconds);
        return this;
    },
    
    
    /**
     * Seeks forward the specified number of seconds from the current playback time
     * 
     *  @param seconds           the number of seconds to seek forward from the current playhead position
     */
    seekForwardNSeconds : function(seconds) {
        this.seekBackNSeconds(-seconds);
        return this;
    },
    
    
    /**
     * Returns true if playback has ended; otherwise, returns false
     * 
     * @return true if playback has ended; otherwise, return false
     */
    isEnded : function() {
        return this.ended;
    },
    
    
    /**
     * Returns the natural width at which the Video was encoded
     * 
     * @return the natural width at which the video was encoded
     */
    getNaturalWidth : function() {
        var video = this.getVideo()
        return video.getNaturalWidth();
    },
    
    
    /**
     * Returns the natural height at which the Video was encoded
     * 
     * @return the natural height at which the video was encoded
     */
    getNaturalHeight : function() {
        var video = this.getVideo();
        return video.getNaturalHeight();
    },
    
    
    /**
     * Returns the time scale for the underlying video for this instance
     * 
     * @return the time scale for the underlying video for this instance
     */
    getTimeScale : function() {
        return 25;
    },
    
    
    /**
     * Returns true if the underlying video supports fullscreen mode; otherwise, returns false
     * 
     * @return true if the underlying video supports fullscreen mode; otherwise, returns false
     */
    supportsFullscreen : function() {
        var video = this.getVideo();
        return video.isFullscreenSupported();
    },
    
    
    /**
     * Stops playing the underlying video and clears its reference. This method performs some final cleanup before the
     * browser disposes of the current page
     * 
     * @param event             the "unload" event fired by the browser before recycling the current page
     */
    cleanup : function(event) {
        if(this.video) {
            this.video.stop();
            this.video.style.display = "none";
            this.video = null;
        }
    }
    
});


Object.extend(Concierge.Media.FlashVideo, {
    requiredMethods : ["onVideoPlayable", "onMetadataLoaded", "onTimeUpdate", "onDurationChanged", "onProgress",
        "onVideoPlayback", "onVideoPaused", "onPlaybackEnded"],
    defaultOptions : {
        isFlash : true,
        autoplay : false,
        fullscreen : false,
        captions : null,
        className : "akwaba-video-element akwaba-flash-video",
        swfMovieURL : "/concierge/flash/LightweightVideoPlayer.swf?071311"
    },
    events : {
        LOADED_METADATA : "mediaPlayer:loadedmetadata",
        TIME_UPDATE : "mediaPlayer:timeupdate",
        PROGRESS : "mediaPlayer:progress",
        DURATION_CHANGE : "mediaPlayer:durationchange",
        CAN_PLAY : "mediaPlayer:canplay",
        PLAY : "mediaPlayer:play",
        PLAYING : "mediaPlayer:playing",
        PAUSE : "mediaPlayer:pause",
        ENDED : "mediaPlayer:ended",
        FULLSCREEN : "mediaPlayer:fullscreen"
    }
});



/**---------------------------------------------------------------------------------------------------------------------
 *                                      MEDIA FACTORY
 *                      
 *  A class that creates instances of Video widget of diffent types: HTML5, Flash or Quicktime. It also features the 
 *  "createVideo" method that automatically creates the appropriate video element based on the  capabilities of the
 *  current browser
 *  
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.MediaFactory = (function() {
    
    "use strict";
    
    var videoTypes = {
        HTML5 : Concierge.Media.HTML5Video,
        FLASH : Concierge.Media.FlashVideo
    },
    registry = { };
    
    
    /**
     * Creates a video of the specified type and with the given options
     * 
     * @param type              the type of video to create
     * @param options           the options to customize the video instance
     * @return a video of the specified type and with the given options
     */
    function create(type, options) {
        if(!videoTypes[type]) {
            return null;
        }
        
        var videoAlreadyExists = !!(options.id && registry[options.id]),
        videoID = options.id,
        video;
            
        if(!videoID) {
            videoID = ["video", Concierge.generateUID()].join("_");
            options.id = videoID;
        }
        
        video = new videoTypes[type](options);
        registry[videoID] = video;
        
        return video;
    }
    
    
    return {
      
        /**
         * Creates an instance of the Concierge.Media.HTML5Video widget with the given options
         * 
         * @param options         the options to customize the HTML5 video widget
         * @return an instance of the Concierge.Media.HTML5Video widget with the given options
         */
        createHTML5Video : function(options) {
            var video = create("HTML5", options);
            // if not HTML5 supported, create Quicktime or Flash fallback...
            return video;
        },
      
        
        /**
         * Creates an instance of the Concierge.Media.FlashVideo widget with the given options
         * 
         * @param options         the options to customize the Flash video widget
         * @return an instance of the Concierge.Media.FlashVideo widget with the given options
         */
        createFlashVideo : function(options) {
            var video = create("FLASH", options);
            // if not HTML5 supported, create Quicktime or Flash fallback...
            return video;
        },
        
        
        /**
         * Creates the appropriate instance of an Concierge,Media.VideoElement (HTML5Video, FlashVideo or QuicktimeVide)
         * based on the capabilities of the current browser environment
         * 
         * @param options             an object that contains the properties to customize the video instance
         * @return an instance of the Concierge.Media.VideoElement based on the capabilities of the browser environment
         */
        createVideo : function(options) {
            // Autodetect video support and create the appropriate video type...
        },


        /**
         * Returns the instance of the Concierge.Media.VideoElement with the given ID, or null if no such widget exists
         * 
         * @param id                  the ID of the video instance to return
         * @return the instance of the Concierge.Media.VideoElement with the given ID, or null of no such widget exists
         */
        getVideoByID : function(id) {
            return registry[id];
        }
        
    };
    
})();



/**---------------------------------------------------------------------------------------------------------------------
 *                                      MEDIA PLAYER
 *                      
 *  A class that is the Controller in the MVC pattern used by the Concierge Media architecture. 
 *  
 *  It holds a reference an instance of the Concierge.Media.VideoElement, which represents the model in the pattern, 
 *  and acts as the delegate for that component. It also holds a reference to the view, which is the DOM element in 
 *  which the video will be rendered. And finally, it creates an instance of Concierge.Media.Controls, which allows the
 *  user to interact with the video element
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.Media.Player = Concierge.Class.create({
  
    /**
     * Initializes this controller instance with the given options
     * 
     * @param options               an object that contains the options to customize this controller instance
     */
    initialize : function(options) {
        var defaultOptions = Object.clone(this.constructor.defaultOptions);
        this.options = Object.extend(defaultOptions, options || { });
        this.videoType = this.options.videoType;
        this.videoContainer = (Object.isString(this.options.videoContainer))? Concierge.DOM.get(this.options.videoContainer) : 
            this.options.videoContainer;
        
        if(!this.videoContainer) {
            throw DOMException.NOT_FOUND_ERR;
        }
        
        this.videoSource = this.options.videoSource;
        this.poster = this.options.poster;
        this.videoID = (this.options.videoID || "Concierge-Video-" + Concierge.generateUID());
        
        this.handlers = {
            handleMouseMovements : this.handleMouseMovements.bind(this),
            onPlayIconClicked : this.onPlayIconClicked.bind(this)
        };
        
        this.initializeView();
    },
  
    
    /**
     * Sets the DOM container in which to display the video
     * 
     * @param container             the DOM container to set
     */
    initializeView : function() {
        var DOM = Concierge.DOM;
        
        if(!DOM.hasClassName(this.videoContainer, "akwaba-video-player")) {
            DOM.addClassName(this.videoContainer, "akwaba-video-player");
        }
        
        var isPositioned = (/relative|absolute/.test(DOM.getStyle(this.videoContainer, "position")));
        
        if(!isPositioned) {
            DOM.setStyle(this.videoContainer, {
                position: "relative"
            });
        }
        
        this.sizeContainerToFitVideo(this.videoContainer);
        
        var createHTML5Video = (this.options.videoType == Concierge.VideoType.HTML5),
        createFlashVideo = (this.options.videoType == Concierge.VideoType.FLASH);
        
        if(createHTML5Video) {
            this.videoElement = this.createHTML5Video(this.videoID);
            this.videoContainer.appendChild(this.videoElement.video);
        } else {
            this.videoElement = this.createFlashVideo(this.videoID);
        }

        this.videoElement.setVideoContainer(this.videoContainer);
        this.videoBeganPlaying = false;
        this.videoEndedPlaying = false;
        
        this.hasNativeControls = (Concierge.Media.Detector.isiPad() || Concierge.Media.Detector.isMobile());
        var createControlBar = (!this.hasNativeControls && !this.controlBar)
        
        if(createControlBar) {
            this.initializeControlBar().createPlayIconOverlay();
        }
        
        return this;
    },
  
    
    /**
     * Creates an instance of an HTML5 video with the given ID
     * 
     * @param videoID               the ID of the HTML5 video to create
     * @return an instance of an HTML5 video with the given ID
     */
    createHTML5Video : function(videoID) {
        var video = Concierge.MediaFactory.createHTML5Video({
            id : videoID,
            width : this.options.width,
            height : this.options.height,
            autoplay : this.options.autoplay,
            src : this.videoSource,
            poster : this.poster,
            delegate : this
        });
        
        var enableAirplay = (this.options.enableAirplay && 
            (Concierge.Media.Detector.isiPad() || Concierge.Media.Detector.isMobile()));
        
        if(enableAirplay) {
            try {
                
                video.setAttribute("x-webkit-airplay", "allow");
            
            } catch(airplayNotSupported) {
            }
        }
        
        return video;
    },
    
    
    /**
     * Creates an instance of a Flash video with the given ID
     * 
     * @param videoID               the ID of the Flash video to create
     * @return an instance of a Flash video with the given ID
     */
    createFlashVideo : function(videoID) {
        return Concierge.MediaFactory.createFlashVideo({
            id : videoID,
            element : this.videoContainer,
            width: this.options.width,
            height: this.options.height,
            autoplay : this.options.autoplay,
            src : this.videoSource,
            poster : this.poster,
            swfMovieURL : this.options.swfMovieURL,
            delegate : this
        });
    },
    
    
    /**
     * Sets the instance of Concierge.Media.VideoElement for this controller
     * 
     * @param videoElement          the video instance to set for this controller
     */
    setVideoElement : function(videoElement) {
        this.videoElement = videoElement;
        return this;
    },
  
    
    /**
     * Initializes the control bar component used to interact with the video player
     */
    initializeControlBar : function() {
        this.controlBar = new Concierge.Media.ControlBar({
            container : this.videoContainer,
            delegate : this 
        });
        
        this.observeMouseEvents();
        return this;
    },
    
    
    /**
     * Creates the "Play" icon and displays it as an overlay over the video container
     */
    createPlayIconOverlay : function() {
        this.playIcon = Concierge.DOM.createElement("a", {
            id : "Concierge-Play-" + this.videoID,
            href : "#play",
            className : "play-icon"
        });
        this.videoContainer.appendChild(this.playIcon);
        Concierge.Event.add(this.playIcon, "click", this.handlers.onPlayIconClicked);
        
        return this;
    },
    
    
    /**
     * A callback method invoked when the user clicks the "Play" icon. It plays the current video
     * 
     * @param event             the click event on the Play icon
     */
    onPlayIconClicked : function(event) {
        Concierge.Event.stop(event);
        this.play();
    },
    
    
    /**
     * Observes mouse move events on the video container, and hides or show the control bar based on the position of
     * the mouse
     */
    observeMouseEvents : function() {
        Concierge.Event.add(this.videoContainer, "mousemove", this.handlers.handleMouseMovements);
    },
    
    
    /**
     * Adjusts the size of the video container to the dimensions of the current video
     * 
     * @param container             the container whose size tp adjust
     */
    sizeContainerToFitVideo : function() {
        var dimensions = Concierge.DOM.measure(this.videoContainer),
        isSizedToVideo = ((dimensions.width == this.options.width) && (dimensions.height == this.options.height));
        
        if(!isSizedToVideo) {
            Concierge.DOM.setStyle(this.videoContainer, {
                width : this.options.width + "px",
                height : this.options.height + "px"
            });
        }
    },
    
    
    /**
     * A callback method invoked when the underlying Video has downloaded enough data so that it can start playing. 
     * It starts playing the video
     * 
     * @param videoElement          the underlying video element sending the notification
     */
    onVideoPlayable : function(videoElement, duration) {
        if(videoElement.isHTML5()) {
            if(this.options.autoplay) {
                this.play();
            }

            if(this.controlBar) {
                this.controlBar.setVolumeLevel(this.videoElement.getVolume());
            }
        }
    },
    
    
    /**
     * A callback method invoked when the metadata for the underlying Video is available. It extracts the data for the
     * video
     * 
     * @param metadata              the metadata that's now available
     */
    onMetadataLoaded : Function.empty,
    
    
    /**
     * A callback method that is invoked repeatedly when the playback time of the underlying Video is updated, It updates
     * the elapsed duration of the video
     * 
     * @param videoElement          the video element sending the notification
     */ 
    onTimeUpdate : function(info) {
        var currentTime = parseInt(info.elapsed);
        
        if(this.controlBar) {
            this.controlBar.onTimeUpdate(currentTime, this.videoDuration);
        }
    },
    
    
    /**
     * A callback method that is invoked when the duration of the underlying Video is available, It updates the video
     * controls with the duration
     * 
     * @param duration              the duration of the video
     */
    onDurationChanged : function(duration) {
        this.videoDuration = duration;
        
        if(this.controlBar) {
            this.controlBar.onDurationAvailable(duration);
        }
    },
    
    
    /**
     * A callback method that is invoked when playback starts or resumes
     * 
     * @param videoElement          the video element sending the notification
     */ 
    onVideoPlayback : function(videoElement) {
        if(!this.playbackStarted) {
            this.playbackStarted = true;
        }
        
        if(this.controlBar) {
            this.controlBar.onPlaybackStateChanged("play");
            Concierge.DOM.hide(this.playIcon);
        }
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the download progress of the video is updated. It updates the
     * width of the percentage indicator
     * 
     * @param loaded                the percentage of data currently loaded
     */
    onProgress : function(loaded) {
        if(this.controlBar) {
            this.controlBar.onProgress(loaded);
        }
    },
    
    
    /**
     * A callback method that is invoked when the video is paused
     * 
     * @param videoElement          the video element sending the notification
     */ 
    onVideoPaused : function(videoElement) {
        if(this.controlBar) {
            this.controlBar.onPlaybackStateChanged("pause");
            Concierge.DOM.show(this.playIcon);
        }
    },
    
    
    /**
     * A callback method that is invoked when playback ends
     * 
     * @param videoElement          the video element sending the notification
     */
    onPlaybackEnded : function(videoElement) {
        if(this.controlBar) {
            this.controlBar.onPlaybackEnded();
            Concierge.DOM.show(this.playIcon);
        }
    },
  
    
    /**
     * Plays the current video source
     */
    play : function() {
        this.videoElement.play();
        
        if(this.controlBar) {
            this.controlBar.onPlaybackStateChanged("play");
            this.prepareToHideControlBar();
        }
        
        return this;
    },

  
    /**
     * Pauses the current video
     */
    pause : function() {
        this.videoElement.pause();

        if(this.controlBar) {
            this.controlBar.onPlaybackStateChanged("pause");
        }

        return this;
    },
  
    
    /**
     * Returns the video element for this media player
     * 
     * @return the video element for this media player
     */
    getVideoElement : function() {
        return this.videoElement;
    },
    
    
    /**
     * Sets the source for the underlying video
     * 
     * @param source                the source to set for the video
     */
    setVideoSource : function(source) {
        this.videoSource = source;
        
        if(this.videoElement) {
            this.videoElement.pause().setSource(source);
        }
        
        if(this.controlBar) {
            this.controlBar.resetView().onPlaybackStateChanged("pause");
        }

        this.playbackStarted = false;

        return this;
    },
  
  
    /**
     * Returns the source for the underlying video
     * 
     * @return the source for the underlying video
     */
    getVideoSource : function() {
        return this.videoElement.getVideoSource();
    },
  
  
    /**
     * Sets the poster image to display while the content is being downloaded
     * 
     * @param poster            the poster to display while the content is being downloaded
     */
    setPoster : function(poster) {
        this.poster = poster;
        return this;
    },
  
  
    /**
     * Returns the poster to display while the content is being downloaded
     * 
     * @return the poster to display while the content is being downloaded
     */
    getPoster : function() {
        return this.poster;
    },
  
 
    /**
     * A callback method invoked repeatedly as the user moves the mouse. It shows the control bar, then computes the 
     * position of the mouse within the confines of the video container. If the mouse is within the control bar, the
     * control bar remains visible; otherwise, it fades after 2.5 seconds
     * 
     * @param event                 the "mouse move" event fired as the user moves the mouse on the video container
     */
    handleMouseMovements : function(event) {
        if(!this.playbackStarted) {
            return this;
        }
        
        if(this.controlBar) {
            this.controlBar.show();
        }
        
        var controlBarBoundaries = this.getControlBarBoundaries(),
        videoContainerOffset = Concierge.DOMStorage.retrieve(this.videoContainer, "offsets");
        
        if(!videoContainerOffset) {
            videoContainerOffset = Concierge.DOM.cumulativeOffset(this.videoContainer);
            Concierge.DOMStorage.store(this.videoContainer, "offsets", videoContainerOffset);
        }
        
        var relativePositionX = Concierge.Event.pointerX(event) - videoContainerOffset.left,
        relativePositionY = Concierge.Event.pointerY(event) - videoContainerOffset.top;
        
        var isWithinControlBar = ((relativePositionX > controlBarBoundaries.left) &&
                                  (relativePositionX < controlBarBoundaries.right) &&
                                  (relativePositionY > controlBarBoundaries.top) &&
                                  (relativePositionY < controlBarBoundaries.bottom));
        
        if(isWithinControlBar) {
            if(this.controlFadeTimeout) {
                clearTimeout(this.controlFadeTimeout);
            }

            return this;
        }
        
        if(this.controlFadeTimeout) {
            clearTimeout(this.controlFadeTimeout);
        }
        
        this.prepareToHideControlBar();
        return this;
    },
    
    
    /**
     * Returns an object that contains the coordinates of the control bar boundaries
     * 
     * @return an object that contains the coordinates of the control bar boundaries
     */
    getControlBarBoundaries : function() {
        if(this.controlBarBoundaries) {
            return this.controlBarBoundaries;
        }
        
        var offset = Concierge.DOM.positionedOffset(this.controlBar.element),
        dimensions = Concierge.DOM.measure(this.controlBar.element);
        
        this.controlBarBoundaries = {
            left : offset.left,
            top : offset.top,
            right : offset.left + dimensions.width,
            bottom : offset.top + dimensions.height
        };
        
        return this.controlBarBoundaries;
    },
    
    
    /**
     * Triggers a time that will hide the control bar after 2.5 seconds
     */
    prepareToHideControlBar : function() {
        if(this.controlFadeTimeout) {
            clearTimeout(this.controlFadeTimeout);
        }
        
        this.controlFadeTimeout = setTimeout(this.hideControlBar.bind(this), 2500);
    },
    
    
    /**
     * Hides the control panel
     */
    hideControlBar : function() {
        if(this.controlFadeTimeout) {
            clearTimeout(this.controlFadeTimeout);
        }
        
        if(this.controlBar) {
            this.controlBar.hide();
        }
    },
    
    
    /**
     * A method invoked when the user cliks a playback button (play, pause, rewind, fast=-forward). It takes the appropriate
     * action based on the button clicked
     * 
     * @param info              an object that contains the playback action
     */
    onPlaybackActions : function(info) {
        var playVideo = (info.action == "play"),
        pauseVideo = (info.action == "pause"),
        rewindVideo = (info.action == "rewind"),
        fastForward = (info.action == "fast-forward");
        
        if(playVideo || pauseVideo) {
            this[(playVideo)? "play" : "pause"]();
        } else {
            
            var TEN_SECONDS = 10,
            currentTime = this.getCurrentTime(),
            proceed = ((rewindVideo && currentTime > TEN_SECONDS) || 
                       (fastForward && currentTime + TEN_SECONDS < this.videoDuration));
            
            if(proceed) {
                var isPlayingBeforeSeek = this.videoElement.isPlaying(),
                rewindTime = Math.max(0, currentTime - TEN_SECONDS),
                fastForwardTime = Math.min(this.videoDuration, currentTime + TEN_SECONDS);

                this.videoElement.setCurrentTime((rewindVideo)? rewindTime : fastForwardTime);
                this[(isPlayingBeforeSeek)? "play" : "pause"]();
            }
        }
    },
    
    
    /**
     * Begins seeking to the given value
     * 
     * @param controlBar            the control bar sending the notification
     * @param value                 the video time to seek to
     */
    beginSeeking : function(controlBar, value) {
        
        if(!this.seeking) {
            this.wasPlayingBeforeSeek = this.videoElement.isPlaying();
            this.pause();
        } 
        
        this.seeking = true;
        var playheadPosition = this.videoDuration * value;
        this.videoElement.setCurrentTime(playheadPosition);
        return this;
    },
    
    
    /**
     * Ends the seeking action
     * 
     *  @param controlBar           the control  bar sending the notification
     *  @param value                the current value to seek to
     */
    endSeeking : function(controlBar, value) {
        this.seeking = false;
        this.videoElement.setCurrentTime(this.videoDuration * value);
        this[(this.wasPlayingBeforeSeek)? "play" : "pause"]();
        
        return this;
    },
    
    
    /**
     * Resets the state of this controller
     */
    reset : function() {
        this.seeking = false;
        this.videoPlaying = false;
        this.videoBeganPlaying = false;
        this.videoChangingSize = false;
        this.videoEndedPlaying = false;
    },
    
    
    /**
     * Returns the current time of the underlying video for this controller
     * 
     * @return the current time of the underlying video for this controller
     */
    getCurrentTime : function() {
        return (this.videoElement.getCurrentTime());
    },
    
    
    /**
     * Sets the current time for the underlying video for this widget. This action causes the video to jump to the
     * new time and start or resume playing
     * 
     * @param time              the current time to set for the video
     */
    setCurrentTime : function(time) {
        this.videoElement.setCurrentTime(time);
        return this;
    },
    
    
    /**
     * Returns the duration of the current video for this widget
     * 
     * @return the duration of the current video for this widget
     */
    getDuration : function() {
        return this.videoElement.getDuration();
    },
    
    
    /**
     * Returns the current volume of the underlying video 
     * 
     * @return the current volume of the underlying video for this controller
     */
    getVolume : function() {
        return this.videoElement.getVolume();
    },
    
    
    /**
     * A callback method invoked when the user sets the volume by dragging the volume slider. It sets the current volume
     * 
     * @param volumeLevel               the current volume level (between 0 and 1)
     */
    onVolumeChanged : function(volumeLevel) {
        this.setVolume(volumeLevel);
    },
    
    
    /**
     * Sets the volume of the underlying video
     * 
     * @param volume            the volume to set for the video
     */
    setVolume : function(volume) {
        this.videoElement.setVolume(volume);
        return this;
    },
    
    
    /**
     * Mutes or unmutes the video for this controller
     * 
     * @param muted             a flag that specifies whether to mute or unmute the video
     */
    setMuted : function(muted) {
        this.videoElement.setMuted(muted);
        return this;
    },
    
    
    /**
     * Sets the playback rate for the underlying video
     * 
     * @param rate              the playback rate to set
     */
    setRate : function(rate) {
        this.videoElement.setPlaybackRate(rate);
        // Show the playback rate as an overlay
        return this;
    },
    
    
    /**
     * Captures a frame of the video
     * 
     * @param video             the video for which to capture the frame
     * @param width             the width of the video to capture
     * @param height            the height of the video to capture
     */
    captureFrame : function(video, width, height) {
        var canvas = document.createElement("canvas");
        canvas.width = width;
        canvas.height = height;
        var context = canvas.getContext("2d");
        canvas.drawImage(video, 0, 0, width, height);
    },
            
    
    /**
     * Destroys the current video player, cleans up its dependencies and removes its event handlers
     */
    destroy : function() {
        if(this.controlBar) {
            Concierge.Event.remove(this.videoContainer, "mousemove", this.handlers.handleMouseMovements);
            this.controlBar.destroy();
            this.controlBar = null;
        }
        
        if(this.playIcon) {
            Concierge.Event.remove(this.playIcon, "click", this.handlers.onPlayIconClicked);
        }
        
        this.videoElement = null;
    }
    
});


Concierge.Media.Player.defaultOptions = {
    videoType : Concierge.VideoType.HTML5,
    width : 640,
    height : 360,
    autoresize : false,
    autoplay : true,
    enableAirplay : false,
    poster : null,
    videoBeganPlaying : false,
    videoChangingSize : false,
    swfMovieURL : "LightweightVideoPlayer.swf"
};


/**---------------------------------------------------------------------------------------------------------------------
 *                                          VIDEO CONTROLS
 *                      
 *  A class that renders the control bar and associated elements to interact with a video. This class is used by the
 *  Concierge.Media.Controller and displayed over the video in its container. It's also hidden after a short delay in there
 *  no mouse activity within the boundaries of the video container, and shown when the mouse activity resumes.
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.Media.ControlBar = Concierge.Class.create({
    
    
    /**
     * Initializes this instance of the ControlBar component with the given options
     * 
     * @param options               the customization options for this component
     */
    initialize : function(options) {
        var defaultOptions = Object.clone(this.constructor.defaultOptions);
        this.options = Object.extend(defaultOptions, options || { });

        this.handlers = {
            handlePlaybackActions : this.handlePlaybackActions.bind(this),
            onDragStart : this.onDragStart.bind(this),
            onDragUpdate : this.onDragUpdate.bind(this),
            onDragEnd : this.onDragEnd.bind(this)
        };

        this.container = this.options.container;
        this.id = this.options.id || "akwaba-video-controls-" + Concierge.generateUID();
        var isValidContainer = (this.container && Object.isElement(this.container));

        if(!isValidContainer) {
            throw DOMException.INVALID_ELEMENT_ERR;
        }
        
        this.delegate = this.options.delegate;
        this.render();
    },
   
   
    /**
     * Renders the view of this ControlBar instance
     */
    render : function() {
        var proceed = (this.delegate && Concierge.Class.conformsToProtocol(this.delegate, this.constructor));
        
        if(!proceed) {
            var message = ["<p class=\"error-message\">Please provide a valid delegate for this MediaControl component.</p>",
                "<p class=\"error-message\">Must implement the following required methods: {methods}.</p>"].join("");

            var errorMessage = String.format(message, {
                methods : this.constructor.requiredMethods.join(", ")
            });
            
            throw {
                title : "IllegalState",
                message : errorMessage
            };
        }
        
        var DOM = Concierge.DOM,
        CSS = Concierge.CSS,
        template = this.options.templates.CONTENT,
        className = [this.options.className, this.options.theme].join(" ");

        this.element = DOM.createElement("div", {
            id : this.id,
            className : className
        });
        DOM.setContent(this.element, template);
        
        this.volumeTrack = CSS.selectFirst("div.volume-slider-track", this.element);
        this.volumeHandle = CSS.selectFirst("span.volume-slider-handle", this.element);
        this.volumeFiller = CSS.selectFirst("span.volume-filler", this.element);
        this.playControlsContainer = CSS.selectFirst("ul.play-controls", this.element);
        this.playPauseButton = CSS.selectFirst("a.play", this.element);
        
        this.fullscreenButton = CSS.selectFirst("a.fullscreen", this.element);
        this.elapsedContent = CSS.selectFirst("p.elapsed", this.element);
        this.remainingContent = CSS.selectFirst("p.remaining", this.element);
        this.bufferedContent = CSS.selectFirst("div.buffered", this.element);
        this.playedContent = CSS.selectFirst("div.played", this.element);
        
        this.progressBar = CSS.selectFirst("div.progress-slider-track", this.element);
        this.progressBarHandle = CSS.selectFirst("span.progress-slider-handle", this.element);
        this.percentageLoaded = 0;
        
        this.container.appendChild(this.element);
        this.toggleControls("disable").setVolumeLevel(1);
        DOM.hide(this.element);

        this.activateProgressBar().activateVolumeControl();
        this.enterNormalPlaybackMode().registerEvents();
    },
    
    
    /**
     * Enables or disables the view of this control bar element
     * 
     * @param visibility            a flag that specifies whether to enable or disable the view
     */
    toggleControls : function(visibility) {
        this.enabled = (visibility == "enable");
        Concierge.DOM[(this.enabled)? "removeClassName" : "addClassName"](this.element, "disabled");
        return this;
    },
    
    
    /**
     * Returns true if the view of this control bar is enabled; otherwise, returns false
     * 
     * @return true if the view of this control bar is enabled; otherwise, returns false
     */
    isEnabled : function() {
        return this.enabled;
    },
    
    
    /**
     * Activates the behavior of the progress bar used to scrub through the video
     */
    activateProgressBar : function() {
        Concierge.Event.add(this.progressBarHandle, "mousedown", this.handlers.onDragStart);
        return this;
    },
    
    
    /**
     * A callback method invoked when the user clicks down on the drag handle for the progress bar. It sets the type
     * of the drag (progress update or volume update) and initializes the coordinates of the mouse for further 
     * reference when the user moves the mouse
     * 
     * @param event             the mouse down event on the progress bar drag handle
     */
    onDragStart : function(event) {
        var Event = Concierge.Event,
        DOM = Concierge.DOM,
        source = Event.element(event);
        
        this.isProgressBarHandle = (DOM.hasClassName(source, "progress-slider-handle"));
        this.dragOriginX = Event.pointerX(event);
        
        if(this.isProgressBarHandle) {
            if(!this.isVideoSeekingMode()) {
                this.enterVideoSeekingMode();
            }

            this.containerOffset = DOM.cumulativeOffset(this.progressBar);
            this.elementInitialX = DOM.positionedOffset(this.progressBarHandle).left;
            this.seekableInterval = Math.floor(this.bufferedContent.offsetWidth - DOM.getWidth(this.progressBarHandle));
        } else {
            this.containerOffset = DOM.cumulativeOffset(this.volumeTrack);
            this.elementInitialX = DOM.positionedOffset(this.volumeHandle).left;
            this.seekableInterval = this.volumeTrack.offsetWidth - DOM.getWidth(this.volumeHandle);
        }
        
        Event.add(document, "mousemove", this.handlers.onDragUpdate);
        Event.add(document, "mouseup", this.handlers.onDragEnd);
    },
    
    
    /**
     * Updates the playhead of the movie or the current volume as the user moves the move
     * 
     * @param event                 the "mouse move" event fired as the user moves the mouse
     */
    onDragUpdate : function(event) {
        var Event = Concierge.Event,
        DOM = Concierge.DOM,
        offsetLeft = Event.pointerX(event) - this.dragOriginX,
        positionX = Math.max(0, Math.min(this.elementInitialX + offsetLeft, this.seekableInterval));
            
        if(this.isProgressBarHandle) {
            DOM.setStyle(this.progressBarHandle, {
                left : positionX + "px"
            });
            
            this.playheadOffsetPercentage = DOM.positionedOffset(this.progressBarHandle).left / DOM.getWidth(this.progressBar);
            this.fillElapsedDurationAtCurrentPlayhead();
            
            this.delegate.beginSeeking(this, this.playheadOffsetPercentage);
        
        } else {
            DOM.setStyle(this.volumeHandle, {
                left : positionX + "px"
            });
            
            var volumeLevel = DOM.positionedOffset(this.volumeHandle).left / this.seekableInterval;
            this.delegate.onVolumeChanged(volumeLevel);
        }
    },
    
    
    /**
     * A callback method invoked when the user releases the mouse. It ends the video seek or the volume update and 
     * removes the events observed during the drag
     * 
     * @param event                 the "mouse up" event fired when the user releases the mouse
     */
    onDragEnd : function(event) {
        if(this.isProgressBarHandle) {
            this.leaveVideoSeekingMode();
            this.delegate.endSeeking(this, this.playheadOffsetPercentage);
        }
        
        Concierge.Event.remove(document, "mousemove", this.handlers.onDragUpdate);
        Concierge.Event.remove(document, "mouseup", this.handlers.onDragEnd);
    },
    
    
    /**
     * Fills the elapsed duration at the current playhead, which is represented by the position of the progress handle. 
     * This method is invoked during normal playhead as well as when the user starts dragging the progress handle
     */
    fillElapsedDurationAtCurrentPlayhead : function() {
        var DOM = Concierge.DOM,
        pixelOffset = DOM.positionedOffset(this.progressBarHandle).left + DOM.getWidth(this.progressBarHandle) / 2,
        elapsedPercentage = (pixelOffset * 100 / DOM.getWidth(this.progressBar)).toFixed(2);
            
        DOM.setStyle(this.playedContent, {
            width : elapsedPercentage + "%"
        });
    },
    
    
    /**
     * Activates the volume control used to set the current volume
     */
    activateVolumeControl : function() {
        Concierge.Event.add(this.volumeHandle, "mousedown", this.handlers.onDragStart);
        return this;
    },
    
    
    /** 
     * Sets the volume level to the specified value
     * 
     * @param volumeLevel               the volume level to set
     */
    setVolumeLevel : function(volumeLevel) {
        var DOM = Concierge.DOM,
        handleWidth = DOM.getWidth(this.volumeHandle),
        pixelOffset = Math.floor((DOM.getWidth(this.volumeTrack) - handleWidth) * volumeLevel);
        
        DOM.setStyle(this.volumeHandle, {
            left : pixelOffset + "px"
        });

        return this;
    },
    
    
    /**
     * Registers event handlers for the actions of this control bar: play controls, volume control, fullscreen action, etc
     */
    registerEvents : function() {
        Concierge.Event.add(this.playControlsContainer, "click", this.handlers.handlePlaybackActions);
    },
    
    
    /**
     * Updates the elapsed duration and the remaining duration of the movie
     * 
     * @param duration                  the duration of the movie
     */
    onDurationAvailable : function(duration) {
        this.videoDuration = duration;
        
        var remainingTime = Concierge.MediaUtils.formatRemainingTime(duration);
        Concierge.DOM.setContent(this.elapsedContent, "00:00");
        Concierge.DOM.setContent(this.remainingContent, remainingTime);
        
        if(!this.isEnabled()) {
            this.toggleControls("enable");
        }
    },
    
    
    /**
     * A callback method invoked when the user interacts with the play controls. It takes the appropriate action based
     * on the button that was clicked. No action is taken if the controls are not yet enabled.
     * 
     * @param event             the click event on the play control element
     */
    handlePlaybackActions : function(event) {
        if(!this.isEnabled()) {
            return;
        }
        
        var Event = Concierge.Event,
        DOM = Concierge.DOM;
        
        Event.stop(event);
        var source = Event.element(event),
        tagName = source.tagName.toUpperCase();
        
        switch(tagName) {
            
            case "A":
                var isPlayButton = DOM.hasClassName(source, "play"),
                isRewindButton = DOM.hasClassName(source, "rewind"),
                isFastForwardbutton = DOM.hasClassName(source, "fast-forward"),
                action;
                
                if(isPlayButton) {
                    var isPlaying = DOM.hasClassName(source, "pause");
                    DOM[(isPlaying)? "removeClassName" : "addClassName"](source, "pause");
                    action = (isPlaying)? "pause" : "play";
                } else {
                    action = (isRewindButton)? "rewind" : "fast-forward";
                }
                
                this.delegate.onPlaybackActions({
                    action : action
                });
                
                break;
                
            default:
                break;
            
        }
    },
    
    
    /**
     * A callback method invoekd when the state of the video playback changes. It updates the UI to reflect the current
     * state by showing either the "Play" or "Pause" buttons
     * 
     * @param action                the action that specifies the current state of the video playback
     */
    onPlaybackStateChanged : function(action) {
        var isPlaying = (action == "play");
        Concierge.DOM[(isPlaying)? "addClassName" : "removeClassName"](this.playPauseButton, "pause");
        return this;
    },
    
    
    /**
     * Resets the state of the progress bar and playhead indicator when playback ends
     */
    onPlaybackEnded : function() {
        this.onPlaybackStateChanged("pause");
        this.onProgress(0).onTimeUpdate(0);
        
        return this;
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the playback time of the current Video is updated, It updates
     * the elapsed and remaining durations of the video, updates the position of the playhead and fills the elapsed
     * duration of the movie
     * 
     * @param elapsed               the elapsed duration of the movie
     */ 
    onTimeUpdate : function(elapsed) {
        if(!this.videoDuration) {
            return this;
        }
        
        var DOM = Concierge.DOM,
        elapsedTime = Concierge.MediaUtils.formatTime(elapsed),
        remainingTime = Concierge.MediaUtils.formatRemainingTime(this.videoDuration - elapsed);
        DOM.setContent(this.elapsedContent, elapsedTime);
        DOM.setContent(this.remainingContent, remainingTime);
        
        if(this.isNormalPlaybackMode()) {
            var percentagePlayed = elapsed / this.videoDuration,
            pixelValue = Math.floor(DOM.getWidth(this.progressBar) * percentagePlayed),
            handleOffset = Math.max(0, pixelValue - DOM.getWidth(this.progressBarHandle));
            
            DOM.setStyle(this.progressBarHandle, {
                left : handleOffset + "px"
            });
            
            this.fillElapsedDurationAtCurrentPlayhead();
        }
        
        return this;
    },
    
    
    /**
     * A callback method that is invoked repeatedly when the download progress of the video is updated. It updates the
     * width of the buffered content indicator
     * 
     * @param loaded                the percentage of data currently loaded
     * @param duration              the total duration of the video
     */
    onProgress : function(loaded) {
        this.percentageLoaded = loaded;
        Concierge.DOM.setStyle(this.bufferedContent, {
            width : loaded + "%"
        });
        
        return this;
    },
    
    
    /**
     * Enters normal playback mode
     */
    enterNormalPlaybackMode : function() {
        this.mode = "playback";
        return this;
    },
    
    
    /**
     * Returns true if this control bar is in normal playback mode; otherwise, returns false
     * 
     * @return true if this control bar is in normal playback mode; otherwise, returns false
     */
    isNormalPlaybackMode : function() {
        return (this.mode == "playback");
    },
    
    
    /**
     * Enters video seeking mode
     */
    enterVideoSeekingMode : function() {
        this.mode = "seek";
        return this;
    },
    
    
    /**
     * Returns true if this control bar is in video seeking mode; otherwise, returns false
     * 
     * @return true if this control bar is in video seeking mode; otherwise, returns false
     */
    isVideoSeekingMode : function() {
        return (this.mode == "seek");
    },
    
    
    /**
     * Leaves video seeking mode and enters the normal mode
     */
    leaveVideoSeekingMode : function() {
        this.enterNormalPlaybackMode();
        return this;
    },
    
    
    /**
     * Shows the control panel
     */
    show : function() {
        Concierge.DOM.show(this.element).setOpacity(this.element, 1);
        return this;
    },
    
    
    /**
     * Hides the control panel by fading it
     */
    hide : function() {
        if(this.effectInProgress) {
            return;
        }
        
        this.effectInProgress = true;
        var controlBar = this;
        
        Concierge.Effect.fade(this.element, {
            duration : 0.25,
            after : function() {
                Concierge.DOM.hide(controlBar.element);
                controlBar.effectInProgress = false;
            }
        });
    },
    
    
    /**
     * Returns true if the control panel is showing; otherwise, returns false
     * 
     * @return true if the control panel is showing; otherwise, returns false
     */
    isShowing : function() {
        return this.element.visible();
    },
    
    
    /**
     * Resets the state of the view elements for this control bar component
     */
    resetView : function() {
        this.onPlaybackEnded();
        
        Concierge.DOM.setContent(this.elapsedContent, "--:--");
        Concierge.DOM.setContent(this.remainingContent, "--:--");
        Concierge.DOM.hide(this.element);
        
        return this;
    },
      
    
    /**
     * Destroys this control bar component and removes its event handlers
     */
    destroy : function() {
        Concierge.Event.remove(this.progressBarHandle, "mousedown", this.handlers.onDragStart);
        Concierge.Event.remove(this.volumeHandle, "mousedown", this.handlers.onDragStart);
        Concierge.Event.remove(this.playControlsContainer, "click", this.handlers.handlePlaybackActions);
    }
    
});

Object.extend(Concierge.Media.ControlBar, {
   requiredMethods : ["onPlaybackActions", "beginSeeking", "endSeeking", "onVolumeChanged"],
    defaultOptions : {
        className : "block akwaba-video-controls",
        theme : "itunes",
        templates : {
       
            CONTENT : ['<div class="block control-section">',
                '<div class="volume-container">',
                '<p class="volume-min">&nbsp;</p>',
                '<div class="volume-slider-track">',
                '<span class="volume-filler">&nbsp;</span>',
                '<span class="volume-slider-handle">&nbsp;</span>',
                '</div><p class="volume-max">&nbsp;</p></div>',
                '<div class="play-section"><ul class="block play-controls">',
                '<li><a id="akwaba-video-controls-rewind" href="#rewind" rel="rewind" class="rewind">Rewind</a></li>',
                '<li><a href="#play" rel="play" class="play">Play</a></li>',
                '<li><a href="#fast-forward" rel="fast-forward" class="fast-forward">Forward</a></li>',
                '</ul></div>',
                '<div class="fullscreen-section"><a href="#fullscreen" class="fullscreen" rel="fullscreen">fullscreen</a></div>',
                '</div>',
                '<div class="block progress-section">',
                '<p class="elapsed">--:--</p>',
                '<div class="progress-bar">',
                '<div class="buffered">&nbsp;</div>',
                '<div class="played">&nbsp;</div>',
                '<div id="akwaba-media-sliderTrack" class="progress-slider-track">',
                '<span id="akwaba-media-sliderHandle" class="progress-slider-handle">&nbsp;</span>',
                '</div></div>',
                '<p class="remaining">--:--</p>',
                '</div>'
           ].join("")
        }
    }
});


/**---------------------------------------------------------------------------------------------------------------------
 *                                          MEDIA UTILS
 *                      
 *  A collection of utility methods used by the different components of the Media Player framework
 *--------------------------------------------------------------------------------------------------------------------*/

Concierge.MediaUtils = {

    /**
     * Returns true if the browser has the required major and minor versions of the Flash player (in this case, 10.1 
     * or higher); otherwise, returns false
     *
     * @param flashVersionMajor         the major version of Flash Player
     * @param flashVersionMinor         the minor version of Flash Player
     * @return true if the browser has the required major and minor versions of the Flash player (10.1 or higher); 
     * otherwise, returns false
     */
    hasRequiredFlashVersion : function(flashVersionMajor, flashVersionMinor) {
        var flashVersion = swfobject.getFlashPlayerVersion();
        return ((flashVersion.major > flashVersionMajor) || 
            ((flashVersion.major == flashVersionMajor) && (flashVersion.minor >= flashVersionMinor)));
    },
    
    
    /**
     * Formats the given time in the "h:mm:ss" format. The formatted string is used to display the time in the UI.
     * 
     * @param time              the time to format, in seconds
     * @return a formatted string for the given time
     */
    formatTime : function(time) {
        var ONE_HOUR = 60 * 60,
        displayHours = (time >= ONE_HOUR),
        minutes = Math.floor(time / 60),
        hours;

        if(displayHours) {
            hours = Math.floor(time / ONE_HOUR)
        }

        var seconds = Math.floor(time % 60);
        var template = (displayHours)? "{hours}:{minutes}:{seconds}" : "{minutes}:{seconds}",
        formattedTime = String.format(template, {
            hours : hours,
            minutes : (minutes < 10)? "0" + minutes : minutes,
            seconds : (seconds < 10)? "0" + seconds : seconds
        });

        return formattedTime;
    },
    
    
    /**
     * Formats the remainig time in the "-h:mm:ss" format. The formatted string is used to display the remaining time
     * of a video
     * 
     * @param time              the remainig time to format, in seconds
     * @return a formatted string for the given remainig time
     */
    formatRemainingTime : function(time) {
        var formattedTime = this.formatTime(time);
        return ["-", formattedTime].join("");
    }
  
};