/**---------------------------------------------------------------------------------------------------------------------
                                            CONCIERGE UI

     A collection of UI widgets used by the Workshop application. Each widget implements a specific user interaction, 
and collaborates with other widgets to accomplish a task.
All widgets communicate through a custom event model (publish/subscribe), which they all inherit from the 
{@code Concierge.WidgetBase} module
----------------------------------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------------------------------------
                            LIST VIEW

A widget that a displays data in a list view. Each view is formatted based on either the default
format (a header and a description text) or a custom format specified by the delegate of the
instance. The implementation also allows the addition or removal of new item as well as the
redrawing of the entire view when the data changes

This implementation uses the default delegate for its basic behavior. So it's recommended to provide
a custom delegate, an instance of {@code Concierge.UI.ListViewDelegate} that will override the respective
methods to provide the expected customization and behaviors
----------------------------------------------------------------------------------------------------------------------*/

Concierge.UI.ListView = Concierge.Class.create(Concierge.WidgetBase, {

    /**
     * Initializes the widget with the options specified
     *
     * @param options           the widget configuration options
     */
    initialize : function(options) {
        this.init(options);
        this.delegate = this.options.delegate;
        
        this.handlers = {
            onItemsClicked : this.onItemsClicked.bind(this)
        };
        
        this.render();
    },


    /**
     * Renders and initializes the view and view elements of the list view widget. If a style is provided,
     * the list container is initialized with that style. Otherwise, it uses the "item-list" class name.
     * The list view also requests its data source from the delegate and renders the list of items
     * accordingly
     *
     * This method displays a descriptive error message if the delegate doesn't implements all the required
     * methods as defined by the ListView constructor
     */
    render : function() {
        this.itemContainer = $(this.options.itemContainer) || $("<ul></ul>").addClass(this.options.className);
        this.itemContainer.appendTo(this.element);
        
        if(!(this.delegate && Concierge.Class.conformsToProtocol(this.delegate, this.constructor))) {
            var message = ["<p class=\"error-message\">Please provide a valid delegate for this ListView instance.</p>",
                "<p class=\"error-message\">Must implement the following required methods: #{methods}.</p>"].join("");

            var errorContainer = $("<li></li>").html(String.format(message, {
                methods : this.constructor.requiredMethods.join(", ")
            }));
            errorContainer.appendTo(this.itemContainer);
            
            return this;
        }


        this.dataSource = this.delegate.getDataSource();

        if(this.dataSource) {
            this.renderItems();
        }

        return this.registerEvents();
    },


    /**
     * Renders the view for the items in this list. The array of items is retrieved from the delegate class
     */
    renderItems : function() {
        if(!Object.isArray(this.dataSource)) {
            return;
        }

        if(this.dataSource.size() == 0) {
            this.displayEmptyContentMessage();
        } else {
            this.dataSource.forEach(function(item) {
                this.addItemNode(item);
            }, this);
        }

        var viewReloaded = this.reloadingData;
        this.reloadingData = false;
        this.delegate.afterViewRendered.call(this.delegate, this.itemContainer, viewReloaded);
    },


    /**
     * Reloads the data and re-renders the updated view of this list. Since this is a reload of the data, no event
     * is fired once the update is complete
     */
    reloadData : function() {
        this.reloadingData = true;
        $(this.itemContainer).children().remove();
        
        this.template = null;
        this.dataSource = this.delegate.getDataSource();
        this.renderItems();
    },
    
    
    /**
     * Displays a default message when the current list view doesn't have any items to render
     */
    displayEmptyContentMessage : function() {
        $C.log("Displaying empty content...");
        $("<li></li>").addClass("no-items").html("<p>" + this.options.emptyContentMessage + "</p>").appendTo(this.itemContainer);
    },


    /**
     * Creates a DOM element for the item specified, and adds it to the parent list container node. If the
     * item has a "disabled" property, the list is rendered in a disabled state to reflect that property
     *
     * @param item                 the item to create the DOM node for
     * @param options               an object that contains HTML properties for the DOM element to create
     */
    addItemNode : function(item) {
        var template = this.template || (this.template = this.delegate.getTemplate()),
        itemClass = (item.className || "block"),
        htmlContent = String.format(template, item);
        
        var container = $("<li></li>").addClass(itemClass).html(htmlContent).hide().appendTo(this.itemContainer);
        container.id = item.id || "list-" + Concierge.generateUID();
        
        if(this.itemContainer.hasClass("empty-list")) {
            this.itemContainer.removeClass("empty-list");
        }
            
        if(item.disabled) {
            container.addClass("disabled");
        }
        
        container.show();
        
        return container;
    },


    /**
     * Registers event handlers on for the list view container. The main event consists of a "click" event on
     * the parent container, which is a bubbled event fired by the child elements
     */
    registerEvents : function() {
        this.itemContainer.click(this.handlers.onItemsClicked);
        return this;
    },


    /**
     * This method gets invoked when the user clicks on a list item. The event is stopped and forwarded to
     * the delegate's {@code onItemClicked} callback for further processing
     *
     * @param event             the click event on the list items
     */
    onItemsClicked : function(event) {
        var eventStopped = !!this.delegate.onItemClicked.call(this.delegate, event);
        
        if(!eventStopped) {
            event.preventDefault();
        }
    },


    /**
     * Adds the specified item to this items list.
     *
     * @param item              the item to add to the list
     * @param options           the additional options that specify whether to animate the appearance
     *                          of the element
     */
    addItem : function(item, options) {
        var emptyContainer = this.itemContainer.find("li.no-items");
        
        if(emptyContainer) {
            emptyContainer.remove();
        }
        
        var node = this.addItemNode(item, options);
        this.delegate.afterItemAdded.call(this.delegate, node);

        return this;
    },


    /**
     * Removes the item at the index specified. This method simply returns if no such item exists at that
     * index
     *
     * @param index             the index of the item to remove
     */
    removeItemAtIndex : function(index) {
        var nodeToRemove = this.itemContainer.find("li:nth-child(" + index + ")"),
        delegate = this.delegate;

        if(!nodeToRemove) {
            return this;
        }

        nodeToRemove.remove();
        this.dataSource.splice(index, 1);
        
        (delegate.afterItemRemoved || Function.empty).call(delegate, nodeToRemove);
        return this;
    }


});


Object.extend(Concierge.UI.ListView, {
    requiredMethods : ["getDataSource", "getTemplate", "afterViewRendered", "afterItemAdded", "onItemClicked"],
    defaultOptions : {
        element : "itemContainer",
        style : "default",
        className : "block item-list",
        highlightAlternateRows : false,
        emptyContentMessage : "You currently don't have any items"
    },

    getInstance : Concierge.WidgetBase.getInstance
});




/*----------------------------------------------------------------------------------------------------------
                                        CALENDAR

A widget that a displays a calendar view starting at the current date (day, month, year). The view displays
controls for navigation forward to upcoming months or backwards to previous months.

This implementation also notifies the delegate of the selected date when the user clicks on a given date.
----------------------------------------------------------------------------------------------------------*/

Concierge.UI.Calendar = Concierge.Class.create(Concierge.WidgetBase, {

    /**
     * Initializes the calendar widget with the options specified
     *
     * @param options           the widget configuration options
     */
    initialize : function(options) {
        this.init(options);
        this.delegate = this.options.delegate;
        
        this.viewContainers = { };
        this.handlers = {
        };

        var now = Date.now();
        this.currentDate = now;
        
        this.components = {
            day : now.getDate(),
            month : now.getMonth(),
            year : now.getFullYear(),
            hours : now.getHours(),
            minutes : now.getMinutes(),
            seconds : now.getSeconds()
        };
        
        $C.log("Calendar.initialize - components: %s", this.components);
    },


    /**
     * Renders and initializes the calendar view with the current date and "default" mode. If a class name is
     * provided, the calendar is initialized with that class name.
     *
     * This method displays a descriptive error message if the delegate doesn't implements all the required
     * methods as defined by the Calendar constructor
     *
    render : function() {
        if(!(this.delegate && Class.conformsToProtocol(this.delegate, this.constructor))) {
            var message = ["<p class=\"error-message\">Please provide a valid delegate for this Calendar instance.</p>",
                "<p class=\"error-message\">Must implement the following required methods: #{methods}.</p>"].join("");

            var errorContainer = new Element("li").update(message.interpolate({
                methods : this.constructor.requiredMethods.join(", ")
            }));
            this.element.appendChild(errorContainer);

            return this;
        }

        this.model = {
            timeInMillis : new Date().getTime(),
            currentDay : new Date().getDate(),
            month : new Date().getMonth(),
            currentYear : new Date().getFullYear()
        };
        
        this.currentDate = new Date(this.model.timeInMillis);
        this.element.className = this.options.className;
        this.element.addClassName("calendar-" + (this.options.mode || "standard"));
        
        this.drawCalendar().registerEvents();
        
        if(this.isDayviewMode()) {
            this.element.addClassName("day-view");
            this.showLocalizedDate(this.currentDate).sendDateChangeNotification(this.currentDate);
        }
    },


    setMode : function(mode) {
        this.currentMode = mode;
    },
    
    
    renderInView : function(container, mode) {
        var containerID = (container.id || ("viewContainer-" + Concierge.generateUID())),
        alreadyRendered = (this.viewContainers[container] != null);
        
        if(alreadyRendered) {
            return this;
        }
        
        var drawTableCalendar = (mode == Concierge.UI.Calendar.type.TABLE);
        
        if(drawTableCalendar) {
            this.drawTableCalendar(container);
            this.viewContainers[containerID] = {
                containerID : containerID,
                calendarType : mode
            };
        }
    },
    
    
    /**
     * Renders the days in the calendar, and highlights the current date
     *
    drawTableCalendar : function(container) {
        var model = this.model,
        currentDate = new Date(model.currentYear, model.month, model.currentDay);
        
        var tableTemplate = ['<table class="calendar calendar-month">',
            '<thead><tr><th class="current-month" colspan="7">', currentDate.getMonthName(), ' ', currentDate.getFullYear(), '</th>',
            '</tr></thead><tbody></tbody>'].join("");
        $(tableTemplate).appendTo(container);
        
        var calendarDays = this.generateCalendarDays();
        
        calendarDays.forEach(function(element) {
           $("tbody", container).append(element);
        });
    },
    
    
    /**
     * Generates the days for the current calendar and returns the content as an array of table rows to be appended to
     * the calendar container
     * 
     * @return the calendar days as an array of table rows
     *
    generateCalendarDays : function() {
        var daysInMonth = this.currentDate.getDaysInMonth(),
        firstDayOfMonth = this.currentDate.getFirstDayOfMonth(),
        model = this.model;
        
        var previousMonth = true;
        var x = 0, currentDay = 0, rows = [ ],
        currentIndex = 0;
        
        var now = new Date(), today = now.getDate(), thisMonth = now.getMonth(),
        thisYear = now.getFullYear();

        while (currentDay <= daysInMonth) {
            var display, className = undefined;
            
            if (previousMonth) {
                rows[currentIndex] = $("<tr></tr>");

                if (firstDayOfMonth > 0) {
                    while (x < firstDayOfMonth) {
                        var weekday = Date.SHORT_WEEKDAYS[x];
                        $("<td></td>").html("<span>" + weekday + "<span>").appendTo(rows[currentIndex]);
                        x++;
                    }
                }
                
                previousMonth = false;
                currentDay = 1;
            }

            if (x % 7 == 0) {
                currentIndex++;
                rows[currentIndex] = $("<tr></tr>");
            }

            var isToday = ((currentDay == today) && (model.currentYear == thisYear) && (model.month == thisMonth)),
            currentWeekDay = Date.SHORT_WEEKDAYS[x % 7];

            if(isToday) {
               className = "today";
               display = ["<p><em>Today</em><span>", ((currentIndex == 0)? currentWeekDay + " " : ""), currentDay, "</span></p>"].join("");
            } else {
                display = ["<span>", ((currentIndex == 0)? currentWeekDay + " " : ""), currentDay, "</span>"].join("");
            }

            var td = $("<td></td>").html(display);
            
            if(!!className) {
                td.addClass(className);
            }
            
            td.appendTo(rows[currentIndex]);
            x++;
            currentDay++;
        }

        var lastRow = rows.last(),
        padding = 7 - $("td", lastRow).size();
        
        if(padding > 0) {
            (padding).times(function(index) {
               $("<td><span></span></td>").appendTo(lastRow); 
            });
        }
        
        return rows;
    },*/
    
    gotoPreviousDay : function() {
        
        
    },
    
    
    gotoNextDay : function() {
        
    },
    
    
    gotoPreviousMonth : function() {
        this.components.month--;
        
        if(this.components.month < 0) {
            this.components.month = 11;
            this.components.currentYear--;
        }

        $C.log("Going to previous month: %s", this.components.month);
    },
    
    
    gotoNextMonth : function() {
        this.components.month++;
            if(this.components.month > 11) {
                this.components.month = 0;
                this.components.currentYear++;
            }
        $C.log("Going to next month: %s", this.components.month);
    },
    
    
    
    /**
     * Navigates to the given day in the current year and month
     * 
     * @param year              the current calendar year
     * @param month             the current calendar month
     * @param day               the current calendar day
     */
    navigateToCalendarDay : function(year, month, day) {
        var currentDate = this.currentDate,
        proceed = ((year == currentDate.getFullYear()) && (month == currentDate.getMonth()));
        
        if(proceed) {
            var offset = (day - this.currentDate.getDate()), 
            direction = (offset > 0)? "next" : "previous",
            ONE_DAY = 24 * 60 * 60 * 1000;
            $C.log("Navigating to calendar day...");
        }
    },
    

    /**
     * This method gets invoked when the user clicks on a calendar day. The corresponding Date object is
     * constructed and the invocation is forwarded to the delegate's {@code onDateSelected} mthod for
     * further processing
     *
     * @param event             the click event on the date container
     */
    onDateSelected : function(event) {
        if(!event.stopped) {
            event.stop();
        }

        var source = event.element(), model = this.components,
        selectedDate = new Date(model.currentYear, model.month, parseInt(source.innerHTML));
        model.timeInMillis = selectedDate.getTime();
        
        if(this.activeDate) {
            this.activeDate.removeClassName("active-date");
        }
        
        source.addClassName("active-date");
        this.activeDate = source;
        this.sendDateChangeNotification(selectedDate);
        
        return this;
    },

    
    /**
     * Returns the date components of this calendar
     * 
     * @return the date components of this calendar
     */
    getDateTimeComponents : function() {
        return this.components;
    },
    
    
    /**
     * Returns the current month of the calendar
     *
     * @return the current month of the calendar
     */
    getCalendarCurrentMonth : function() {
        return this.components.month;
    },


    /**
     * Returns the current year of the calendar
     *
     * @return the current year of the calendar
     */
    getCalendarCurrentYear : function() {
        return this.components.currentYear;
    },
    
    
    /**
     * Returns the current date in milliseconds
     * 
     * @return the current date in milliseconds
     */
    getTimeInMillis : function() {
        var components = this.components;
        return new Date(components.year, components.month, components.day, components.hours,
            components.minutes, components.seconds).getTime();
    }

});


Object.extend(Concierge.UI.Calendar, {
    requiredMethods : [ ],
    type : {
        TABLE : "month",
        LIST : "list",
        POP_OVER : "popover"
    },
    defaultOptions : {
        element : "calendarContainer",
        mode : "default",
        className : "calendar-default",
        templates : { }
    },

    getInstance : Concierge.WidgetBase.getInstance
});


