/*----------------------------------------------------------------------------------------------------------------------
                                                CONCIERGE.JS

This module defines the core methods used by the WorkshopUI application. It extends the core Javascript objects with 
the recommended implementations (i.e. new Array methods defined in Javascrip 1.6) if they're missing, and adds
convenience ones for easy manipulation of the core objects (Array, Function, String, etc).

The module also defines an Event module that provides event handling in a cross-browser fashion. This module is mainly
used for the creation of custom events.

It finally defines a class module that allows for the creation of most class inheritance systems

Author: Daniel Ziaoure
(c) 2013 Akwaba Systems, Inc.
----------------------------------------------------------------------------------------------------------------------*/


/**
 * Merges the properties of the source object with those of the destination object
 *
 * @param destination           the destination object whose properties to augment
 * @param source                the source object whose properties to merge
 * @return the destination object
 */
Object.extend = function(destination, source) {
    for(var prop in source) {
        if(source.hasOwnProperty(prop)) {
            destination[prop] = source[prop];
        }
    }
    return destination;
};

(function() {

    /**
     * Returns the native type of an object: "Function" for a function, "Array"
     * for an array, etc
     */
    function getClass(object) {
        var classRegExp = /\[object (\w+)\]/,
        type = Object.prototype.toString.call(object),
        typeMatch = type.match(classRegExp);

        return typeMatch[1];
    }

    Object.extend(Object, {

        /**
         * Performs a deep copy of the properties of the source object into those of the destination object. If the 
         * property is another object, the function recursively invokes itself to ensure that the structure is copied 
         * as well. It also makes sure to not copy DOM elements
         *
         * @param destination           the destination object to augment
         * @param source                the source object to copy the properties from
         * @return the destination object
         */
        deepExtend : function(destination, source) {
            for(var prop in source) {
                if(source[prop] && source[prop].constructor
                    && source[prop].constructor === Object) {
                    destination[prop] = destination[prop] || { };
                    arguments.callee(destination[prop], source[prop]);
                } else {
                    destination[prop] = source[prop];
                }
            }

            return destination;
        },


        /**
         * Returns an array containing the properties of the specified object
         *
         * @param object            the object to return the properties for
         * @return an array containing the properties of this object
         */
        keys : function(object) {
            var keys = [];
            
            for(var prop in object) {
                keys.push(prop);
            }

            return keys;
        },


        /**
         * Returns an array containing the values for the different properties of the object specified
         *
         * @param object            the object to return the values for
         * @return an array containing the values of this object
         */
        values : function(object) {
            var values = [];
            
            for(var prop in object) {
                values.push(object[prop]);
            }

            return values;
        },


        /**
         * Clones the specified object
         *
         * @param object            the object to clone
         * @return a clone of the specified object
         */
        clone : function(object) {
            return Object.extend({ }, object);
        },


        /**
         * Returns true if the object specified is a DOM element (NodeType = 1); otherwise, returns false
         *
         * @param object            the object to check
         * @return true if the specified object is a DOM element; otherwise, returns false
         */
        isElement : function(object) {
            return !!(object && object.nodeType == 1);
        },


        /**
         * Returns true if this object is of type "undefined".
         *
         * @param object            the object to check
         * @return true if the specified object is of type undefined; otherwise, returns false
         */
        isUndefined : function(object) {
            return (object === undefined);
        },


        /**
         * Returns true if this object is defined; otherwise, returns false
         *
         * @param object            the object to check
         * @return true if the object is defined; otherwise, returns false
         */
        isDefined : function(object) {
            return !(Object.isUndefined(object));
        },


        /**
         * Returns true if the object is a function; otherwise, returns false
         *
         * @param object            the object to check
         * @return true if the object is a function; otherwise, returns false
         */
        isFunction : function(object) {
            return getClass(object) === "Function";
        },


        /**
         * Returns true if the object specified is a string; otherwise, returns false
         *
         * @param object            the object to check
         * @return true if the object is a string; otherwise, returns false
         */
        isString : function(object) {
            return getClass(object) === "String";
        },


        /**
         * Returns true if the specified object is a number; false otherwise
         *
         * @param object            the object to check
         * @return true if the object is a number; false otherwise
         */
        isNumber : function(object) {
            return getClass(object) === "Number";
        },


        /**
         * Returns true if the specified object is an array; otherwise, returns false
         *
         * @param object            the object to check
         * @return true if the specified object is an array; otherwise, returns false
         */
        isArray : function(object) {
            return getClass(object) === "Array";
        },


        /**
         * Returns a JSON representation of the object.
         *
         * @param object            the object to return the JSON representation of
         * @return a JSON representation of the object
         */
        //toJSONString : toJSONString,


        /**
         * Parses and returns a JSON string. This implementation removes all script tags from the string
         * prior to parsing the resulting content.
         *
         * @param json            the JSON string to parse
         * @return a JSON object from the JSON string
         */
        evalJSON : function(json) {
            json = String.removeScripts(json);
            var response;

            try {
                
                response = eval('(' + json + ')');
            
            } catch(exception) {
                response = DOMException.JSON_FORMAT_ERR;
            }

            return response;
        },


        /**
         * Converts the keys and values of an object into a URL-encoded string to use for submitting requests
         * to the server. If the value of a key is null or undefined, it's coerced to the equivalent string value
         * ("null" or "undefined"). If the value is an array, each item of the array is added under the same key
         * (i.e. days=mon&days=tue&days=wed).
         * This implementation uses Javascript's native "encodeURIComponent" function for encoding the strings
         *
         * @param object                the object whose properties to convert to a query parameter
         * @return a URL-encoded string of the properties of the object specified
         */
        toQueryParameters : function(object) {
            function toKeyValuePair(key, value) {
                if(value === undefined) {
                    return key;
                }

                return key + "=" + encodeURIComponent(String(value));
            }

            var parameters = [ ];
            Object.keys(object).forEach(function(key) {
                if(Object.isArray(object[key])) {
                    var values = object[key].map(toKeyValuePair.curry(key)).join("&");
                    parameters.push(values);
                } else {
                    parameters.push(toKeyValuePair(key, object[key]));
                }
            });

            return parameters.join("&");
        },


        /**
         * Returns a debug-oriented string representation of the object specified
         *
         * @param object            the object to inspect
         * @return a string representation of the object specified
         */
        inspect : function(object) {
            try {
                if(Object.isUndefined(object)) {
                    return "undefined";
                }

                if(object === null) {
                    return "null";
                }

                return String(object);

            } catch(exception) {
                throw exception;
            }
        }
    });

})();



/*----------------------------------------------------------------------------------------------------------------------
 *                                  ARRAY EXTENSION METHODS
 *
 * These methods provide easy and handy array manipulations. The logic first checks for any native Javascript 1.6 
 * implementations of the methods specified and only adds them if not.
 *
 ---------------------------------------------------------------------------------------------------------------------*/

(function() {

	var extensions = {

        /**
         * Array#forEach(iterator[,context])  --> void
         * 
         * Invokes the specified iterator function on each item of this array. The function is invoked with the value
         * of the item as well as its index. If the context is provided, it will execute in that context
         *
         * @param iterator          the iterator method to invoke on each item of the array
         * @param context           the context for the "this" keyword
         *
         * Reference:
         * https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/forEach
         */
		forEach : function(iterator, context) {
			if(!Object.isFunction(iterator)) {
				iterator = Function.empty;
			}

			for(var i = 0, len = this.length; i < len; i++) {
				iterator(this[i], i);
			}
		},


        /**
         * Array#first() --> Object
         * 
         * Returns the first item of this array
         *
         * @return the first item of this array
         */
		first : function() {
			return this[0];
		},


        /**
         * Array#last() --> Object
         * 
         * Returns the last item of this array
         *
         * @return the last item of this array
         */
		last : function() {
			return this[this.length - 1];
		},


        /**
         * Array#min()  --> Object
         * 
         * Returns the smallest item of this array. It applies the natural order comparison algorithm of the Math object
         *
         * @return the smallest item of this array
         */
        min : function() {
            return Math.min.apply(Math, this);
        },


        /**
         * Array#max()  --> Object
         * 
         * Returns the largest item of this array. It applies the natural order comparison algorithm of
         * the Math object
         *
         * @return the largest item of this array
         */
        max : function() {
            return Math.max.apply(Math, this);
        },


        /**
         * Array#clone()  --> Array
         * 
         * Clones and returns the cloned version of this array, leaving the original one intact
         *
         * @return a cloned version of this array
         */
        clone : function() {
            return Array.prototype.slice.apply(this, 0);
        },


        /**
         * Array#clear()  --> Array
         * 
         * Clears the content of this array
         *
         * @return a reference to this array
         */
		clear : function() {
			this.length = 0;
			return this;
		},


        /**
         * Array#size() --> Number
         * 
         * Returns the length of this array
         *
         * Returns the length of this array
         */
		size : function() {
			return this.length;
		},


        /**
         * Array#find(fn[,context])     --> Array item | undefined
         * 
         * Applies the specified function to each item of this array and returns the first one
         * for which the method returns true. If the context is specified, the iterator will
         * executes in that context
         *
         * @param fn            the iterator function to apply to each item of the array
         * @param context       the context for the "this" keyword
         * @return the first element of this array for which the iterator returns a true value
         */
		find : function(fn, context) {
            var result;
            
            for(var i = 0, len = this.length; i < len; i++) {
                if(fn.call(context, this[i], i)) {
                    result = this[i];
                    break;
                }
            }

            return result;
		},


        /**
         * Array#filter(fn[,context])  --> Array items
         * 
         * Applies the specified function to each item of this array and returns an array of the ones for which the 
         * method returns true. If the context is specified, the iterator executes in that context
         *
         * @param fn            the iterator function to apply to each item of the array
         * @param context       the context for the "this" keyword
         * @return the elements of this array for which the iterator returns a true value
         *
         * Reference:
         * https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/filter
         */
		filter : function(fn, context) {
            var results = [ ];
            
            for(var i = 0, len = this.length; i < len; i++) {
                if(fn.call(context, this[i], i)) {
                    results.push(this[i]);
                }
            }

            return results;
		},


        /**
         * Array#reject(fn[,context])     --> Array
         * 
         * Applies the specified function to each item of this array and returns an array of the ones for which the 
         * method returns false. If the context is specified, the iterator executes in that context
         *
         * @param iterator          the iterator function to apply to each item of the array
         * @param context           the context for the "this" keyword
         * @return the elements of this array for which the iterator returns a false value
         */
        reject : function(iterator, context) {
            var results = [ ];
            for(var i = 0, len = this.length; i < len; i++) {
                if(!iterator.call(context, this[i], i)) {
                    results.push(this[i]);
                }
            }

            return results;
        },


        /**
         * Array#map(fn[,context]) --> Array
         * 
         * Creates a new array as a result of calling the specified function on every item of this array. If the 
         * context is specified, the iterator executes in that context
         *
         * @param iterator              the iterator function to apply to each item of the array
         * @param context               the context for the "this" keyword
         * @return a new array that contain the results of applying the iterator function to each item of this array
         *
         * Reference:
         * https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/map
         */
        map : function(iterator, context) {
            var results = [ ];
            for(var i = 0, len = this.length; i < len; i++) {
                results.push(iterator.call(context, this[i], i));
            }

            return results;
        },


        /**
         * Array#recursivelyInvoke(fn[,parameters]) --> Array
         * 
         * Invokes the specified method, with the parameters specified, on all elements of this array,
         * and returns an array containing the results of the invocations. These methods are either
         * own methods of the array items or methods inherited from their superclasses or prototype chain
         *
         * @param method            the method to invoke on all the elements
         * @return an array consisting of the results of the method invocation
         */
        recursivelyInvoke : function(method) {
            var args = Array.from(arguments), parameters = args.slice(1);
            
            return this.map(function(element) {
                return element[method].apply(element, parameters);
            })
        },


        /**
         * Sorts the elements of this array based on the criterion function specified. The sort criteria must have
         * a clearly defined comparison semantics: there must exist the "element1 < element2" relationship between
         * any two elements of the array.
         * If no sorting function is specified, this implementation reverts to the native Array sorting function
         *
         * @param criterion             the sort function used to compare the elements of this array
         * @return a sorted array based on the specified sort function
         */
        sortItems : function(criterion) {
            if(!criterion) {
                return this.sort();
            }

            var sortedItems = this.map(function(value, index) {
               return {
                   value : value,
                   order : criterion.call(criterion, value, index)
               };
            }).sort(function(left, right) {
                return (left.order < right.order)? -1 : (left.order > right.order)? 1 : 0;
            });

            return sortedItems.collectValues("value");
        },


        /**
         * Array#collectValues(property[,context]) --> Array
         * 
         * Collects the values of the property specified on all items of the array, and returns a new array of those 
         * values
         *
         * @param property            the property to retrieve the values for
         * @return an array consisting of the values of this property for each item in the array
         */
        collectValues : function(property) {
            var results = [ ];
            
            for(var i = 0, len = this.length; i < len; i++) {
                results.push(this[i][property]);
            }

            return results;
        },


        /**
         * Array#partition(iterator[,context])     --> [truthy items, falsy items]
         * 
         * Partitions the elements of this array into two groups based on the invocation of the iterator specified: 
         * the ones that return true and those that return false
         *
         * @param iterator              the iterator function to apply to each item of the array
         * @param context               the context for the "this" keyword
         * @return an array that contains two sub-arrays after invoking the iterator function: one for truthy values
         * and one for falsy ones
         */
		partition : function(iterator, context) {
            var trues = [ ], falses = [ ];
            
            for(var i = 0, len = this.length; i < len; i++) {
                (iterator.call(context, this[i], i))? trues.push(this[i]) : falses.push(this[i]);
            }

            return [trues, falses];
		},


        /**
         * Array#inject(model, iterator, context) --> Object
         * 
         * Incrementally builds an object that is the result of the cumulative results of invoking the iterator on each 
         * item in the array
         *
         * @param model                 the accumulator object that holds the cumulative execution results
         * @param iterator              the function to invoke on each item in the array
         * @param context               the invocation context of the iterator method, if provided
         */
        inject : function(model, iterator, context) {
            for(var i = 0, len = this.length; i < len; i++) {
                model = iterator.call(context, model, this[i], i);
            }

            return model;
        },


        /**
         * Array#contains(item)     -->  boolean
         * 
         * Returns trus if the array contains the item specified; false otherwise
         *
         * @param item              the item to look for in this array
         * @return true if the array contains this item; otherwise, returns false
         */
		contains : function(item) {
            if(this.indexOf && Object.isFunction(this.indexOf)) {
                return (this.indexOf(item) != -1);
            }

            var found = false;
            for(var i = 0, len = this.length; i < len; i++) {
                if(this[i] === item) {
                    found = true;
                    break;
                }
            }

            return found;
		},


        /**
         * Array#indexOf(item) --> Number
         * 
         * Returns the first index at which the element specified can be found in the array, or -1 if it's not present
         *
         * @param item              the element to retrieve the index for
         * @param i                 the index at which to start the search; defaults to 0
         *
         * Reference:
         * https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/indexOf
         */
		indexOf : function(item, i) {
			i || (i = 0);
			var len = this.length;

			if(i < 0) {
				i = len + i;
			}

			for(; i < len; i++) {
				if(this[i] === item) {
					return i;
				}
			}

			return -1;

		},


        /**
         * Array#lastIndexOf(item, index) --> Number
         * 
         * Returns the last index at which the element specified can be found in the array, or -1 if it's not present. 
         * The array is searched backwards, starting at index
         *
         * @param item              the element to retrieve the index for
         * @param i                 the index at which to start the search; defaults to 0
         *
         * Reference:
         * https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/lastIndexOf
         */
		lastIndexOf : function(item, i) {
			i = (isNaN(i))? this.length : (i < 0? this.length + i : i) + 1;
			var n = this.slice(0,i).reverse().indexOf(item);
            return (n < 0) ? n : i - n - 1;
		}
	};


    for(var method in extensions) {
        if(!Array.prototype[method]) {
            Array.prototype[method] = extensions[method];
        }
    }

})();


Array.from = function(object) {
    if("toArray" in object && Object.isFunction(object.toArray)) {
        return object.toArray();
    }

    var len = object.length || 0, results = new Array(len);
    
    while(len--) {
        results[len] = object[len];
    }

    return results;
};




/*----------------------------------------------------------------------------------------------------------------------
                                        FUNCTION EXTENSION METHODS
----------------------------------------------------------------------------------------------------------------------*/

Object.extend(Function, {

    /**
     * Sets a reference to an empty function. This method is intended to be used
     * as default value for function properties that need to be overridden
     */
    empty : function() { },


    /**
     * Defines an abstract function. This method is intended to be used by abstract classes
     * to define methods that need to be implemented by subclasses
     */
    abstractFunction : function() {
        throw new Error("This method is abstract and needs to be implemented.");
    },


    /**
     * Returns the argument names for a given function
     *
     * @param fn            the function to return the argument names for
     * @return the argument names for the function specified
     */
    argumentNames : function(fn) {
        var names = fn.toString().match(Function.fnArguments)[1].replace(/\s+/g,"");
        return (names.length == 1 && !names[0])? [ ] : names;
    },


    /**
     * Regular expression for extracting the argument names in a function, and to check whether a function can be 
     * decompiled (only Opera Mobile fails this test) and, if so, set the regular expression for extracting a call 
     * to a superclass method
     */
    fnArguments : /^[\s\(]*function[^(]*\(([^\)]*)\)/,
    fnTest : /xyz/.test("function() { xyz; }")? /\b_super\b/ : /.*/

});


(function() {

    /**
     * Extends the Function.prototype object with the following utility methods if they don't already exist. Most 
     * implementations are taken from the Prototype library
     */
	var extensions = {

        /**
         * Function#bind(context)   --> Function
         * 
         * Binds this function to the context specified as the first argument. It wraps the original function in a 
         * new one that is returned. This ensures that object methods or event callbacks are bound to their respective 
         * contexts when executing
         *
         * @return a function bound to the context specified and wrapping the original function
         */
        bind : function() {
            if(arguments.length < 2 && Object.isUndefined(arguments[0])) {
                return this;
            }

            var _method = this, args = Array.from(arguments), context = args.shift();
            return function() {
                return _method.apply(context, args.concat(Array.from(arguments)));
            }
        },


        /**
         * Function#curry(arguments) --> Function
         * 
         * Returns a function that prepopulates some parameters with the arguments specified. When the function is 
         * invoked, its final list of arguments will be a collection of the curried arguments and the new ones passed 
         * to the returned function
         *
         * @return a curried function that contains pre-filled arguments
         */
        curry : function() {
            if(!arguments.length) {
                return this;
            }

            var _method = this, args = Array.from(arguments);
            return function() {
                return _method.apply(this, args.concat(Array.from(arguments)));
            }
        },


        /**
         * Function#delay(number)  --> Function
         * 
         * Returns a function that wraps the original function in a timer (in seconds). The function can be bound to 
         * its execution context using Function#bind and will execute after the timer elapses
         *
         * @return a wrapper function that will execute after the timer has elapsed
         */
        delay : function() {
            var _method = this, args = Array.from(arguments), timeout = args.shift() * 1000;
            return setTimeout(function() {
                return _method.apply(_method, args);
            }, timeout);
        },


        /**
         * Function#defer() --> Function
         * 
         * Returns a function that wraps the original function in a default timer of 10ms. This is a convenience
         * implementation of Function#delay that ensures that a function is invoked last
         *
         * @return a wrapper function that will execute after 10ms
         */
        defer : function() {
            var args = [0.01].concat(Array.from(arguments));
            return this.delay.apply(this, args);
        }
    };

    for(var method in extensions) {
        if(!Function.prototype[method]) {
            Function.prototype[method] = extensions[method];
        }
    }

})();



/*----------------------------------------------------------------------------------------------------------------------
                                            STRING EXTENSION METHODS
 
  These are extension methods to the built-in String object. They provide utility methods for performing
  common string manipulation tasks
 
----------------------------------------------------------------------------------------------------------------------*/
(function() {
    
    "use strict";
    
    var scriptRegExp = /<script[^>]*>([\S\s]*?)<\/script>/gim,
    tagRegExp = /<\/?[^>]+>/gi,
    JSONFilter = /^\/\*-securejson-([\s\S]*)\*\/\s*$/,
    templateFormat = /(\{[^}]+\})/gm;


    /**
     * String#truncate(string, length)   --> String
     * 
     * Truncates a given string to the specified length and appends a truncation string to the result
     *
     * @param length            the length to truncate to
     * @return a string truncated to the specified length
     */
    String.truncate = function(string, length) {
        length = length || 30;
        var truncation = "...";
        return (string.length > length)? string.slice(0, length - truncation.length) + truncation : String(string);
    };


    /**
     * String#trim()   --> String
     * 
     * Removes all leading and trailing spaces from this string
     *
     * @return a string with all leading and trailing spaces removed
     */
    String.trim = function(string) {
        return string.replace(/^\s+/,"").replace(/\s+$/,"");
    };


    /**
     * String#toCamelCase()   --> String
     * 
     * Formats the current string in a camel-case format by removing all dashes and capitalizing the resulting words
     *
     * @return a camel-case version of this string
     */
    String.toCamelCase = function(string) {
        return string.replace(/-\D/g, function(match){
            return match.charAt(1).toUpperCase();
        });
    };


    /**
     * String#capitalize()   --> String
     * 
     * Capitalizes the first letter of this string
     *
     * @return a string with the first letter capitalized
     */
    String.capitalize = function(string) {
        return string.replace(/\b[a-z]/g, function(match){
            return match.toUpperCase();
        });
    };


    /**
     * String#removeScripts()   --> String
     * 
     * Removes all "script" tags from this string and returns the resulting string
     *
     * @return a string with all "script" tags removed
     */
    String.removeScripts = function(string) {
        return string.replace(scriptRegExp, "");
    };


    /**
     * String#removeTags()   --> String
     * 
     * Removes all HTML tags from this string and returns the resulting string
     *
     * @return a string with all HTML tags removed
     */
    String.removeTags = function(string) {
        return string.replace(tagRegExp, "");
    };


    /**
     * String#escapeHTML()  --> String
     * 
     * Replaces all HTML special characters in this string with the equivalent HTML entities
     *
     *  @return a string with all HTML special characters replaced with their equivalent HTML entities
     */
    String.escapeHTML = function(string) {
        return string.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
    };


    /**
     * String#unescapeHTML()  --> String
     * 
     * Reverts all HTML entities back to their normal form
     *
     * @return a string with all HTML entities reverted back to their normal form
     */
    String.unescapeHTML = function(string) {
        return string.replace(/&amp;/g, "&").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
    };


    /**
     * String#isEmpty()  --> Boolean
     * 
     * Returns true if this string is empty (consists of one or more white spaces); false otherwise
     *
     * @return true if this string is empty; otherwise, returns false
     */
    String.isEmpty = function(string) {
        return (String.trim(string).length == 0);
    };


    /**
     * String#contains(pattern)  --> Boolean
     * 
     * Returns true if this string contains the specified pattern; otherwise, returns false
     *
     * @return true if this string contains the specified pattern; otherwise, returns false
     */
    String.contains = function(string, pattern) {
        return (string.indexOf(pattern) != -1);
    };


    /**
     * String#startsWith(pattern)  --> Boolean
     * 
     * Returns true if this string starts with the specified pattern; otherwise, returns false
     *
     * @return true if this string starts with the specified pattern; otherwise, returns false
     */
    String.startsWith = function(string, pattern) {
        return (string.indexOf(pattern) === 0);
    };


    /**
     * String#endsWith(pattern)  --> Boolean
     * 
     * Returns true if this string ends with the specified pattern; otherwise, returns false
     *
     * @return true if this string ends with the specified pattern; otherwise, returns false
     */
    String.endsWith = function(string, pattern) {
        var delta = string.length - pattern.length;
        return (delta > 0 && string.lastIndexOf(pattern) === delta);
    };


    /**
     * String#format(replacement)  --> String
     * 
     * Creates a new template based on this string, and replaces its properties with those of the replacement 
     * object provided
     *
     * @param string                the string template to format
     * @param replacement           the object that contains the values to fill the template with
     * @return a string that is the result of replacing the properties of this string with the values in the 
     * replacement object
     */
    String.format = function(string, replacement) {
        return string.replace(templateFormat, function(match) {
            var key = match.substring(1, match.length - 1);
            return (replacement[key] || match);
        });
    };

    /**
     * String#unfilterJSON(filter)      --> String
     * 
     * Removes security comment delimiters around a JSON string using the specified filter. If no
     * filter is provided, it uses the default one: /*-securejson-* /
     *
     * @param string                the string from which to extract the JSON object 
     * @param filter                the security filter to use
     * @return a JSON string without the security comments delimiters
     */
    String.extractSecureJSON = function(string, filter) {
        return string.replace(filter || JSONFilter, '$1');
    };

})();



/*----------------------------------------------------------------------------------------------------------------------
                                        NUMBER EXTENSION METHODS
----------------------------------------------------------------------------------------------------------------------*/

(function() {

    /**
     * Extends the Function.prototype object with the following utility methods if they don't already exist. Most 
     * implementations are taken from the Prototype library
     */
	var extensions = {

        /**
         * Number#seconds()   --> Number
         * 
         * Returns the millisecond equivalent of the specified number
         *
         * @return the millisecond representation of the specified number
         */
        seconds: function() {
            return this * 1000;
        },


        /**
         * Number#minutes()   --> Number
         * 
         * Converts the specified number from its minutes unit to milliseconds
         *
         * @return the millisecond representation of the specified number, which is expressed in minutes
         */
        minutes: function() {
            return this * (60).seconds();
        },

        
        /**
         * Number#hours()   --> Number
         * 
         * Converts the specified number from its hours unit to milliseconds
         *
         * @return the millisecond representation of the specified number, which is expressed in hours
         */
        hours: function() {
            return this * (60).minutes();
        },


        /**
         * Number#days()   --> Number
         * 
         * Converts the specified number from its days unit to milliseconds
         *
         * @return the millisecond representation of the specified number, which is expressed in days
         */
        days: function() {
            return this * (24).hours();
        },

        
        /**
         * Number#weeks()   --> Number
         * 
         * Converts the specified number from its weeks unit to milliseconds
         *
         * @return the millisecond representation of the specified number, which is expressed in weeks
         */
        weeks: function() {
            return this * (7).days();
        },

        
        /**
         * Number#months()   --> Number
         * 
         * Converts the specified number from its months unit to milliseconds
         *
         * @return the millisecond representation of the specified number, which is expressed in months
         */
        months: function() {
            return this * (30).days();
        },


        /**
         * Number#years()   --> Number
         * 
         * Converts the specified number from its years unit to milliseconds
         *
         * @return the millisecond representation of the specified number, which is expressed in years
         */
        years: function() {
            return parseInt(this * (365.25).days())
        },
        
        
        /**
         * Converts this number to a date
         * 
         * @return the date equivalent of this number
         */
        toDateString: function() {
            var date = new Date();
            date.setTime(this);
            return date;
        },
        
        /**
         * Invokes the given iterator this many times in the given context
         * 
         * @param iterator          the iterator to invoke
         * @param context           the context for the "this" keyword
         */
        times : function(iterator, context) {
            if(this < 0) {
                return this;
            }

            for(var i = 0; i < this; i++) {
                iterator.call(context, this, i);
            }

            return this;
        },
        
        /**
         * Returns a string representation of the given number, padded with zeros
         * 
         * @param length            the number of zeros to add as padding
         * @param radix             the radix to use for the conversion; defaults to 10
         */
        toPaddedString : function(length, radix) {
            var string = this.toString(radix || 10);
            return '0'.times(length - string.length) + string;
        }

    };

    for(var method in extensions) {
        if(!Number.prototype[method]) {
            Number.prototype[method] = extensions[method];
        }
    }

})();



/*----------------------------------------------------------------------------------------------------------------------
                                    CORE CONCIERGE MODULE

This is the core module for the Workshop UI application. It defines properties and methods that are used to perform 
common tasks at the UI level such as firing initialization events, registering and initializing widgets, forwarding
custom events, etc
This implementation is independent of any Javascript framework 
----------------------------------------------------------------------------------------------------------------------*/

var Concierge = Concierge || { };

Object.extend(Concierge, (function() {
    "use strict";
    
    var ApplicationState = {
        INITIALIZING : "initializing",
        INITIALIZED : "initialized"
    }, 
    state;
    
    var browser = {
        WebKit: (navigator.userAgent.indexOf('AppleWebKit/') > -1),
        Gecko:  (navigator.userAgent.indexOf('Gecko') > -1 && navigator.userAgent.indexOf('KHTML') === -1),
        MobileSafari: (!!navigator.userAgent.match(/Apple.*Mobile.*Safari/)),
        IE: !!(window.attachEvent && navigator.userAgent.indexOf('Opera') === -1),
        Opera:  (navigator.userAgent.indexOf('Opera') > -1)
    };
    
    
    /**
     * Returns true if the specified name is a valid namespace string
     *
     * @param name          the name to use as a namespace
     * @return true if the string is a valid namespace; false otherwise
     */
    function isValidNamespace(name) {
       return (name.charAt(0) != "." && name.charAt(name.length - 1) != "."
           && (name.indexOf("..") == -1));
    }


    /**
    * Creates a namespace by splitting the provided string and parsing to generate the required structure. 
    * The method checks for the validity of the path and also maintains the integrity of any existing object being 
    * used as parent
    *
    * @param namespace         the namespace to create
    * @return the newly created namespace
    */
    function createNamespace(namespace) {

       if(!isValidNamespace(namespace)) {
           throw new Error("Please provide a valid namespace to create.");
       }

       var container = window, parts = namespace.split(".");

       for(var i = 0, len = parts.length; i < len; i++) {
           var part = parts[i];
           container = container[part] || (container[part] = { });
       }

       return container;
    }

    
    /**
     * Logs a message to the browser's console if it exists; otherwise outputs an alert message
     *
     * @param message		the message to output to the console
     */
    function log() {
        try {

            console.log.apply(console, arguments);

        } catch(exception) {
            try {
                opera.postError.apply(opera, arguments);
            } catch(ex) {
                alert(Array.prototype.join.call(arguments, ""));
            }
        }
    }
    
    /**
     * External Concierge module initialization method. This method is invoked when the "dom:loaded" custom event
     * is fired by the browser
     */
    function initialize() {
        state = ApplicationState.INITIALIZING;
        state = ApplicationState.INITIALIZED;
    }
    
    /**
     * Generates a sequential unique ID (number)
     *
     * @return an incremental integer
     */
    function generateUID() {
        return Concierge.UID++;
    }


    /**
     * Returns the current state of the application
     * 
     * @return the current state of the application
     */
    function getApplicationState() {
        return state;
    }
    
    
    /**
     * Returns true if the application is initialized; otherwise, returns false
     * 
     * @return true if the application is initialized; otherwise, returns false
     */
    function isInitialized() {
        return state == ApplicationState.INITIALIZED;
    }
    
    return {
        createNamespace : createNamespace,
        log : log,
        initialize : initialize,
        generateUID : generateUID,
        getApplicationState : getApplicationState,
        isInitialized : isInitialized,
        Browser : browser
    }

})());


Concierge.UID = 1;
Concierge.createNamespace("Concierge.UI");

if(!window.$C) {
    window.$C = Concierge;
}


/*----------------------------------------------------------------------------------------------------------------------
                                CLASS CREATION AND INHERITANCE

This module defines an inheritance system for the application. The "inherits" method sets the prototype chain between a 
class and its superclass, and the "create" method generates a constructor function that contains the inheritance chain 
as well as references to its superclass and subclasses.

This system also allows a subclass to invoke a method of its superclass by using the following syntax: 
this._super(arguments)

This implementation is inspired by the base2 and Prototype libraries
----------------------------------------------------------------------------------------------------------------------*/

Concierge.Class = {

    /**
     * Creates an inheritance chain for a subclass by inheriting properties from the superclass. It also adds a 
     * convenient "superclass" reference to the superclass
     *
     * @param child         the subclass to create
     * @param parent        the superclass to inherit from
     * @return a subclass that inherits from the superclass specified
     */
    inherits : (function() {
        function F() { }

        return function(child, parent) {
            F.prototype = (Object.isFunction(parent))? parent.prototype : parent;
            child.prototype = new F();
            child.prototype.constructor = child;
            child.superclass = parent;

            return child;
        };

    })(),


    /**
     * Creates a new class from the specified argument objects. If the parent is provided, thenthe class inherits from the superclass and can invoke the superclass methods through the
     * "this._super" syntax
     *
     * @param parent            the superclass to inherit from (optional)
     * @param props             the mixin object containing the properties and methods to add to the
     *                          class' prototype
     * @return a new class with the specified properties and methods
     */
    create : function(parent, props) {
        var superclass = parent, properties = props, _superPrototype,
        inherits = Concierge.Class.inherits;

        /**
         If a single argument was provided, then we're creating a new class: use the argument as a mixin for the new 
         class and set the superclass to null
         */
        if(arguments.length == 1) {
            superclass = null;
            properties = arguments[0];
        }

        /**
            If more than 1 argument are provided, save a reference to the prototype of the
            superclass in a "_superPrototype" variable; it'll be used later for augmenting
            the subclass' prototype
         */
        if(superclass && superclass.prototype) {
            _superPrototype = superclass.prototype;
        }

        /**
         * This is the main constructor function that will be augmented and returned. Sets a
         * "superclass" property that references the parent constructor, creates a "subclasses"
         * array to hold subclasses that will be created from it, and augments it with the superclass'
         * prototype
         */
        function Class() {
            this.initialize.apply(this, arguments);
        }

        Class.subclasses = [ ];

        /**
         * If a superclass was provided, set the inheritance chain and add this class
         * as a subclass of the parent
         */
        if(superclass) {
            inherits(Class, superclass);

            if(superclass.subclasses) {
                superclass.subclasses.push(Class);
            }
        }

        /**
         * Copy the properties of the provided mixin to those of the constructor's prototype.
         * If the value of a property is a function and the serialized body of the method
         * contains the "_super" keyword, then create a temporary "_super" property on the constructor
         * that invokes the parent's inherited method
         */
        for(var name in properties) {
            Class.prototype[name] = (Object.isFunction(properties[name])
                && (_superPrototype && Object.isFunction(_superPrototype[name]))
                && (Function.fnTest.test(properties[name])))?
                (function(name, fn) {
                    return function() {
                        var temp = this._super || null;

                        // Add a new "_super" method that's the same but on the parent
                        this._super = _superPrototype[name];

                        // The method needs to be bound only once, so we remove it
                        // when we're done invoking it
                        var returnValue = fn.apply(this, arguments);
                        this._super = temp;

                        return returnValue;
                    };
                })(name, properties[name]) : properties[name];
        }

        if(!Class.prototype.initialize) {
            Class.prototype.initialize = Function.empty;
        }

        return Class;
    },
    
    
    /**
     * Returns true if the class instance specified implements the method specified; otherwise,
     * returns false
     *
     * @param instance              the instance to check for the implementation of the method
     * @param method                the method to look for in the instance specified
     * @return true if the instance implements the method specified; otherwise, returns false
     */
    implementsMethod : function(instance, method) {
        return (method in instance);
    },


    /**
     * Returns true if the instance specified conforms to the protocol specified. In this implementation,
     * the conformance to a protocol simply means that the instance implements all the "requiredMethods" defined
     * by the constructor of the protocol class. This is a loose implementation concetp only intended for use by
     * the classes that act as delegates to the different widgets in the application
     *
     * @param instance              the instance to check for the protocol conformance
     * @param protocol              the constructor of the class to check the conformance to
     * @return true if the instance conforms to the protocol specified; false otherwise
     */
    conformsToProtocol : function(instance, protocol) {
        var conforms = true, 
        requiredMethods = protocol.requiredMethods;

        if(!(requiredMethods && Object.isArray(requiredMethods))) {
            return conforms;
        }

        for(var i = 0, len = requiredMethods.length; i < len; i++) {
            var method = requiredMethods[i];
            if(!instance[method]) {
                conforms = false;
                break;
            }
        }

        return conforms;
    }

};



/*---------------------------------------------------------------------------------------------------------------------
 *                  DATA STRUCTURE IMPLEMENTATIONS
 *                  
 * This module implements some commonly-used data structures (Stack, Queue, etc.). 
 * These structures are used primarily as conceptual aids rather than pure data storage devices.
 * They're created to carry out a particular task during the operations of the program.
 *--------------------------------------------------------------------------------------------------------------------*/
    

Concierge.DataStructure = {

    items : [ ],
    
    /**
     * Adds the specified element to the list of elements for this data structure
     * 
     * @param element                   the element to add to the list
     * @return the newly added element
     */
    add : function(element) {
        if(!(this.items && Object.isArray(this.items))) {
            return null;
        }
        
        this.items.push(element);
        return element;
    },


    /**
     * Abstract function to be implemented by subsclasses
     */
    remove : Function.empty,


    /**
     * Returns a reference to the last element in this structure. Note that this method does not remove the element; it
     * simply specifies whether there's an element present in the structure
     * 
     * @return a reference to the last item in the structure
     */
    peek : function() {
        return this.items[this.items.length - 1];
    },


    /**
     * Returns the total number of elements in this structure
     * 
     * @return the total number of elements in this structure
     */
    size : function() {
        return this.items.length;
    },


    /**
     * Returns true if this structure is empty; otherwise, returns false
     * 
     * @return true if this structure is empty; otherwise, returns false
     */
    isEmpty : function() {
        return this.size() == 0;
    },
    
    
    /**
     * Returns the element at the specified index, or null if no such element exists
     * 
     * @apram index                 the index of the element to retrieve
     * @return the element at the specified index, or null if no such element exists
     */
    getObjectAtIndex : function(index) {
        return (index < this.size())? this.items[index] : null;
    }
    
};


/**
 * An implementation of the {@code Concierge.DataStructure} to simulate a Stack
 */
Concierge.DataStructure.Stack = Concierge.Class.create(Concierge.DataStructure, {
    
    initialize : function() { 
        this.items = [ ];
    },

    /**
     * Removes the last element in the structure and returns its reference
     * 
     * @return a reference to the last element in this structure
     */
    remove : function() {
       return this.items.pop();
    }
});


/**
 * An implementation of the {@code Concierge.DataStructure} to simulate a queue
 */
Concierge.DataStructure.Queue = Concierge.Class.create(Concierge.DataStructure, {
    initialize : function() { 
        this.items = [ ];
    },

    /**
     * Removes the last element in the structure and returns its reference
     * 
     * @return a reference to the last element in this structure
     */
    remove : function() {
       return this.items.shift();
    }
});



/*----------------------------------------------------------------------------------------------------------------------
                                "WIDGET BASE" MIXIN
  This module serves as the base mixin object for all widgets. It includes properties and methods that are shared by 
  all widget class implementations
 ---------------------------------------------------------------------------------------------------------------------*/
Concierge.WidgetBase = {

    /**
     * Initializes the widget and its options. The resulting options object
     * is the user-defined options merged with those of the widget constructor's
     * default options
     *
     * @param options		the widget declaration options; these options will
     *						be merged with the widget constructor's default options
     */
    init : function(options) {
        this.setOptions(options);
        return this;
    },


    /**
     * Sets the options for this widget instance. The basic options are defined on
     * the widget constructor and copied over to each widget instance
     *
     * @param options		the options hash to merge with the widget constructor's
     *						default options
     * @return a JSON object resulting from the merge of widget options
     */
    setOptions : function(options) {
        this.options = Object.clone(this.constructor.defaultOptions);
        return Object.extend(this.options, options || { });
    },


    /**
     * Publishes a topic (custom event) with the related data object. The topic
     * must follow the "eventType:eventName" pattern (i.e "widget:loaded"). The
     * data contains the event model and will be made available to the listeners
     * through the "memo" property of the event object
     *
     * @param topic			the topic to publish
     * @param data			the data to send with the topic
     */
    publish : function(topic, data) {
        if(!Concierge.Event.isCustomEvent(topic)) {
            throw DOMException.CUSTOM_EVENT_ERR;
        }

        Concierge.Event.fire(document, topic, data);
        return this;
    },


    /**
     * Subscribes to a topic (custom event) published by a widget or any other code,
     * provided the topic follows the same "eventType:eventName" format. The event model
     * will be made available to the "callback" method through the "model" property
     * of the event object
     *
     * @param topic			the topic to subscribe to
     * @param callback		the callback method to invoke when the topic event is fired
     */
    subscribe : function(topic, callback) {
        Concierge.Event.add(document, topic, callback);
        return this;
    },


    /**
     * Displays a string representation of the current widget
     *
     * @return a JSON string representing the structure of the widget
     */
    inspect : function() {
        return "Concierge Widget : { type : \"" + this.options.type + "\", id : \"" + this.options.id + "\" }";
    }
};



/*----------------------------------------------------------------------------------------------------------------------
 *                                      WIDGET CONTROLLER
 * 
 * This module manages the registration, creation and retrieval of widgets: it registers widget instances and initializes 
 * them when the "dom:loaded" event is fired by the document
 *--------------------------------------------------------------------------------------------------------------------*/
Concierge.Widgets = (function() {
    
    "use strict";
    
    var instances = { }, 
    registry = new Concierge.DataStructure.Queue();


    /**
     * Registers a widget instance for later invocation when the page loads
     *
     * @param widget            a reference to the widget's constructor function
     * @param options           the widget declaration options
     */
    function register(widget, options) {
        if(Concierge.isInitialized()) {
            return;
        }
        
        var id = (options.id)? options.id : "widget_" + Concierge.generateUID();
        registry.add({
            id : id,
            instance : widget,
            widgetOptions : options || { },
            delay : options.delay || 0.5
        });
    }


    /**
     * Removes a widget instance from the instances hash. If the instance has a destroy method
     * (for custom cleanup), this method is invoked prior to the removal of the widget
     *
     * @param id                the ID of the widget to remove
     */
    function unregister(id) {
        var widget = get(id);

        if(!widget) {
            return;
        }
        
        if(widget.destroy && Object.isFunction(widget.destroy)) {
            widget.destroy();
        }

        delete instances.id;
    }


    /**
     * Returns the widget instance whose ID matches the specified ID
     *
     * @param id                the ID of the widget to look for
     * @throws DOMException.WIDGET_NOT_FOUND_ERR if no widget matching
     *      the specified ID is found
     */
    function get(id) {
        var instance = instances.get(id);

        if(Object.isUndefined(instance)) {
            throw String.format(DOMException.WIDGET_NOT_FOUND_ERR, {id : id});
        }
        
        return instance;
    }


    /**
     * Initializes the registered widgets based on their respective constructor
     * and declared options
     */
    function initialize() {
        for(var i = 0, len = registry.size(); i < len; i++) {
            (function initializeWidget(index) {
                var declaration = registry.getObjectAtIndex(index), 
                delay = declaration.delay;
                
                try {

                    (function() {
                        var type = eval(declaration.instance),
                        instance = new type(declaration.widgetOptions);
                        instances[declaration.id] = instance;
                    }).delay(delay);

                } catch (exception) {
                    Concierge.log("Unable to instantiate widget: %s - Exception: %s", declaration.id, exception.message);
                }
                
            })(i);
        }
    }

    return {
        initialize : initialize,
        register : register,
        unregister : unregister,
        get : get
    };

})();




/*----------------------------------------------------------------------------------------------------------------------
                                            AJAX MODULE

    This module manages Ajax (Asynchronous Javascript and XML) requests in a cross-browser fashion. It defines a factory
method for retrieving the appropriate XHR object and stores its reference for later retrieval
----------------------------------------------------------------------------------------------------------------------*/

Concierge.Ajax = Concierge.Class.create({
    
    /**
     * Default constructor. Defines the default options for this Ajax instance; those settings
     * will later be complemented with the user-defined one to form the final set of options
     * for the request
     */
    initialize : function() {
        this.options = {
            asynchronous : true,
            contentType : "application/x-www-form-urlencoded",
            encoding : "UTF-8",
            requestHeaders : {"Accept" : "application/json"},
            success : Function.empty,
            failure : Function.empty
        };
    },


    /**
     * Sends an asynchronous request to the URL specified, using this method and options
     *
     * @param method            the request method; defaults to "POST"
     * @param url               the request URL
     * @param options           the request options; contains settings such as the content type,
     *                          the request parameters, the success and failure callback methods, etc
     */
	request : function(method, url, options) {
		this.transport = this.createAjaxTransport();
        this.url = url;
        this.method = (method)? method.toUpperCase() : "POST";
        Object.extend(this.options, options || { });
        var queryString;

        if(this.method == "GET" && this.options.parameters) {
            var parameters = this.options.parameters;
            queryString = (Object.isString(parameters))? parameters : Object.toQueryParameters(parameters);
            this.url += ((String.contains(this.url, "?")? "&" : "?") + queryString);
        }
        
		this.transport.onreadystatechange = this.onStateChange.bind(this);
        this.transport.open(this.method, this.url, this.options.asynchronous);
        this.configureHeaders();

        var postBody = (this.method == "POST")? (this.options.postBody || queryString) : null;
        this.transport.send(postBody);
	},


    /**
     * Configures the headers for this request. These headers are defined in the "requestHeaders"
     * property of the request options parameter
     */
    configureHeaders : function() {
        var headers = this.options.requestHeaders;

        if(this.method == "POST") {
            this.options.requestHeaders["Content-Type"] = this.options.contentType +
                ((this.options.encoding)? "; charset=" + this.options.encoding : "");
        }

        for(var header in headers) {
            this.transport.setRequestHeader(header, headers[header]);
        }
    },


    /**
     * This callback method is invoked when the ready state of the transport changes. When the request has
     * reached the final state (readyState is 4), the respective callback methods are invoked
     */
    onStateChange : function() {
        var xhr = this.transport;

        if(xhr.readyState != 4) {
            return;
        }
        
        if(xhr.status == 200) {
            var acceptJSON = (this.options.requestHeaders.Accept == "application/json"),
            responseText = (acceptJSON)? Object.evalJSON(xhr.responseText) : xhr.responseText;
            this.options.success(responseText, xhr.responseXML);
        
        } else {
            this.options.failure(xhr);
        }
    },


    /**
     * A factory method that creates an instance of the XMLHttpRequest transport object
     * for a specific environment. It caches the result of the invocation for later reuse
     *
     * @return an instance of an XMLHttpRequest object for a specific browser
     */
	createAjaxTransport : function() {
		var methods = [
			function() {return new XMLHttpRequest();},
			function() {return new ActiveXObject("Msxml2.XMLHTTP");},
			function() {return new ActiveXObject("Microsoft.XMLHTTP");}
		];

		for(var i = 0; i < methods.length; i++) {
			try {
                
				methods[i]();
			
            } catch(e) {
				continue;
			}

			this.createAjaxTransport = methods[i];
			return methods[i]();
		}

		throw new Error("XMLHttpRequest couldn't be initialized!");
	}
});



/*----------------------------------------------------------------------------------------------------------------------
                            CORE DOM MODULE

  This module handles DOM element creation and scripting, and provides utility methods
for accessing common DOM-related tasks. It also defines a DOM storage mechanism for
caching properties of some DOM elements
----------------------------------------------------------------------------------------------------------------------*/
if (!Node) {
    var Node = { };
}

if (!Node.ELEMENT_NODE) {
    Object.extend(Node, {
        ELEMENT_NODE: 1,
        ATTRIBUTE_NODE: 2,
        TEXT_NODE: 3,
        CDATA_SECTION_NODE: 4,
        ENTITY_REFERENCE_NODE: 5,
        ENTITY_NODE: 6,
        PROCESSING_INSTRUCTION_NODE: 7,
        COMMENT_NODE: 8,
        DOCUMENT_NODE: 9,
        DOCUMENT_TYPE_NODE: 10,
        DOCUMENT_FRAGMENT_NODE: 11,
        NOTATION_NODE: 12
    });
}

/**
 * Extend the DOMException object by adding more descriptive error messages
 * TO-DO: These strings will be moved to the "i18n" module
 */
if(!DOMException) {
    var DOMException = { };
}

Object.extend(DOMException, {
	NOT_FOUND_ERR : "The element specified was not found; please check the element ID or reference and try again",
    INVALID_EVENT_ELEMENT : "Please provide a valid element for '{0}' event",
	SYNTAX_ERR : "Your script contains a syntax error; please check expression or declaration",
	NAMESPACE_ERR : "Please provide a valid namespace string",
	SCRIPT_ERR : "A page initialization script returned the following error: {error}",
	WIDGET_NOT_FOUND_ERR : "No matching widget was found for the specified ID: {id}",
	CUSTOM_EVENT_ERR : "Please provide a valid custom event name; must follow the 'name:nameExtra' pattern",
	WIDGET_INIT_ERR : "A widget initialization method returned the following error: {error}",
	JSON_FORMAT_ERR : "Badly formed JSON string"
});


Concierge.DOM = (function() {

    "use strict";
    
    /**
     * Creates an HTML element with specified tagname, attributes and children
     *
     * @param tagName                the tag of the element to create (div, p, etc)
     * @param attributes             the attributes to set for this element (id, className, etc)
     * @param children               the child nodes to append to this element
     * @return a DOM element of the specified type and with the specified attributes
     */
     function createElement(tagname, attributes, children) {

         if(arguments.length == 2 && (Object.isArray(attributes)
             || Object.isString(attributes))) {
             children = attributes;
             attributes = null;
         }

         var element = document.createElement(tagname);

         if(attributes) {
             for(var name in attributes) {
                 element[name] = attributes[name];
             }
         }

         if(children != null) {
             if(Object.isArray(children)) {
                 children.forEach(function(child){
                     if(Object.isString(child)) {
                         child = document.createTextNode(child);
                     }
                     element.appendChild(child);
                 });

             } else if(Object.isString(children)) {
                 element.appendChild(document.createTextNode(children));

             }else {
                 element.appendChild(children);
             }
         }

         return element;
     }


    /**
     * Returns the DOM element that matches the specified ID
     *
     * @param id            the ID of the DOM element to retrieve
     * @return the DOM element that matches the ID specified
     */
    function get(id) {
        return document.getElementById(id);
    }


    /**
     * Normalizes the DOM element specified by triggering its "hasLayout" property; this
     * only applies to IE browsers
     *
     * @param element           the element to trigger the "hasLayout" property on
     * @return the normalized element
     */
    function normalize(element) {
        if(Concierge.Browser.IE && (!element.currentStyle.hasLayout)) {
            setStyle(element, "zoom:1");
        }

        return element;
    }

    /**
     * Shows a DOM element by setting its display to "block"
     *
     * @param element           the DOM element to show
     */
    function show(element) {
        element = (Object.isString(element))? get(element) : element;
        element.style.display = '';
        return this;
    }


    /**
     * Hides a DOM element by setting its display to "none"
     *
     * @param element           the DOM element to hide
     */
    function hide(element) {
        element = (Object.isString(element))? get(element) : element;
        element.style.display = "none";
        return this;
    }


    /**
     * Returns true if the DOM element specified is visible (display set to "block"); false otherwise
     *
     * @param element           the DOM element to check the visible for
     * @return true if the DOM element is visible; false otherwise
     */
    function isVisible(element) {
        element = (Object.isString(element))? get(element) : element;
        return element.style.display != "none";
    }


    /**
     * Disables the current form element (input field)
     *
     * @param element           the DOM element to disable
     */
    function disable(element) {
        element = (Object.isString(element))? get(element) : element;
        element.disabled = true;
        return this;
    }


    /**
     * Enables the current form element (input field)
     *
     * @param element           the DOM element to enable
     */
    function enable(element) {
        element = (Object.isString(element))? get(element) : element;
        element.disabled = false;
        return this;
    }


    /**
     * Removes the current DOM element
     *
     * @param element           the DOM element to remove
     */
    function remove(element) {
        element = (Object.isString(element))? get(element) : element;
        element.parentNode.removeChild(element);
        return this;
    }


    /**
     * Recursively collects all the element's parent, siblings or child elements that match the property specified
     *
     * @param element               the element to retrieve the DOM nodes for
     * @param property              the property to look for when collecting the nodes
     * @return an array of DOM elements that match this element's property
     */
    function recursivelyCollect(element, property) {
        element = (Object.isString(element))? get(element) : element;
        var elements = [ ];
        
        while(element != null && ((element = element[property]) != document.body)) {
            if(Object.isElement(element)) {
                elements.push(element);
            }
        }

        return elements;
    }


    /**
     * Returns all the ancestors for this element up to the document body
     *
     * @parma element               the element to collect the ancestors for
     * @return an array of the ancestors for this element
     */
    function getAncestors(element) {
        element = (Object.isString(element))? get(element) : element;
        return recursivelyCollect(element, "parentNode");
    }


    /**
     * Returns the ancestors of this element that match the expression specified
     *
     * @param element               the element to retrieve the ancestors for
     * @param expression            the CSS expression to match
     * @return an array of this element's ancestors that match the expression specified
     */
    function up(element, expression) {
        element = (Object.isString(element))? get(element) : element;
        
        if(arguments.length == 1) {
            return element.parentNode;
        }

        var ancestors = getAncestors(element);
        return (Object.isNumber(expression))? ancestors[expression] :
            ancestors.find(function(node) {
                return Concierge.CSS.matches(expression, node);
            });
    }


    /**
     * Sets the inner HTML content of the element; all script tags are removed before the content is added
     *
     * @param element           the element to set the content for
     * @param html              the HTML content to set
     */
    function setContent(element, html) {
        element = (Object.isString(element))? get(element) : element;
        element.innerHTML = html;
        return this;
    }


    /**
     * Returns true if the element has the class name specified; false otherwise
     *
     * @param element           the element to query for the class
     * @param className         the class name to look for
     * @return true if the element has the class name specified; false otherwise
     */
    function hasClassName(element, className) {
        element = (Object.isString(element))? get(element) : element;
        return (new RegExp("(^|\\s)" + className + "(\\s|$)").test(element.className));
    }


    /**
     * Adds the specified CSS class name to this element's exisiting class names
     *
     * @param element           the element to add the class name to
     * @param className         the class name to add
     */
    function addClassName(element, className) {
        element = (Object.isString(element))? get(element) : element;
        
        if(!hasClassName(element, className)) {
            element.className += ((element.className)? " " : "") + className;
        }

        return this;
    }


    /**
     * Removes a CSS class name from this element's list of classes
     *
     * @param element           the element to remove the class name from
     * @param className         the CSS class name to remove
     */
    function removeClassName(element, className) {
        element = (Object.isString(element))? get(element) : element;
        var pattern = new RegExp("(^|\\s)" + className + "(\\s|$)");
        element.className = String.trim(element.className.replace(pattern, " "));
        return this;
    }


    /**
     * Toggles a class name on this element: adds it to the list of classes if the element doesn't have it, and
     * removes it otherwise
     *
     * @param element           the element to toggle the class name on
     * @param className         the class name to toggle on this element
     */
    function toggleClassName(element, className) {
        element = (Object.isString(element))? get(element) : element;
        if(hasClassName(element, className)) {
            removeClassName(element, className);
        } else {
            addClassName(element, className);
        }

        return this;
    }


    /**
     * Retrieves a style declaration on this element
     *
     * @param element           the element to retrieve the style declaration on
     * @param style             the CSS style to retrieve the declaration for (i.e width, color, background, etc)
     */
    function getStyle(element, style) {

        /** Uncomment to enable support for IE. Also uncomment the "getIEStyle" function.
        if(Concierge.Browser.IE) {
            return getIEStyle(element, style);
        }*/

        element = (Object.isString(element))? get(element) : element;
        style = (style == "float")? "cssFloat" : String.toCamelCase(style);
        var value = element.style[style];

        if (!value) {
            var css = document.defaultView.getComputedStyle(element, null);
            value = (css)? css[style] : null;
        }

        if (style == "opacity") {
            return (value)? parseFloat(value) : 1.0;
        }

        return (value == "auto")? null : value;
    }


    /**
     * Uncomment to enable support for IE
     * Retrieves a style declaration on this element. This implementation is specific to IE
     *
     * @param element           the element to retrieve the style declaration on
     * @param style             the CSS style to retrieve the declaration for (i.e width, color, background, etc)
     */
    function getIEStyle(element, style) {
        element = (Object.isString(element))? get(element) : element;
        style = (style == "float" || style == "cssFloat")? "styleFloat" : style.toCamelCase();

        var value = element.style[style];
        if (!value && element.currentStyle) {
            value = element.currentStyle[style];
        }

        if (style == "opacity") {
            if (value = (getIEStyle(element, "filter") || "").match(/alpha\(opacity=(.*)\)/)) {
                if (value[1]) {
                    return parseFloat(value[1]) / 100;
                }
            }
            return 1.0;
        }

        if (value == "auto") {
            if ((style == "width" || style == "height") && (getIEStyle(element, "display") != "none"))
            return element["offset" + style.capitalize()] + "px";
            return null;
        }

        return value;
    }


    /**
     * Sets a CSS style on this element. The style is the normal declaration string as it appears in the CSS file
     * (i.e. margin:0 10px;color: #069;border: 1px solid #ccc)
     *
     * @param element           the element to apply the CSS styles on
     * @param styles            the CSS styles to set on this element
     */
    function setStyle(element, styles) {
        element = (Object.isString(element))? get(element) : element;

        if(Object.isString(styles)) {
            element.style.cssText += ";" + styles;
        } else {
            for(var prop in styles) {
                element.style[prop] = styles[prop];
            }
        }

        return this;
    }


    /**
     * Sets the opacity for the element specified
     *
     * @param element           the element to set the opacity for
     * @param opacity           the opacity to set, between 0 and 1
     */
    function setOpacity(element, opacity) {
        element = normalize(element);

        if(Concierge.Browser.IE) {
            element.style.filter = (opacity == 1)? "" : "alpha(opacity='" + opacity * 100 + "')";
        }

        setStyle(element, {opacity : opacity});
        return this;
    }


    /**
     * Measures the dimensions (widht, height) of this element and returns it as a JSON object
     *
     * @param element           the element to get the dimensions for
     * @return the width and height of this element as a JSON object
     */
    function measure(element) {
        element = (Object.isString(element))? get(element) : element;
        var display = getStyle(element, "display");

        if (display != "none" && display != null) { // Safari bug
            return {
                width: element.offsetWidth, 
                height: element.offsetHeight
            };
        }

        /** All *Width and *Height properties give 0 on elements with display none,
         so enable the element temporarily */
        var styles = element.style,
        originalVisibility = styles.visibility,
        originalPosition = styles.position,
        originalDisplay = styles.display;

        styles.visibility = "hidden";
        styles.position = "absolute";
        styles.display = "block";

        var originalWidth = element.clientWidth,
        originalHeight = element.clientHeight;
        styles.display = originalDisplay;
        styles.position = originalPosition;
        styles.visibility = originalVisibility;

        return {
            width : originalWidth, 
            height: originalHeight
        };
    }


    /**
     * Returns the width of this element
     *
     * @param element           the element to retrieve the width for
     * @return the width of this element
     */
    function getWidth(element) {
        element = (Object.isString(element))? get(element) : element;
        return measure(element).width;
    }


    /**
     * Returns the height of this element
     *
     * @param element           the element to retrieve the height for
     * @return the height of this element
     */
    function getHeight(element) {
        element = (Object.isString(element))? get(element) : element;
        return measure(element).height;
    }


    /**
     * Clips the content of this element by giving it an overflow style of "hidden"
     *
     * @param element           the element whose content to clip
     * @return a reference to this element
     */
    function clipElement(element) {
        element = (Object.isString(element))? get(element) : element;
        
        if(element._overflow) {
            return element;
        }

        element._overflow = getStyle(element, "overflow") || "auto";
        
        if(element._overflow !== "hidden") {
            element.style.overflow = "hidden";
        }

        return element;
    }


    /**
     * Returns the cumulative top and left offsets of this element. The values include
     * the offsets of this element as well as those of its ancestors up to the body element
     *
     * @param element           the element to retrieve the cumulative offsets for
     * @return the cumulative offset of this element
     */
    function cumulativeOffset(element) {
        element = (Object.isString(element))? get(element) : element;
        var topOffset = 0, leftOffset = 0;

        do {
            topOffset += element.offsetTop || 0;
            leftOffset += element.offsetLeft || 0;
            element = element.offsetParent;
        } while (element);

        return {
            left : leftOffset, 
            top : topOffset
        };
    }


    /**
     * Returns the positioned top and left offsets of this element, which are the positions of this element relative to
     * its parent node
     *
     * @param element           the element to retrieve the positioned offsets for
     * @return the positioned offset of this element
     */
    function positionedOffset(element) {
        element = (Object.isString(element))? get(element) : element;

        var elementOffsets = cumulativeOffset(element),
        parentOffsets = cumulativeOffset(element.parentNode);
        
        return {
            left : elementOffsets.left - parentOffsets.left, 
            top : elementOffsets.top - parentOffsets.top
        };
    }
    
    
    /**
     * Enables hints on an input field: clears the content of the field when it gains focus and restores the default
     * text if no change was made to it
     *
     * @param element               the input field to activate
     */
    function enableHints(element) {
        element = (Object.isString(element))? get(element) : element;
        element._hint = element._default || element.defaultValue;
        
        var DOM = Concierge.DOM, 
        Event = Concierge.Event;

        if(element._hint == element.defaultValue) {
            DOMaddClassName(element, "hint");
        }

        Event.add(element, "focus", function(event){
            if(element._hint != element.value) {
                return;
            }
            
            DOM.removeClassName(element, "hint");
            element.value = "";
        });

        Event.add(element, "blur", function(event){
            if(String.trim(element.value) != "") {
                return;
            }
            
            DOM.addClassName(element, "hint");
            element.value = element._hint;
        });
    }


    /**
     * Scrolls the DOM element in the specified direction
     *
     * @param element           the DOM element to scroll
     * @param direction         the direction in which to scroll the element: top or bottom
     */
    function scroll(element, direction) {
        (direction == "top")? element.scrollTop = 0 : element.scrollTop = element.scrollHeight;
    }


    /**
     * Returns the viewport dimensions (width, height) of the currend document
     *
     * @return the viewport dimensions (width, height) of the currend document
     */
    function getDocumentViewport() {
        var viewportWidth, viewportHeight;

        /** For standards-compliant browsers (Firefox, Safari, Chrome, etc) */
        if (Object.isDefined(window.innerWidth)) {
            viewportWidth = window.innerWidth;
            viewportHeight = window.innerHeight;

        /** For IE in standards mode */
        } else if(Object.isDefined(document.documentElement) && Object.isDefined(document.documentElement.clientWidth)
            && document.documentElement.clientWidth != 0) {
            viewportWidth = document.documentElement.clientWidth;
            viewportHeight = document.documentElement.clientHeight;

        /** For older browsers */
        } else {
            viewportWidth = document.body.clientWidth;
            viewportHeight = document.body.clientHeight;
        }

        return {
            width : viewportWidth,
            height : viewportHeight
        };
    }


    /**
     * Returns the horizontal and vertical offsets of the viewport (scrollbar position)
     *
     * @return the horizontal and vertical offset of the viewport
     */
    function getViewportOffsets() {
        return {
            left : window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft,
            top : window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop
        };
    }


    /**
     * Parses a string and returns the resulting XML document. This method provides a cross-browser implementation
     *
     * @param text             the text to parse
     */
    function parseXML(text) {
        if(Object.isDefined(DOMParser)) {
            return (new DOMParser()).parseFromString(text, "application/xml");
        } else if(Object.isDefined(ActiveXObject)) {
            var doc = XML.newDocument();
            doc.loadXML(text);
            return doc;
        }
    }


    /**
     * Returns a debug-oriented string representation of this element (its ID and class name)
     *
     * @param element               the element to inspect
     * @return a debug-oriented string representation of an element
     */
    function inspectElement(element) {
        element = (Object.isString(element))? get(element) : element;
        
        var info = "<" + element.tagName.toLowerCase(),
        properties = {id : "id", className: "class"};

        Object.keys(properties).forEach(function(property) {
            var value = element[property] || "";
            if(value) {
                info += " " + properties[property] + "=\"" + value + "\"";
            }
        });

        return info + ">";
    }

    return {
        createElement : createElement,
        get : get,
        normalize : normalize,
        show : show,
        hide : hide,
        isVisible : isVisible,
        enable : enable,
        disable : disable,
        remove : remove,
        recursivelyCollect : recursivelyCollect,
        getAncestors : getAncestors,
        up : up,
        setContent : setContent,
        hasClassName : hasClassName,
        addClassName : addClassName,
        removeClassName : removeClassName,
        toggleClassName : toggleClassName,
        getStyle : getStyle,
        setStyle : setStyle,
        setOpacity : setOpacity,
        measure : measure,
        getWidth : getWidth,
        getHeight : getHeight,
        clipElement : clipElement,
        inspectElement : inspectElement,
        parseXML : parseXML,
        cumulativeOffset : cumulativeOffset,
        positionedOffset : positionedOffset,
        enableHints : enableHints,
        scroll : scroll,
        getDocumentViewport : getDocumentViewport,
        getViewportOffsets : getViewportOffsets
    };

})();


if(!window.$DOM) {
    window.DOM = Concierge.DOM;
}


/*----------------------------------------------------------------------------------------------------------------------
                                        DOM STORAGE

   This module defines a temporary (page-level) storage unit for DOM elements. It only persists for
the duration of the page. This module is primarily used to store the event handlers for an element but
it can also be used to store other custom properties for an element, thus preventing those properties
from being attached directly to the element
----------------------------------------------------------------------------------------------------------------------*/
Concierge.DOMStorage = {
    
    model : { },

    /**
     * Retrieves the storage entry for this element. The element's "_storageId" property is used as the
     * key for its entries
     *
     * @param element               the element to retrieve the storage for
     * @return the storage entry for this element
     */
    getStorage : function(element) {
    	var id = element._storageID || (element._storageID = "storage_" + Concierge.generateUID());
        return this.model[id] || (this.model[id] = { });
    },


    /**
     * Stores a key/value pair of content for this element. If the key parameter is an object, stores
     * each property of the object under its specific value
     *
     * @param element           the element to store the properties for
     * @param key               the property key
     * @param value             the property value
     */
    store : function(element, key, value) {
       if(arguments.length === 2) {
           Object.extend(this.getStorage(element), key);
       } else {
           this.getStorage(element)[key] = value;
       }

       return element;
    },


    /**
     * Retrieves the value of the key specified in this element's storage unit
     *
     * @param element           the element to retrieve the value for
     * @param key               the property key
     * @return the value for this key in this element's storage unit
     */
    retrieve : function(element, key) {
       return this.getStorage(element)[key];
    }
};



/*----------------------------------------------------------------------------------------------------------------------
                            CORE EVENT MODULE

   This module defines event-handler registration and deregistration functions in a cross-browser fashion. It handles 
normal mouse and keyboard events as well as custom events.

----------------------------------------------------------------------------------------------------------------------*/

Concierge.Event = (function() {
    "use strict";
    
	var events = {
		KEY_BACKSPACE : 8,
		KEY_TAB : 9,
		KEY_RETURN : 13,
		KEY_ESC : 27,
		KEY_LEFT : 37,
		KEY_UP : 38,
		KEY_RIGHT : 39,
		KEY_DOWN : 40,
		KEY_DELETE : 46,
		KEY_HOME : 36,
		KEY_END : 35,
		KEY_PAGEUP : 33,
		KEY_PAGEDOWN : 34,
		KEY_INSERT : 45
	}, eventCache = [ ];


    /**
     * Event#normalize(event)       --> Event
     * Normalizes the event object in a cross-browser fashion. For W3C-compliant browsers, it simply
     * returns the original event object. For IE, it wraps the event in a object that exposes the missing
     * properties and methods to make them more compatible
     *
     * @param event             the event to normalize
     * @param element           the target of the event
     * @return a normalized event object
     */
    function normalize(event, element) {

        if(!window.attachEvent) {
            return event;
        }

        var normalized = {
            _event : event,
            type : event.type,
            target : event.srcElement,
            currentTarget : element,
            relatedTarget : event.fromElement? event.fromElement : event.toElement,
            eventPhase : (event.srcElement == element)? 2 : 3,

            clientX : event.clientX,
            clientY : event.clientY,
            screenX : event.screenX,
            screenY : event.screenY,

            altKey : event.altKey,
            ctrlKey : event.ctrlKey,
            shiftKey : event.shiftKey,
            charCode : event.keyCode,
            memo : event.memo || { },

            stopPropagation : function() {
                this._event.cancelBubble = true;
            },
            
            preventDefault : function() {
                this._event.returnValue = false;
            }
        };

        /** Handle mouse position */
        if(event.clientX != null) {
            var doc = document.documentElement, body = document.body;
            normalized.pageX = event.clientX + ((doc && doc.scrollLeft) || (body && body.scrollLeft) || 0)
                - ((doc && doc.clientLeft) || (body && body.cientLeft) || 0);

            normalized.pageY = event.clientY + ((doc && doc.scrollTop) || (body && body.scrollTop) || 0)
                - ((doc && doc.clientTop) || (body && body.cientTop) || 0);
        }

        return normalized;
    }


    /**
     * Returns true if the current event is a custom one (the eventName is in the "noun:verb" format)
     *
     * @param eventName             the event name to check for
     * @return true if the name follows the custom event format; false otherwise
     */
    function isCustomEvent(eventName) {
        return String.contains(eventName, ":");
    }


    /**
     * Creates a handler for an event. The handler is stored in an array of event handlers for a specific
     * event type; all event types are also stored in a map for a given DOM element. This pattern allows for
     * a centralization of the handlers for an element, and frees it from any possibility of memory leaks
     *
     * @param element               the element to observe then event on
     * @param eventName             the name of the event (i.e. click, data:loaded)
     * @param callback              the callback to invoke when the event is fired
     */
    function createHandler(element, eventName, callback) {
        var storage = Concierge.DOMStorage, 
        model = storage.retrieve(element, "events");

        if(Object.isUndefined(model)) {
            model = { };
            storage.store(element, "events", model);
            eventCache.push(element);
        }

        var handlers = model[eventName];

        if(Object.isUndefined(handlers)) {
            handlers = [ ];
            model[eventName] = handlers;
        }

        /**
         * Prevents an event of the same type from being added twice with the same handler
         */
        var handlersForEvent = handlers.collectValues("callback");

        if(handlersForEvent.contains(callback)) {
            return false;
        }

        var handler;

        if(isCustomEvent(eventName)) {
            handler = function(event) {
                if(Object.isUndefined(event.eventName)) {
                    return false;
                }

                if(event.eventName != eventName) {
                    return false;
                }

                event = normalize(event, element);
                callback.call(element, event);
            };
        } else {
            handler = function(event) {
                event = normalize(event, element);
                callback.call(element, event);
            };
        }

        handler.callback = callback;
        handlers.push(handler);

        return handler;
    }


    /**
     * Event#element(event)	-->	Element
     * 
     * Returns the DOM element on which the event occured; the same element will be returned
     * whether the event is in capture mode or bubble mode. The implementation addresses
     * the use case where Firefox screws up the "click", "load" or "error" event on images.
     * It also fixes Safari's bug where an anchor's text node get passed as the target rather than
     * the anchor itself
     *
     * @param event             the event to retrieve the element for
     * @return the DOM element on which the event occured
     */
    function element(event) {
        var node = event.target, type = event.type, currentTarget = event.currentTarget;

        if(currentTarget && currentTarget.tagName) {
			if(type === "load" || type === "error"
				|| (type === "click" && currentTarget.tagName.toLowerCase() === "input" &&
					currentTarget.type === "radio")) {
				node = currentTarget;
			}
		}

		if(node.nodeType === Node.TEXT_NODE) {
			node = node.parentNode;
		}

		return node;
	 }


	 /**
	  * Event#pointer(event)	--> object
      * 
	  * Returns the absolute X and Y coordinates of the event relative to the document
      *
      * @param event            the event to retrieve the coordinates for
      * @return the absolute X and Y coordinates of the event
	  */
	 function pointer(event) {
		return {
			x : pointerX(event),
			y : pointerY(event)
		};
	 }


    /**
     * Event#pointerX(event)	--> Number
     * 
     * Returns the absolute horizontal coordinate for the event
     *
     * @param event              the event to retrieve the coordinate for
     * @return the absolute horizontal coordinate for the event
     */
    function pointerX(event) {
        var docElem = document.documentElement, body = document.body || {scrollLeft : 0};
        return (event.pageX) || (event.clientX + (docElem.scrollLeft || body.scrollLeft)
            - (document.clientLeft || 0));
    }


	/**
	  * Event#pointerY(event)	--> Number
      * 
	  * Returns the absolute vertical coordinate for the event
	  */
	 function pointerY(event) {
        var docElem = document.documentElement, body = document.body || {scrollTop : 0};
		return (event.pageY) || (event.clientY + (docElem.scrollTop || body.scrollTop)
			- (document.clientTop || 0));
	 }


	 /**
	  * Event#stop(event)       --> void
      * 
	  * Cancels the default action for this event and prevents it from bubbling
      *
      * @param event            the event to prevent the behavior for
	  */
	 function stop(event) {
        event.preventDefault();
        event.stopPropagation();
        event.stopped = true;
	 }


	 /**
	  * Event.inspect(event)	--> 	String
      * 
	  * Returns a string representation of the type of this event object
	  */
	 function inspect(event) {
		return "[object Event]";
	 }


     /**
      * Event#add(element, eventName, callback)     --> Element
      *
      * Registers either a normal event or a custom event on a DOM element. Custom events
      * follow the "noun:verb" naming convention. An event handler is created for each actual
      * event and stored in the handlers array of the specified type. This centralized model
      * allows for easy registration and de-registration of events
      *
      * @param element          the DOM element to observe the event on
      * @param eventName        the name of the event to observe (i.e. click, submit...)
      * @param callback         the callback method to invoke when the event is fired
      *
      * @return the element on which the event is being observed
      */
     function add(element, eventName, callback) {
         element = (Object.isString(element))? Concierge.DOM.get(element) : element;
         
         if(!element) {
             throw String.format(DOMException.INVALID_EVENT_ELEMENT, [eventName]);
         }

         var handler = createHandler(element, eventName, callback);

         if(!handler) {
             return element;
         }

         if(isCustomEvent(eventName)) {
             if(element.addEventListener) {
                 element.addEventListener("dataavailable", handler, false);
             } else {
                 element.attachEvent("ondataavailable", handler);
                 element.attachEvent("onfilterchange", handler);
             }
         } else {

             if(element.addEventListener) {
                 element.addEventListener(eventName, handler, false);
             } else {
                 element.attachEvent("on" + eventName, handler);
             }
         }

         return element;
     }


     /**
      * Event#remove(element, eventName, callback)     --> Element
      *
      * Removes either a normal event or a custom event from a DOM element. The handler for this
      * event is retrieved from the event model of the DOM element
      *
      * @param element          the DOM element to remove the event from
      * @param eventName        the name of the event to remove (i.e. click, submit...)
      * @param callback         the callback method associated with this event
      *
      * @return the element from which the event is being remove
      */
     function remove(element, eventName, callback) {
         var storage = Concierge.DOMStorage, model = storage.retrieve(element, "events");
         
         if(Object.isUndefined(model)) {
             return element;
         }

         if(!eventName) {
             Object.keys(model).forEach(function(evtName){
                 remove(element, evtName);
             });
             return element;
         }

         var handlers = model[eventName];

         if(Object.isUndefined(handlers)) {
             return element;
         }

         /**
          * If no callback is provided, remove all events of the type specified. The double-check logic is needed for 
          * IE, which still returns a null reference on some handlers
          */
         if(!callback) {
             handlers.forEach(function(handler){
                if(handler) {
                    remove(element, eventName, handler.callback);
                }
             });
             return element;
         }

         var handler = handlers.find(function(h){
             return h.callback == callback;
         });

         if(Object.isUndefined(handler)) {
             return element;
         }

         if(isCustomEvent(eventName)) {
             if(element.removeEventListener) {
                 element.removeEventListener("dataavailable", handler, false);
             } else {
                 element.detachEvent("ondataavailable", handler);
                 element.detachEvent("onfilterchange", handler);
             }
         } else {
             if(element.removeEventListener) {
                element.removeEventListener(eventName, handler, false);
             } else {
                 element.detachEvent("on" + eventName, handler);
             }

             var index = handlers.indexOf(handler);
             handlers.splice(index,1);
         }

         delete handler.callback;

         if(handlers.size() == 0) {
             delete model[eventName];
         }

         return element;
     }


     /**
      * Event#fire(element, eventName, data, bubble)    --> Event
      *
      * Fires a custom "dataavailable" event on a DOM element. The event object has
      * a "model" property that contains the specific data to send with the event. The
      * custom event is not queued; it's automatically fired to all the registered
      * handlers
      *
      *  @param element             the element to fire the synthetic event on
      *  @param eventName           the name of this event
      *  @param data                the data to send with this event
      *  @param bubble              specifies whether the event bubbles or not
      *  @return the custom event
      */
     function fire(element, eventName, data, bubble) {
         bubble = bubble || true;
         var event;

         if(element == document && document.createEvent && (!element.dispatchEvent)) {
             element = document.documentElement;
         }

         if(document.createEvent) {
             event = document.createEvent("HTMLEvents");
             event.initEvent("dataavailable", true, true);
         } else if(document.createEventObject) {
             event = document.createEventObject();
             event.eventType = (bubble)? "ondataavailable" : "onfilterchange";
         } else {
             return;
         }

         event.eventName = eventName;
         event.memo = data || { };

         if(document.createEvent) {
             element.dispatchEvent(event);
         } else {
             element.fireEvent(event.eventType, event);
         }

         return event;
     }


     /**
      * Clears the event cache by removing all the events on each element
      * stored in it
      */
     function destroyCache() {
        eventCache.forEach(function(element){
            Concierge.Event.remove(element);
        });

        return true;
     }

     /** If IE, clear the event cache on page unload */
     if(window.attachEvent) {
         window.attachEvent("onbeforeunload", destroyCache);
     }

    return {
        keys : events,
        normalize : normalize,
        element : element,
        pointer : pointer,
        pointerX : pointerX,
        pointerY : pointerY,
        add : add,
        remove : remove,
        fire : fire,
        stop : stop,
        isCustomEvent : isCustomEvent,
        inspect : inspect
    };

})();

Concierge.publish = Concierge.Event.fire.curry(document);
Concierge.subscribe = Concierge.WidgetBase.subscribe;

(function() {
    this.__ConciergePublish = function(name, model) {
        Concierge.publish(name, model);
    };
})();


/** Creates the different custom events used by the widgets */
Concierge.CoreEvents = {
    UI_INITIALIZED : "ui:initialized",
    
    DIALOG_CLOSE : "dialog:close",
    MODAL_ENABLE : "modal:enable",
    MODAL_DISABLE : "modal:disable",
    PANEL_DISPLAYED : "panel:displayed",
    PANEL_DISMISSED : "panel:dismissed",
    
    FILTER_SELECTED : "filter:selected"
};


/*
 * Support for the DOMContentLoaded event; this event is fired when the DOM structure is loaded and ready for
 * scripting. This implementation is based on work by Dan Webb, Matthias Miller,  Dean Edwards and John Resig.
 */
(function() {

    var timer;

    function fireContentLoadedEvent() {
        if (document.loaded) {
            return;
        }

        if (timer) {
            window.clearInterval(timer);
        }

        Concierge.Event.fire(document, "dom:loaded",  { });
        document.loaded = true;
    }

    if (document.addEventListener) {
        if (Concierge.Browser.WebKit) {
            timer = window.setInterval(function() {
                if (/loaded|complete/.test(document.readyState)) {
                    fireContentLoadedEvent();
                }
            }, 0);

            Concierge.Event.add(window, "load", fireContentLoadedEvent);

        } else {
            window.addEventListener("DOMContentLoaded", fireContentLoadedEvent, false);
        }
    } else {
        document.write("<script id=__onDOMContentLoaded defer src=//:><\/script>");
        document.getElementById("__onDOMContentLoaded").onreadystatechange = function() {
            if (this.readyState == "complete") {
                this.onreadystatechange = null;
                fireContentLoadedEvent();
            }
        };
    }

})();


/*----------------------------------------------------------------------------------------------------------------------
                            CSS ENGINE

  This module defines the CSS engine for performing fast CSS selections. It's based on the Sizzle selection engine
----------------------------------------------------------------------------------------------------------------------*/

Concierge.CSS = {

    select : function(expression, scope) {
        var elements = Sizzle(expression, scope);
        return elements;
	},

    selectFirst : function(expression, scope) {
		var elements = Sizzle(expression, scope);
        return (elements)? elements[0] : null;
	},

	matches : function(expression, element) {
		return Sizzle.matches(expression, [element]).length == 1;
	}
};

if(!window.$CSS) {
    window.$CSS = Concierge.CSS;
}



/*----------------------------------------------------------------------------------------------------------------------
                                INTERNATIONALIZATION

This module manages the internationalization and localization of the application strings. Language packs are added to 
the "i18n" modules by the respective supported languages
----------------------------------------------------------------------------------------------------------------------*/

Concierge.i18n = (function() {
    var supportedLocales = { }, currentLocale;


    /**
     * Adds a language pack to the message resource bundle
     * 
     * @param locale                the locale for which to set the language pack
     * @param pack                  the language pack to set    
     */
    function addLanguagePack (locale, pack) {
        setLocale(locale);
        supportedLocales[locale] = pack;
        return this;
    }


    /**
     * Sets the current locale
     * 
     * @param locale                    the locale to set
     */
     function setLocale(locale) {
        currentLocale = locale;
        return this;
    }


    /**
     * Returns the current locale code
     */
    function getLocale() {
        return (currentLocale || "en");
    }


    /**
     * Returns the localized text associated with this string, or just the pattern representation if no matching string 
     * is found
     * 
     * @param string                    the string for which to return the localized text
     * @return the localized text associated with this string
     */
    function getString(string) {
        var locale = getLocale();
        return (supportedLocales[locale][string] || ("#{" + string + "}"));
    }

    
    /**
     * Formats a resource bundle template with the specified replacement object
     * 
     * @param string                the resource bundle string to format; this will contain placeholder patterns
     * @param replacement           an object or array whose elements are used to populate the template string
     * @return a string formatted with the contents of the specified replacement object
     */
    function format(string, replacement) {
        var template = getString(string);
        return String.format(template, replacement);
    }


    /**
     * Formats a resource bundle template with the specified replacement object
     * 
     * @param string                the resource bundle string to format; this will contain placeholder patterns
     * @param replacement           an object or array whose elements are used to populate the template string
     * @return a string formatted with the contents of the specified replacement object
     */
    function formatString(string, replacement) {
        return String.format(string, replacement);
    }
    
    
    /**
     * Formats a date with the specified representations
     * 
     * @param date                  the date to format
     * @param dateFormat            the format in which to output the date, i.e. Fri, Aug 31
     * @param useRelativeDate       specifies whether to use relative dates such as "Today" or "Yesterday"
     * @return a representation of the given in the specified format
     */
    function formatDate(date, dateFormat, useRelativeDate) {
        if(!(date instanceof Date)) {
            return "#{invalidDate}";
        }
        
        return date;    /* Temporary implementation */
    }
    
    
    /**
     * Formats the time for the given date with the specified representations
     * 
     * @param date                  the date for which to format the time
     * @param timeFormat            the format in which to output the time, i.e. 10:30 AM
     * @return a representation of the time in the specified format
     */
    function formatTime(date, timeFormat) {
        if(!(date instanceof Date)) {
            return "#{invalidTime}";
        }
        
        return date;    /** Temporary implementation */
    }
    
    
    return {
        addLanguagePack : addLanguagePack,
        setLocale : setLocale,
        getLocale : getLocale,
        getString : getString,
        format : format,
        formatString : formatString,
        formatDate : formatDate,
        formatTime : formatTime
    };

})();



/*----------------------------------------------------------------------------------------------------------------------
                            EFFECTS MODULE

This module contains the effects that visually manipulate the styles of DOM elements over a period of time (duration).

The current implementation is a CSS-based transformation based on Emile, a stand-alone animation framework from 
Scriptaculous

emile.js (c) 2009 Thomas Fuchs
Licensed under the terms of the MIT license.
----------------------------------------------------------------------------------------------------------------------*/

Concierge.EffectBase = (function(){
	var parseEl = document.createElement("div"),
	CSS_PROPERTIES = ("backgroundColor borderBottomColor borderBottomWidth borderLeftColor borderLeftWidth " +
	"borderRightColor borderRightWidth borderSpacing borderTopColor borderTopWidth bottom color fontSize " +
	"fontWeight height left letterSpacing lineHeight marginBottom marginLeft marginRight marginTop maxHeight " +
	"maxWidth minHeight minWidth opacity outlineColor outlineOffset outlineWidth paddingBottom paddingLeft " +
	"paddingRight paddingTop right textIndent top width wordSpacing zIndex").split(" ");

	function interpolate(source, target, pos){
		return (source + (target - source) * pos).toFixed(3);
	}

	function s(str, p, c){
		return str.substr(p, c || 1);
	}

	function color(source, target, pos){
		var i = 2, j, c, tmp, v = [], r = [];
        
		while(j = 3, c = arguments[i-1], i--) {
		  if(s(c,0) == 'r') {
		  	c = c.match(/\d+/g);while(j--) v.push(~~c[j]);

		  } else {
			if(c.length==4) {
				c = "#" + s(c,1) + s(c,1) + s(c,2) + s(c,2) + s(c,3) + s(c,3);
			}
			while(j--) {
				v.push(parseInt(s(c, 1 + j*2, 2), 16));
			}
		  }
		}

		while(j--) {
			tmp = ~~(v[j+3]+(v[j]-v[j+3])*pos);
			r.push((tmp < 0)? 0 : (tmp > 255)? 255 : tmp );
		}

		return "rgb(" + r.join(",") + ")";
	}

	function parse(prop){
        var p = parseFloat(prop), q = prop.replace(/^[\-\d\.]+/,'');
		return isNaN(p)? {v: q, f: color, u: ''} : {v: p, f: interpolate, u: q};
	}

	function normalize(style){
		var css, rules = { }, i = CSS_PROPERTIES.length, v;
		parseEl.innerHTML = '<div style="'+style+'"></div>';
		css = parseEl.childNodes[0].style;
        
		while(i--) {
			if(v = css[CSS_PROPERTIES[i]]) {
				rules[CSS_PROPERTIES[i]] = parse(v);
			}
		}
        
		return rules;
	}


    var EffectBase = function(el, style, opts){
        el = (typeof el == "string")? document.getElementById(el) : el;
		opts = opts || { };
		var target = normalize(style), comp = (el.currentStyle)? el.currentStyle : getComputedStyle(el, null),
		prop, current = { }, start = +new Date, dur = opts.duration || 200, finish = start + dur, interval,
		easing = opts.transition || Concierge.Effect.Transitions.Sinusoidal;

		for(prop in target) {
			current[prop] = parse(comp[prop]);
		}

        if(opts.before && (typeof opts.before == "function")) {
            opts.before();
        }

		interval = setInterval(function(){
            var time = +new Date, pos = (time > finish)? 1 : (time - start) / dur;

            for(prop in target) {
                el.style[prop] = target[prop].f(current[prop].v,target[prop].v,easing(pos)) + target[prop].u;
            }

            if(time > finish) {
                clearInterval(interval);
                
                if(opts.after && typeof opts.after == "function") {
                    opts.after();
                }
            }
		}, 10);
	};

    return EffectBase;

})();


Concierge.Effect = (function(){

    /**
     * Checks whether the DOM element exists, then triggers its "hasLayout" property if it's not enabled
     * in IE
     *
     * @param element               the DOM element to normalize
     */
    function normalize(element) {
        element = (Object.isString(element))? Concierge.DOM.get(element) : element;

        if(Object.isUndefined(element)) {
            throw DOMException.NOT_FOUND_ERR;
        }

        if(Concierge.Browser.IE && (!element.currentStyle.hasLayout)) {
            Concierge.DOM.setStyle(element, "zoom:1");
        }

        return element;
    }


    /**
     * Applies a "Fade" effect to an element by transitioning it from its current state(fully opaque)
     * to its new state (fully transparent)
     *
     * @param element               the element to apply the effect to
     * @param options               the options that further fine-tune the effect
     */
    function fade(element, options) {
        element = normalize(element);
        
        var effectOptions = {
            duration : (options.duration && Object.isNumber(options.duration))? options.duration * 1000 : 250,
            transition : options.transition,
            before : function() {
              Concierge.DOM.setStyle(element, {
                  opacity : 1
              }) ;
              (options.before || Function.empty)();
            },
            after : function() {
                Concierge.DOM.hide(element);
                if(options.after && Object.isFunction(options.after)) {
                    options.after();
                }
            }
        };

        var style = (Concierge.Browser.IE)? "filter:alpha(opacity='0')" : "opacity:0";
        return Concierge.EffectBase(element, style, effectOptions);
    }


    /**
     * Reveals an element ("Appear" effect) by transitioning it from its current state(fully transparent)
     * to its new state (fully opaque). The "startOpacity" option defines the initial opacity of the
     * element from which to start the effect.
     *
     * IE browsers don't support the CSS "opacity" property, so this implementation uses the proprietary
     * equivalent alpha filter property for the effect
     *
     * @param element               the element to apply the effect to
     * @param options               the options that further fine-tune the effect
     */
    function appear(element, options) {
        element = normalize(element);
        var dom =Concierge.DOM, 
        isIE = Concierge.Browser.IE;
        dom.show(element).setOpacity(element, (options.startOpacity || 0));

        var effectOptions = {
            duration : options.duration,
            transition : options.transition,
            before : options.before || Function.empty,
            after : function() {
                if(isIE) {
                    dom.setOpacity(element, 1);
                }

                if(options.after && Object.isFunction(options.after)) {
                    options.after();
                }
            }
        };

        var style = (isIE)? "filter:alpha(opacity='100')" : "opacity:1";
        return Concierge.EffectBase(element, style, effectOptions);
    }


    /**
     * Moves an element from its current position to the new one
     *
     * @param element               the element to move
     * @param options               the options that further fine-tune the effect
     */
    function move(element, options) {
        var dom = Concierge.DOM, 
        position = dom.getStyle(element, "position");

        if(!(/relative|absolute/.test(position))) {
			dom.setStyle(element, "position:relative;");
		}

		["left", "top"].forEach(function(pos) {
			if(!(element.style[pos])) {
				element.style[pos] = "0px";
			}
		});

		var originalLeft = parseFloat(element.style.left) || 0,
		originalTop = parseFloat(element.style.top) || 0,
		posX = Math.round(originalLeft + parseFloat(options.left || 0)),
		posY = Math.round(originalTop + parseFloat(options.top || 0)),
        style = "left:" + posX + "px;top:" + posY + "px;";

        return Concierge.EffectBase(element, style, options);
    }


    /**
     * Scales an element (and its content) up or down based on the options provided
     *
     * @param element               the element to apply the style to
     * @param percent               the percent by which to scale the element
     * @param options               the options that further fine-tune the effect
     */
    function scale(element, percent, options) {
		percent = (/%$/.test(percent))? percent : parseFloat(percent) + "%";
        
        var dom = Concierge.DOM,
        effectOptions = Object.extend({
            scaleX : true,
            scaleY : true,
            scaleContent : true
        }, options || { });

        var ratio = parseFloat(percent) / 100, dimensions = dom.measure(element),
        width = dimensions.width, height = dimensions.height,
        fontSize = (dom.getStyle(element, "font-size")) || "1em", unit;

        ["px", "em", "%", "pt"].forEach(function(size){
           if(fontSize.indexOf(size) > 0) {
               unit = size;
               return;
           }
        });

        var css, newStyle = {
            width : Math.round(width * ratio) + "px",
            height : Math.round(height * ratio) + "px",
            fontSize : parseFloat(fontSize) * ratio + unit
        };
        
        if(effectOptions.scaleContent) {
            css = "width:" + newStyle.width + ";height:" + newStyle.height + ";font-size:" + newStyle.fontSize + ";";
        } else {
            css = "" + ((effectOptions.scaleX)? "width:" + newStyle.width + ";" : "") +
                ((effectOptions.scaleY)? "height:" + newStyle.height + ";" : "");
        }

        return Concierge.EffectBase(element, css, effectOptions);
    }

    
    /**
     * Applies a "blind-up" effect to an element based on the options provided
     *
     * @param element               the element to apply the effect to
     * @param options               the options that further fine-tune the effect
     */
    function blindUp(element, options) {
        element = normalize(element);

        var dom = Concierge.DOM,
        storedStyles = Concierge.DOMStorage.retrieve(element, "originalStyle"),
        dimensions = dom.measure(element);

        if(dimensions.height == 0) {
            return element;
        }

        if(!storedStyles) {
            Concierge.DOMStorage.store(element, "originalStyle", {
                width : dimensions.width,
                height: dimensions.height,
                overflow : dom.getStyle(element, "overflow") || "auto"
            });
        }

        var cssStyle = "height:0px;",
        effectOptions = {
            duration : options.duration,
            transition : options.transition,
            before : options.before || Function.empty,
            after : options.after || Function.empty
        };

        dom.clipElement(element);
        return Concierge.EffectBase(element, cssStyle, effectOptions);
    }


    /**
     * Applies a "blind-down" effect to an element based on the options provided
     *
     * @param element               the element to apply the effect to
     * @param options               the options that further fine-tune the effect
     */
    function blindDown(element, options) {
        element = normalize(element);

        var storedStyles = Concierge.DOMStorage.retrieve(element, "originalStyle"),
        cssStyle = String.format("height:{0}px;", [(storedStyles.height || 0)]);

        var effectOptions = {
            duration : options.duration,
            transition : options.transition,
            before : options.before || Function.empty,
            after : options.after || Function.empty
        };

        return Concierge.EffectBase(element, cssStyle, effectOptions);
    }

    return {
        fade : fade,
        appear : appear,
        move : move,
        scale : scale,
        blindUp : blindUp,
        blindDown : blindDown
    };

})();


/**
 * These transitions are functions that fine-tune how an effect evolves over time. They take a "position" argument, 
 * which is between 0 (start of the effect) and 1 (end of effect) and return a number that is the translation of 
 * the position at a given time
 */
Concierge.Effect.Transitions = {

    Sinusoidal : function(pos) {
      return (-Math.cos(pos * Math.PI) / 2) + 0.5;
    },

    Bounce : function(pos) {
        if (pos < (1/2.75)) {
            return (7.5625 * pos * pos);
        } else if (pos < (2/2.75)) {
            return (7.5625 * (pos -= (1.5/2.75)) * pos + .75);
        } else if (pos < (2.5/2.75)) {
            return (7.5625 * (pos -= (2.25/2.75)) * pos + .9375);
        } else {
            return (7.5625 * (pos -= (2.625/2.75)) * pos + .984375);
        }
    },

    EaseFrom : function(pos) {
        if ((pos /= 0.5) < 1) return 0.5 * Math.pow(pos,4);
        return -0.5 * ((pos -= 2) * Math.pow(pos,3) - 2);
    },

    EaseTo : function(pos) {
    	return Math.pow(pos,0.25);
    }
};



/*----------------------------------------------------------------------------------------------------------------------
                                   SIZZLE.JS

MIT License

Copyright (c) 2009 John Resig

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions
of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *--------------------------------------------------------------------------------------------------------------------*/

(function(){var k=/((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^\[\]]*\]|['"][^'"]*['"]|[^\[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g,done=0,toString=Object.prototype.toString,hasDuplicate=false,baseHasDuplicate=true;[0,0].sort(function(){baseHasDuplicate=false;return 0});var n=function(a,b,c,d){c=c||[];b=b||document;var e=b;if(b.nodeType!==1&&b.nodeType!==9){return[]}if(!a||typeof a!=="string"){return c}var f=[],m,set,checkSet,extra,prune=true,contextXML=n.isXML(b),soFar=a,ret,cur,pop,i;do{k.exec("");m=k.exec(soFar);if(m){soFar=m[3];f.push(m[1]);if(m[2]){extra=m[3];break}}}while(m);if(f.length>1&&p.exec(a)){if(f.length===2&&o.relative[f[0]]){set=t(f[0]+f[1],b)}else{set=o.relative[f[0]]?[b]:n(f.shift(),b);while(f.length){a=f.shift();if(o.relative[a]){a+=f.shift()}set=t(a,set)}}}else{if(!d&&f.length>1&&b.nodeType===9&&!contextXML&&o.match.ID.test(f[0])&&!o.match.ID.test(f[f.length-1])){ret=n.find(f.shift(),b,contextXML);b=ret.expr?n.filter(ret.expr,ret.set)[0]:ret.set[0]}if(b){ret=d?{expr:f.pop(),set:r(d)}:n.find(f.pop(),f.length===1&&(f[0]==="~"||f[0]==="+")&&b.parentNode?b.parentNode:b,contextXML);set=ret.expr?n.filter(ret.expr,ret.set):ret.set;if(f.length>0){checkSet=r(set)}else{prune=false}while(f.length){cur=f.pop();pop=cur;if(!o.relative[cur]){cur=""}else{pop=f.pop()}if(pop==null){pop=b}o.relative[cur](checkSet,pop,contextXML)}}else{checkSet=f=[]}}if(!checkSet){checkSet=set}if(!checkSet){n.error(cur||a)}if(toString.call(checkSet)==="[object Array]"){if(!prune){c.push.apply(c,checkSet)}else if(b&&b.nodeType===1){for(i=0;checkSet[i]!=null;i++){if(checkSet[i]&&(checkSet[i]===true||checkSet[i].nodeType===1&&n.contains(b,checkSet[i]))){c.push(set[i])}}}else{for(i=0;checkSet[i]!=null;i++){if(checkSet[i]&&checkSet[i].nodeType===1){c.push(set[i])}}}}else{r(checkSet,c)}if(extra){n(extra,e,c,d);n.uniqueSort(c)}return c};n.uniqueSort=function(a){if(s){hasDuplicate=baseHasDuplicate;a.sort(s);if(hasDuplicate){for(var i=1;i<a.length;i++){if(a[i]===a[i-1]){a.splice(i--,1)}}}}return a};n.matches=function(a,b){return n(a,null,null,b)};n.find=function(a,b,c){var d;if(!a){return[]}for(var i=0,l=o.order.length;i<l;i++){var e=o.order[i],match;if((match=o.leftMatch[e].exec(a))){var f=match[1];match.splice(1,1);if(f.substr(f.length-1)!=="\\"){match[1]=(match[1]||"").replace(/\\/g,"");d=o.find[e](match,b,c);if(d!=null){a=a.replace(o.match[e],"");break}}}}if(!d){d=b.getElementsByTagName("*")}return{set:d,expr:a}};n.filter=function(a,b,c,d){var e=a,result=[],curLoop=b,match,anyFound,isXMLFilter=b&&b[0]&&n.isXML(b[0]);while(a&&b.length){for(var f in o.filter){if((match=o.leftMatch[f].exec(a))!=null&&match[2]){var g=o.filter[f],found,item,left=match[1];anyFound=false;match.splice(1,1);if(left.substr(left.length-1)==="\\"){continue}if(curLoop===result){result=[]}if(o.preFilter[f]){match=o.preFilter[f](match,curLoop,c,result,d,isXMLFilter);if(!match){anyFound=found=true}else if(match===true){continue}}if(match){for(var i=0;(item=curLoop[i])!=null;i++){if(item){found=g(item,match,i,curLoop);var h=d^!!found;if(c&&found!=null){if(h){anyFound=true}else{curLoop[i]=false}}else if(h){result.push(item);anyFound=true}}}}if(found!==undefined){if(!c){curLoop=result}a=a.replace(o.match[f],"");if(!anyFound){return[]}break}}}if(a===e){if(anyFound==null){n.error(a)}else{break}}e=a}return curLoop};n.error=function(a){throw"Syntax error, unrecognized expression: "+a;};var o=n.selectors={order:["ID","NAME","TAG"],match:{ID:/#((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,CLASS:/\.((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,NAME:/\[name=['"]*((?:[\w\u00c0-\uFFFF\-]|\\.)+)['"]*\]/,ATTR:/\[\s*((?:[\w\u00c0-\uFFFF\-]|\\.)+)\s*(?:(\S?=)\s*(['"]*)(.*?)\3|)\s*\]/,TAG:/^((?:[\w\u00c0-\uFFFF\*\-]|\\.)+)/,CHILD:/:(only|nth|last|first)-child(?:\((even|odd|[\dn+\-]*)\))?/,POS:/:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^\-]|$)/,PSEUDO:/:((?:[\w\u00c0-\uFFFF\-]|\\.)+)(?:\((['"]?)((?:\([^\)]+\)|[^\(\)]*)+)\2\))?/},leftMatch:{},attrMap:{"class":"className","for":"htmlFor"},attrHandle:{href:function(a){return a.getAttribute("href")}},relative:{"+":function(a,b){var c=typeof b==="string",isTag=c&&!/\W/.test(b),isPartStrNotTag=c&&!isTag;if(isTag){b=b.toLowerCase()}for(var i=0,l=a.length,elem;i<l;i++){if((elem=a[i])){while((elem=elem.previousSibling)&&elem.nodeType!==1){}a[i]=isPartStrNotTag||elem&&elem.nodeName.toLowerCase()===b?elem||false:elem===b}}if(isPartStrNotTag){n.filter(b,a,true)}},">":function(a,b){var c=typeof b==="string",elem,i=0,l=a.length;if(c&&!/\W/.test(b)){b=b.toLowerCase();for(;i<l;i++){elem=a[i];if(elem){var d=elem.parentNode;a[i]=d.nodeName.toLowerCase()===b?d:false}}}else{for(;i<l;i++){elem=a[i];if(elem){a[i]=c?elem.parentNode:elem.parentNode===b}}if(c){n.filter(b,a,true)}}},"":function(a,b,c){var d=done++,checkFn=dirCheck,nodeCheck;if(typeof b==="string"&&!/\W/.test(b)){b=b.toLowerCase();nodeCheck=b;checkFn=dirNodeCheck}checkFn("parentNode",b,d,a,nodeCheck,c)},"~":function(a,b,c){var d=done++,checkFn=dirCheck,nodeCheck;if(typeof b==="string"&&!/\W/.test(b)){b=b.toLowerCase();nodeCheck=b;checkFn=dirNodeCheck}checkFn("previousSibling",b,d,a,nodeCheck,c)}},find:{ID:function(a,b,c){if(typeof b.getElementById!=="undefined"&&!c){var m=b.getElementById(a[1]);return m?[m]:[]}},NAME:function(a,b){if(typeof b.getElementsByName!=="undefined"){var c=[],results=b.getElementsByName(a[1]);for(var i=0,l=results.length;i<l;i++){if(results[i].getAttribute("name")===a[1]){c.push(results[i])}}return c.length===0?null:c}},TAG:function(a,b){return b.getElementsByTagName(a[1])}},preFilter:{CLASS:function(a,b,c,d,e,f){a=" "+a[1].replace(/\\/g,"")+" ";if(f){return a}for(var i=0,elem;(elem=b[i])!=null;i++){if(elem){if(e^(elem.className&&(" "+elem.className+" ").replace(/[\t\n]/g," ").indexOf(a)>=0)){if(!c){d.push(elem)}}else if(c){b[i]=false}}}return false},ID:function(a){return a[1].replace(/\\/g,"")},TAG:function(a,b){return a[1].toLowerCase()},CHILD:function(a){if(a[1]==="nth"){var b=/(-?)(\d*)n((?:\+|-)?\d*)/.exec(a[2]==="even"&&"2n"||a[2]==="odd"&&"2n+1"||!/\D/.test(a[2])&&"0n+"+a[2]||a[2]);a[2]=(b[1]+(b[2]||1))-0;a[3]=b[3]-0}a[0]=done++;return a},ATTR:function(a,b,c,d,e,f){var g=a[1].replace(/\\/g,"");if(!f&&o.attrMap[g]){a[1]=o.attrMap[g]}if(a[2]==="~="){a[4]=" "+a[4]+" "}return a},PSEUDO:function(a,b,c,d,e){if(a[1]==="not"){if((k.exec(a[3])||"").length>1||/^\w/.test(a[3])){a[3]=n(a[3],null,null,b)}else{var f=n.filter(a[3],b,c,true^e);if(!c){d.push.apply(d,f)}return false}}else if(o.match.POS.test(a[0])||o.match.CHILD.test(a[0])){return true}return a},POS:function(a){a.unshift(true);return a}},filters:{enabled:function(a){return a.disabled===false&&a.type!=="hidden"},disabled:function(a){return a.disabled===true},checked:function(a){return a.checked===true},selected:function(a){a.parentNode.selectedIndex;return a.selected===true},parent:function(a){return!!a.firstChild},empty:function(a){return!a.firstChild},has:function(a,i,b){return!!n(b[3],a).length},header:function(a){return(/h\d/i).test(a.nodeName)},text:function(a){return"text"===a.type},radio:function(a){return"radio"===a.type},checkbox:function(a){return"checkbox"===a.type},file:function(a){return"file"===a.type},password:function(a){return"password"===a.type},submit:function(a){return"submit"===a.type},image:function(a){return"image"===a.type},reset:function(a){return"reset"===a.type},button:function(a){return"button"===a.type||a.nodeName.toLowerCase()==="button"},input:function(a){return(/input|select|textarea|button/i).test(a.nodeName)}},setFilters:{first:function(a,i){return i===0},last:function(a,i,b,c){return i===c.length-1},even:function(a,i){return i%2===0},odd:function(a,i){return i%2===1},lt:function(a,i,b){return i<b[3]-0},gt:function(a,i,b){return i>b[3]-0},nth:function(a,i,b){return b[3]-0===i},eq:function(a,i,b){return b[3]-0===i}},filter:{PSEUDO:function(a,b,i,c){var d=b[1],filter=o.filters[d];if(filter){return filter(a,i,b,c)}else if(d==="contains"){return(a.textContent||a.innerText||n.getText([a])||"").indexOf(b[3])>=0}else if(d==="not"){var e=b[3];for(var j=0,l=e.length;j<l;j++){if(e[j]===a){return false}}return true}else{n.error("Syntax error, unrecognized expression: "+d)}},CHILD:function(a,b){var c=b[1],node=a;switch(c){case'only':case'first':while((node=node.previousSibling)){if(node.nodeType===1){return false}}if(c==="first"){return true}node=a;case'last':while((node=node.nextSibling)){if(node.nodeType===1){return false}}return true;case'nth':var d=b[2],last=b[3];if(d===1&&last===0){return true}var e=b[0],parent=a.parentNode;if(parent&&(parent.sizcache!==e||!a.nodeIndex)){var f=0;for(node=parent.firstChild;node;node=node.nextSibling){if(node.nodeType===1){node.nodeIndex=++f}}parent.sizcache=e}var g=a.nodeIndex-last;if(d===0){return g===0}else{return(g%d===0&&g/d>=0)}}},ID:function(a,b){return a.nodeType===1&&a.getAttribute("id")===b},TAG:function(a,b){return(b==="*"&&a.nodeType===1)||a.nodeName.toLowerCase()===b},CLASS:function(a,b){return(" "+(a.className||a.getAttribute("class"))+" ").indexOf(b)>-1},ATTR:function(a,b){var c=b[1],result=o.attrHandle[c]?o.attrHandle[c](a):a[c]!=null?a[c]:a.getAttribute(c),value=result+"",q=b[2],check=b[4];return result==null?q==="!=":q==="="?value===check:q==="*="?value.indexOf(check)>=0:q==="~="?(" "+value+" ").indexOf(check)>=0:!check?value&&result!==false:q==="!="?value!==check:q==="^="?value.indexOf(check)===0:q==="$="?value.substr(value.length-check.length)===check:q==="|="?value===check||value.substr(0,check.length+1)===check+"-":false},POS:function(a,b,i,c){var d=b[2],filter=o.setFilters[d];if(filter){return filter(a,i,b,c)}}}};var p=o.match.POS,fescape=function(a,b){return"\\"+(b-0+1)};for(var q in o.match){o.match[q]=new RegExp(o.match[q].source+(/(?![^\[]*\])(?![^\(]*\))/.source));o.leftMatch[q]=new RegExp(/(^(?:.|\r|\n)*?)/.source+o.match[q].source.replace(/\\(\d+)/g,fescape))}var r=function(a,b){a=Array.prototype.slice.call(a,0);if(b){b.push.apply(b,a);return b}return a};try{Array.prototype.slice.call(document.documentElement.childNodes,0)[0].nodeType}catch(e){r=function(a,b){var c=b||[],i=0;if(toString.call(a)==="[object Array]"){Array.prototype.push.apply(c,a)}else{if(typeof a.length==="number"){for(var l=a.length;i<l;i++){c.push(a[i])}}else{for(;a[i];i++){c.push(a[i])}}}return c}}var s;if(document.documentElement.compareDocumentPosition){s=function(a,b){if(!a.compareDocumentPosition||!b.compareDocumentPosition){if(a==b){hasDuplicate=true}return a.compareDocumentPosition?-1:1}var c=a.compareDocumentPosition(b)&4?-1:a===b?0:1;if(c===0){hasDuplicate=true}return c}}else if("sourceIndex"in document.documentElement){s=function(a,b){if(!a.sourceIndex||!b.sourceIndex){if(a==b){hasDuplicate=true}return a.sourceIndex?-1:1}var c=a.sourceIndex-b.sourceIndex;if(c===0){hasDuplicate=true}return c}}else if(document.createRange){s=function(a,b){if(!a.ownerDocument||!b.ownerDocument){if(a==b){hasDuplicate=true}return a.ownerDocument?-1:1}var c=a.ownerDocument.createRange(),bRange=b.ownerDocument.createRange();c.setStart(a,0);c.setEnd(a,0);bRange.setStart(b,0);bRange.setEnd(b,0);var d=c.compareBoundaryPoints(Range.START_TO_END,bRange);if(d===0){hasDuplicate=true}return d}}n.getText=function(a){var b="",elem;for(var i=0;a[i];i++){elem=a[i];if(elem.nodeType===3||elem.nodeType===4){b+=elem.nodeValue}else if(elem.nodeType!==8){b+=n.getText(elem.childNodes)}}return b};(function(){var d=document.createElement("div"),id="script"+(new Date()).getTime();d.innerHTML="<a name='"+id+"'/>";var e=document.documentElement;e.insertBefore(d,e.firstChild);if(document.getElementById(id)){o.find.ID=function(a,b,c){if(typeof b.getElementById!=="undefined"&&!c){var m=b.getElementById(a[1]);return m?m.id===a[1]||typeof m.getAttributeNode!=="undefined"&&m.getAttributeNode("id").nodeValue===a[1]?[m]:undefined:[]}};o.filter.ID=function(a,b){var c=typeof a.getAttributeNode!=="undefined"&&a.getAttributeNode("id");return a.nodeType===1&&c&&c.nodeValue===b}}e.removeChild(d);e=d=null})();(function(){var e=document.createElement("div");e.appendChild(document.createComment(""));if(e.getElementsByTagName("*").length>0){o.find.TAG=function(a,b){var c=b.getElementsByTagName(a[1]);if(a[1]==="*"){var d=[];for(var i=0;c[i];i++){if(c[i].nodeType===1){d.push(c[i])}}c=d}return c}}e.innerHTML="<a href='#'></a>";if(e.firstChild&&typeof e.firstChild.getAttribute!=="undefined"&&e.firstChild.getAttribute("href")!=="#"){o.attrHandle.href=function(a){return a.getAttribute("href",2)}}e=null})();if(document.querySelectorAll){(function(){var f=n,div=document.createElement("div");div.innerHTML="<p class='TEST'></p>";if(div.querySelectorAll&&div.querySelectorAll(".TEST").length===0){return}n=function(a,b,c,d){b=b||document;if(!d&&b.nodeType===9&&!n.isXML(b)){try{return r(b.querySelectorAll(a),c)}catch(e){}}return f(a,b,c,d)};for(var g in f){n[g]=f[g]}div=null})()}(function(){var d=document.createElement("div");d.innerHTML="<div class='test e'></div><div class='test'></div>";if(!d.getElementsByClassName||d.getElementsByClassName("e").length===0){return}d.lastChild.className="e";if(d.getElementsByClassName("e").length===1){return}o.order.splice(1,0,"CLASS");o.find.CLASS=function(a,b,c){if(typeof b.getElementsByClassName!=="undefined"&&!c){return b.getElementsByClassName(a[1])}};d=null})();function dirNodeCheck(a,b,c,d,e,f){for(var i=0,l=d.length;i<l;i++){var g=d[i];if(g){g=g[a];var h=false;while(g){if(g.sizcache===c){h=d[g.sizset];break}if(g.nodeType===1&&!f){g.sizcache=c;g.sizset=i}if(g.nodeName.toLowerCase()===b){h=g;break}g=g[a]}d[i]=h}}}function dirCheck(a,b,c,d,e,f){for(var i=0,l=d.length;i<l;i++){var g=d[i];if(g){g=g[a];var h=false;while(g){if(g.sizcache===c){h=d[g.sizset];break}if(g.nodeType===1){if(!f){g.sizcache=c;g.sizset=i}if(typeof b!=="string"){if(g===b){h=true;break}}else if(n.filter(b,[g]).length>0){h=g;break}}g=g[a]}d[i]=h}}}n.contains=document.compareDocumentPosition?function(a,b){return!!(a.compareDocumentPosition(b)&16)}:function(a,b){return a!==b&&(a.contains?a.contains(b):true)};n.isXML=function(a){var b=(a?a.ownerDocument||a:0).documentElement;return b?b.nodeName!=="HTML":false};var t=function(a,b){var c=[],later="",match,root=b.nodeType?[b]:b;while((match=o.match.PSEUDO.exec(a))){later+=match[0];a=a.replace(o.match.PSEUDO,"")}a=o.relative[a]?a+"*":a;for(var i=0,l=root.length;i<l;i++){n(a,root[i],c)}return n.filter(later,c)};window.Sizzle=n})();



/*----------------------------------------------------------------------------------------------------------------------
                    SWFObject v2.1 <http://code.google.com/p/swfobject/>

	Copyright (c) 2007-2008 Geoff Stearns, Michael Williams, and Bobby van der Sluis
	This software is released under the MIT License <http://www.opensource.org/licenses/mit-license.php>
----------------------------------------------------------------------------------------------------------------------*/
var swfobject=function(){var b="undefined",Q="object",n="Shockwave Flash",p="ShockwaveFlash.ShockwaveFlash",P="application/x-shockwave-flash",m="SWFObjectExprInst",j=window,K=document,T=navigator,o=[],N=[],i=[],d=[],J,Z=null,M=null,l=null,e=false,A=false;var h=function(){var v=typeof K.getElementById!=b&&typeof K.getElementsByTagName!=b&&typeof K.createElement!=b,AC=[0,0,0],x=null;if(typeof T.plugins!=b&&typeof T.plugins[n]==Q){x=T.plugins[n].description;if(x&&!(typeof T.mimeTypes!=b&&T.mimeTypes[P]&&!T.mimeTypes[P].enabledPlugin)){x=x.replace(/^.*\s+(\S+\s+\S+$)/,"$1");AC[0]=parseInt(x.replace(/^(.*)\..*$/,"$1"),10);AC[1]=parseInt(x.replace(/^.*\.(.*)\s.*$/,"$1"),10);AC[2]=/r/.test(x)?parseInt(x.replace(/^.*r(.*)$/,"$1"),10):0}}else{if(typeof j.ActiveXObject!=b){var y=null,AB=false;try{y=new ActiveXObject(p+".7")}catch(t){try{y=new ActiveXObject(p+".6");AC=[6,0,21];y.AllowScriptAccess="always"}catch(t){if(AC[0]==6){AB=true}}if(!AB){try{y=new ActiveXObject(p)}catch(t){}}}if(!AB&&y){try{x=y.GetVariable("$version");if(x){x=x.split(" ")[1].split(",");AC=[parseInt(x[0],10),parseInt(x[1],10),parseInt(x[2],10)]}}catch(t){}}}}var AD=T.userAgent.toLowerCase(),r=T.platform.toLowerCase(),AA=/webkit/.test(AD)?parseFloat(AD.replace(/^.*webkit\/(\d+(\.\d+)?).*$/,"$1")):false,q=false,z=r?/win/.test(r):/win/.test(AD),w=r?/mac/.test(r):/mac/.test(AD);/*@cc_on q=true;@if(@_win32)z=true;@elif(@_mac)w=true;@end@*/return{w3cdom:v,pv:AC,webkit:AA,ie:q,win:z,mac:w}}();var L=function(){if(!h.w3cdom){return}f(H);if(h.ie&&h.win){try{K.write("<script id=__ie_ondomload defer=true src=//:><\/script>");J=C("__ie_ondomload");if(J){I(J,"onreadystatechange",S)}}catch(q){}}if(h.webkit&&typeof K.readyState!=b){Z=setInterval(function(){if(/loaded|complete/.test(K.readyState)){E()}},10)}if(typeof K.addEventListener!=b){K.addEventListener("DOMContentLoaded",E,null)}R(E)}();function S(){if(J.readyState=="complete"){J.parentNode.removeChild(J);E()}}function E(){if(e){return}if(h.ie&&h.win){var v=a("span");try{var u=K.getElementsByTagName("body")[0].appendChild(v);u.parentNode.removeChild(u)}catch(w){return}}e=true;if(Z){clearInterval(Z);Z=null}var q=o.length;for(var r=0;r<q;r++){o[r]()}}function f(q){if(e){q()}else{o[o.length]=q}}function R(r){if(typeof j.addEventListener!=b){j.addEventListener("load",r,false)}else{if(typeof K.addEventListener!=b){K.addEventListener("load",r,false)}else{if(typeof j.attachEvent!=b){I(j,"onload",r)}else{if(typeof j.onload=="function"){var q=j.onload;j.onload=function(){q();r()}}else{j.onload=r}}}}}function H(){var t=N.length;for(var q=0;q<t;q++){var u=N[q].id;if(h.pv[0]>0){var r=C(u);if(r){N[q].width=r.getAttribute("width")?r.getAttribute("width"):"0";N[q].height=r.getAttribute("height")?r.getAttribute("height"):"0";if(c(N[q].swfVersion)){if(h.webkit&&h.webkit<312){Y(r)}W(u,true)}else{if(N[q].expressInstall&&!A&&c("6.0.65")&&(h.win||h.mac)){k(N[q])}else{O(r)}}}}else{W(u,true)}}}function Y(t){var q=t.getElementsByTagName(Q)[0];if(q){var w=a("embed"),y=q.attributes;if(y){var v=y.length;for(var u=0;u<v;u++){if(y[u].nodeName=="DATA"){w.setAttribute("src",y[u].nodeValue)}else{w.setAttribute(y[u].nodeName,y[u].nodeValue)}}}var x=q.childNodes;if(x){var z=x.length;for(var r=0;r<z;r++){if(x[r].nodeType==1&&x[r].nodeName=="PARAM"){w.setAttribute(x[r].getAttribute("name"),x[r].getAttribute("value"))}}}t.parentNode.replaceChild(w,t)}}function k(w){A=true;var u=C(w.id);if(u){if(w.altContentId){var y=C(w.altContentId);if(y){M=y;l=w.altContentId}}else{M=G(u)}if(!(/%$/.test(w.width))&&parseInt(w.width,10)<310){w.width="310"}if(!(/%$/.test(w.height))&&parseInt(w.height,10)<137){w.height="137"}K.title=K.title.slice(0,47)+" - Flash Player Installation";var z=h.ie&&h.win?"ActiveX":"PlugIn",q=K.title,r="MMredirectURL="+j.location+"&MMplayerType="+z+"&MMdoctitle="+q,x=w.id;if(h.ie&&h.win&&u.readyState!=4){var t=a("div");x+="SWFObjectNew";t.setAttribute("id",x);u.parentNode.insertBefore(t,u);u.style.display="none";var v=function(){u.parentNode.removeChild(u)};I(j,"onload",v)}U({data:w.expressInstall,id:m,width:w.width,height:w.height},{flashvars:r},x)}}function O(t){if(h.ie&&h.win&&t.readyState!=4){var r=a("div");t.parentNode.insertBefore(r,t);r.parentNode.replaceChild(G(t),r);t.style.display="none";var q=function(){t.parentNode.removeChild(t)};I(j,"onload",q)}else{t.parentNode.replaceChild(G(t),t)}}function G(v){var u=a("div");if(h.win&&h.ie){u.innerHTML=v.innerHTML}else{var r=v.getElementsByTagName(Q)[0];if(r){var w=r.childNodes;if(w){var q=w.length;for(var t=0;t<q;t++){if(!(w[t].nodeType==1&&w[t].nodeName=="PARAM")&&!(w[t].nodeType==8)){u.appendChild(w[t].cloneNode(true))}}}}}return u}function U(AG,AE,t){var q,v=C(t);if(v){if(typeof AG.id==b){AG.id=t}if(h.ie&&h.win){var AF="";for(var AB in AG){if(AG[AB]!=Object.prototype[AB]){if(AB.toLowerCase()=="data"){AE.movie=AG[AB]}else{if(AB.toLowerCase()=="styleclass"){AF+=' class="'+AG[AB]+'"'}else{if(AB.toLowerCase()!="classid"){AF+=" "+AB+'="'+AG[AB]+'"'}}}}}var AD="";for(var AA in AE){if(AE[AA]!=Object.prototype[AA]){AD+='<param name="'+AA+'" value="'+AE[AA]+'" />'}}v.outerHTML='<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"'+AF+">"+AD+"</object>";i[i.length]=AG.id;q=C(AG.id)}else{if(h.webkit&&h.webkit<312){var AC=a("embed");AC.setAttribute("type",P);for(var z in AG){if(AG[z]!=Object.prototype[z]){if(z.toLowerCase()=="data"){AC.setAttribute("src",AG[z])}else{if(z.toLowerCase()=="styleclass"){AC.setAttribute("class",AG[z])}else{if(z.toLowerCase()!="classid"){AC.setAttribute(z,AG[z])}}}}}for(var y in AE){if(AE[y]!=Object.prototype[y]){if(y.toLowerCase()!="movie"){AC.setAttribute(y,AE[y])}}}v.parentNode.replaceChild(AC,v);q=AC}else{var u=a(Q);u.setAttribute("type",P);for(var x in AG){if(AG[x]!=Object.prototype[x]){if(x.toLowerCase()=="styleclass"){u.setAttribute("class",AG[x])}else{if(x.toLowerCase()!="classid"){u.setAttribute(x,AG[x])}}}}for(var w in AE){if(AE[w]!=Object.prototype[w]&&w.toLowerCase()!="movie"){F(u,w,AE[w])}}v.parentNode.replaceChild(u,v);q=u}}}return q}function F(t,q,r){var u=a("param");u.setAttribute("name",q);u.setAttribute("value",r);t.appendChild(u)}function X(r){var q=C(r);if(q&&(q.nodeName=="OBJECT"||q.nodeName=="EMBED")){if(h.ie&&h.win){if(q.readyState==4){B(r)}else{j.attachEvent("onload",function(){B(r)})}}else{q.parentNode.removeChild(q)}}}function B(t){var r=C(t);if(r){for(var q in r){if(typeof r[q]=="function"){r[q]=null}}r.parentNode.removeChild(r)}}function C(t){var q=null;try{q=K.getElementById(t)}catch(r){}return q}function a(q){return K.createElement(q)}function I(t,q,r){t.attachEvent(q,r);d[d.length]=[t,q,r]}function c(t){var r=h.pv,q=t.split(".");q[0]=parseInt(q[0],10);q[1]=parseInt(q[1],10)||0;q[2]=parseInt(q[2],10)||0;return(r[0]>q[0]||(r[0]==q[0]&&r[1]>q[1])||(r[0]==q[0]&&r[1]==q[1]&&r[2]>=q[2]))?true:false}function V(v,r){if(h.ie&&h.mac){return}var u=K.getElementsByTagName("head")[0],t=a("style");t.setAttribute("type","text/css");t.setAttribute("media","screen");if(!(h.ie&&h.win)&&typeof K.createTextNode!=b){t.appendChild(K.createTextNode(v+" {"+r+"}"))}u.appendChild(t);if(h.ie&&h.win&&typeof K.styleSheets!=b&&K.styleSheets.length>0){var q=K.styleSheets[K.styleSheets.length-1];if(typeof q.addRule==Q){q.addRule(v,r)}}}function W(t,q){var r=q?"visible":"hidden";if(e&&C(t)){C(t).style.visibility=r}else{V("#"+t,"visibility:"+r)}}function g(s){var r=/[\\\"<>\.;]/;var q=r.exec(s)!=null;return q?encodeURIComponent(s):s}var D=function(){if(h.ie&&h.win){window.attachEvent("onunload",function(){var w=d.length;for(var v=0;v<w;v++){d[v][0].detachEvent(d[v][1],d[v][2])}var t=i.length;for(var u=0;u<t;u++){X(i[u])}for(var r in h){h[r]=null}h=null;for(var q in swfobject){swfobject[q]=null}swfobject=null})}}();return{registerObject:function(u,q,t){if(!h.w3cdom||!u||!q){return}var r={};r.id=u;r.swfVersion=q;r.expressInstall=t?t:false;N[N.length]=r;W(u,false)},getObjectById:function(v){var q=null;if(h.w3cdom){var t=C(v);if(t){var u=t.getElementsByTagName(Q)[0];if(!u||(u&&typeof t.SetVariable!=b)){q=t}else{if(typeof u.SetVariable!=b){q=u}}}}return q},embedSWF:function(x,AE,AB,AD,q,w,r,z,AC){if(!h.w3cdom||!x||!AE||!AB||!AD||!q){return}AB+="";AD+="";if(c(q)){W(AE,false);var AA={};if(AC&&typeof AC===Q){for(var v in AC){if(AC[v]!=Object.prototype[v]){AA[v]=AC[v]}}}AA.data=x;AA.width=AB;AA.height=AD;var y={};if(z&&typeof z===Q){for(var u in z){if(z[u]!=Object.prototype[u]){y[u]=z[u]}}}if(r&&typeof r===Q){for(var t in r){if(r[t]!=Object.prototype[t]){if(typeof y.flashvars!=b){y.flashvars+="&"+t+"="+r[t]}else{y.flashvars=t+"="+r[t]}}}}f(function(){U(AA,y,AE);if(AA.id==AE){W(AE,true)}})}else{if(w&&!A&&c("6.0.65")&&(h.win||h.mac)){A=true;W(AE,false);f(function(){var AF={};AF.id=AF.altContentId=AE;AF.width=AB;AF.height=AD;AF.expressInstall=w;k(AF)})}}},getFlashPlayerVersion:function(){return{major:h.pv[0],minor:h.pv[1],release:h.pv[2]}},hasFlashPlayerVersion:c,createSWF:function(t,r,q){if(h.w3cdom){return U(t,r,q)}else{return undefined}},removeSWF:function(q){if(h.w3cdom){X(q)}},createCSS:function(r,q){if(h.w3cdom){V(r,q)}},addDomLoadEvent:f,addLoadEvent:R,getQueryParamValue:function(v){var u=K.location.search||K.location.hash;if(v==null){return g(u)}if(u){var t=u.substring(1).split("&");for(var r=0;r<t.length;r++){if(t[r].substring(0,t[r].indexOf("="))==v){return g(t[r].substring((t[r].indexOf("=")+1)))}}}return""},expressInstallCallback:function(){if(A&&M){var q=C(m);if(q){q.parentNode.replaceChild(M,q);if(l){W(l,true);if(h.ie&&h.win){M.style.display="block"}}M=null;l=null;A=false}}}}}();

