/*--------------------------------------------------------------------------------------------------
						CORE.JS
						
	This is the main UI module. It defines an initialization method that executes then the page
	has loaded. It also provide additional methods that handle the events fired when the user
	interacts with the page elements, especially the image scroller.
	
	The main module is also used as the base object for the FX module
----------------------------------------------------------------------------------------------------*/

var UI = (function(){ 
	
	var viewport, scrollPanel, initOptions, 
	menuContainer, currentSelection;
	
	
	/**
	 * Initializes the UI, sets default options and attaches a click event on the
	 * image container node
	 *
	 * @param options		the user-provided options; these are merged with the default 
	 *						options to provide the final settings for the object
	 */
	 function init(options) {
		initOptions = Object.extend({
			viewportElement : "contentScroller",
			scrollPanel : "imageStrip",
			selector : "li",
			menuContainer : "contextMenu"
		}, options || { });
		
		viewport = $(initOptions.viewportElement);
		scrollPanel = $(initOptions.scrollPanel);
		menuContainer = $(initOptions.menuContainer);
		currentSelection = menuContainer.select(initOptions.selector).find(function(node) {
			return node.hasClassName("active-selection");
		});
		
		menuContainer.observe("click", this.onMenuSelection.bindAsEventListener(this), true);
	}
	
	
	/**
	 * Handles the callback for the click event delegated to the container node
	 *
	 * @param event			the click event observed on the list elements and delegated
	 *						to the parent unordered list container
	 */
	function onMenuSelection(event) {
		event.stop();
		var source = event.findElement("a");
		 
		if(!source) {
			return;
		}
		
		currentSelection.removeClassName("active-selection");
		currentSelection = source.up("li").addClassName("active-selection");
		
		this.scrollToSection(source);
	}
	
	
	/**
	 * Scrolls the panel to reveal the appropriate image view. The reference to the specific
	 * list item to scroll to is kept in the "rel" attribute of the anchor link that was
	 * clicked
	 *
	 * @param source			the anchor link that was clicked
	 */
	function scrollToSection(source) {
		var element = $(source.readAttribute("rel")),
		containerOffsets = viewport.cumulativeOffset(),
		elementOffsets = element.cumulativeOffset(),
		posX = containerOffsets[0] - elementOffsets[0],
		effectDuration = (Math.abs(parseInt(posX / 600)) > 1)? 0.7 : 0.5;
		
		UI.Fx.smoothScroll(scrollPanel, { left : posX }, {
			duration : effectDuration
		});
	}	
	
	
	/**
	 * These are public methods that are exposed by the module
	 */
	return {
		init : init,
		onMenuSelection : onMenuSelection,
		scrollToSection : scrollToSection
	};
				   
})();


UI.Fx = {
	/**
	 * The UI.Fx module defines a "smoothScroll" effect that smoothly scrolls an element from its
	 * current position to the one specified in the "position" argument object. The implementation
	 * computes the relative offset of the element, sets the default effect duration and transition
	 * and delegates the transformation to Scriptaculous' Effect.Morph efefct
	 *
	 * @param element			the DOM element to scroll
	 * @param position			the object containing the CSS declarations (left, top) of the 
	 *							new position to move the element to
	 * @param options			the user-provided effect options (duration, transition, etc)
	 */
	smoothScroll : function(element, position, options) {
		element = $(element);
    	var offsets = element.positionedOffset(),
		effectOptions = Object.extend({
			duration : 0.3,
			transition : UI.Fx.transitions.EASE_FROM
		}, options || { });
		
		return element.morph({
			left : (offsets.left + (position.left || 0)) + "px",
			top : (offsets.top + (position.top || 0)) + "px"
		}, effectOptions);
	}
};


/**
 * The transition object with three pre-defined effect formula. These are basic transitions
 * used for the purpose of this tutorial. A complete list of cooler effect transitions is
 * available on Scripty 2's site (scripty2.com)
 */
Object.extend(UI.Fx, {
	transitions : {
		EASE_FROM : function(pos) {
			return Math.pow(pos,4);
		},
		
		EASE_FROM_TO : function(pos) {
			if ((pos/=0.5) < 1) return 0.5*Math.pow(pos,4);
			return -0.5 * ((pos-=2)*Math.pow(pos,3) - 2);
		},
		
		EASE_TO : function(pos) {
			return Math.pow(pos,0.25);	
		}
	}
});

/** Put it in action... */
document.observe("dom:loaded", UI.init.bind(UI, { }));